#include "stdafx.h"
#include "Pgm11a.h"
#include "ControlDoc.h"
#include "ControlView.h"
#include "ControlFrame.h"
#include "GameDoc.h"
#include "PlayerView.h"
#include "OtherView.h"
#include "HeadData.h"
#include "HeadDoc.h"
#include "PlayerId.h"
#include "resource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(ControlDoc, CDocument)

ControlDoc::ControlDoc() {
 ClearControlValues ();
 controlfile[0] = 0;
}

ControlDoc::~ControlDoc() { }

BEGIN_MESSAGE_MAP(ControlDoc, CDocument)
 ON_COMMAND (CM_LOGON_GERMAN,  CmLogonGerman)
 ON_COMMAND (CM_LOGON_ITALY,   CmLogonItaly)
 ON_COMMAND (CM_LOGON_JAPAN,   CmLogonJapan)
 ON_COMMAND (CM_LOGON_BRITIAN, CmLogonBritian)
 ON_COMMAND (CM_LOGON_FRANCE,  CmLogonFrance)
 ON_COMMAND (CM_LOGON_USA,     CmLogonUSA)
 ON_COMMAND (CM_LOGON_RUSSIA,  CmLogonRussia)
 ON_COMMAND (CM_LOGON_MINOR,   CmLogonMinor)
 ON_UPDATE_COMMAND_UI (CM_LOGON_GERMAN,  CmEnableLogonGerman)
 ON_UPDATE_COMMAND_UI (CM_LOGON_ITALY,   CmEnableLogonItaly)
 ON_UPDATE_COMMAND_UI (CM_LOGON_JAPAN,   CmEnableLogonJapan)
 ON_UPDATE_COMMAND_UI (CM_LOGON_BRITIAN, CmEnableLogonBritian)
 ON_UPDATE_COMMAND_UI (CM_LOGON_FRANCE,  CmEnableLogonFrance)
 ON_UPDATE_COMMAND_UI (CM_LOGON_USA,     CmEnableLogonUSA)
 ON_UPDATE_COMMAND_UI (CM_LOGON_RUSSIA,  CmEnableLogonRussia)
 ON_UPDATE_COMMAND_UI (CM_LOGON_MINOR,   CmEnableLogonMinor)
END_MESSAGE_MAP()

/***************************************************************************/
/*                                                                         */
/* ClearControlValues: initialize beginning state                          */
/*                                                                         */
/***************************************************************************/

void      ControlDoc::ClearControlValues () {
 for (long i=0; i<MAX_PLAYERS; i++) {
  players[i].isrequired    = false;
  players[i].isloggedon    = false;
  players[i].ptrplayerview = NULL;
  players[i].ptrgamedoc  = NULL;
  current_action[i].done = 0;
  current_action[i].type = 0;
 }
 ptrheaddoc  = NULL;
 head_index = -1;
 all_logged_on = false;
 descr[0] = 0;
 game.num_players_required = 0;
 game.bkgrndfile[0] = 0;
 game.active_phase = 0;
 game.first_player = -1;
 game.current_player = -1;
 game.current_turn = 0;
}

/***************************************************************************/
/*                                                                         */
/* SetHeaderAddr: provides access to header doc, id, and our battle title  */
/*                                                                         */
/***************************************************************************/

void      ControlDoc::SetHeaderAddr (HeadDoc *ptrhdoc, long id, char *desc) {
 ASSERT (ptrhdoc->IsKindOf (RUNTIME_CLASS (HeadDoc)));
 if (ptrhdoc && ptrhdoc->IsKindOf (RUNTIME_CLASS (HeadDoc))) {
  ptrheaddoc = ptrhdoc;
  head_index = id;
  strcpy_s (descr, sizeof(descr), desc);
  POSITION pos = GetFirstViewPosition ();
  if (pos != NULL) {
   CView *ptrview = GetNextView (pos);
   if (ptrview && ptrview->IsKindOf (RUNTIME_CLASS (ControlView))) {
    ControlView *ptrcview = (ControlView*) ptrview;
    CString msg;
    msg.LoadString (IDS_CONTROLLER_NAME);
    msg += descr;
    ptrcview->GetParentFrame ()->SetWindowText (msg);
   }
  }
 }  
}

/***************************************************************************/
/*                                                                         */
/* GetControlView: returns ptr to game control view for posting messages   */
/*                                                                         */
/***************************************************************************/

ControlView*   ControlDoc::GetControlView () {
 POSITION pos = GetFirstViewPosition ();
 CView *ptrview;
 while (pos != NULL) {
  ptrview = GetNextView (pos);
  if (ptrview && ptrview->IsKindOf (RUNTIME_CLASS (ControlView)))
   return (ControlView*) ptrview;
 }
 return NULL;
}

/***************************************************************************/
/*                                                                         */
/* ShowPlayer: hides or shows a player's views                             */
/*                                                                         */
/***************************************************************************/

void      ControlDoc::ShowPlayer (long id, bool show) {
 if (id < 0 || id >= MAX_PLAYERS) return;
 if (players[id].isloggedon && players[id].ptrgamedoc && 
     players[id].ptrgamedoc->IsKindOf (RUNTIME_CLASS (GameDoc))) 
  players[id].ptrgamedoc->ShowPlayer (show);
}

/***************************************************************************/
/*                                                                         */
/* ActionDone: active player done, so let other player do something        */
/*                                                                         */
/***************************************************************************/

void      ControlDoc::ActionDone (long id) {
 current_action[id].done = 1;
 long i = NextPlayerIs ();
 if (i == -1) {
  if (all_logged_on) {
   MessageBox (0, "Error - cannot find next player", "Game is confused", MB_OK);
   return;
  }
  else {
   MessageBox (0, "Your request has been accepted. Please Wait.", "Game is suspended until all players are logged on", MB_OK);
   return;
  }
 }
 ShowPlayer (game.current_player, false);
 game.current_player = i;
 ShowPlayer (game.current_player, true);
 players[i].ptrplayerview->GetParentFrame ()->ActivateFrame ();
 UpdateAllViews (NULL, 1, NULL);
 return;
}

/***************************************************************************/
/*                                                                         */
/* NextPlayerIs: returns next player to have a turn                        */
/*                                                                         */
/***************************************************************************/

long      ControlDoc::NextPlayerIs () {
 long i;
 for (i=game.current_player + 1; i<MAX_PLAYERS; i++) {
  if (players[i].isrequired && players[i].isloggedon) return i;
 }
 for (i=0; i<game.current_player; i++) {
  if (players[i].isrequired && players[i].isloggedon) return i;
 }
 return -1;
}

/***************************************************************************/
/*                                                                         */
/* AllLoggedOn: returns TRUE when all players required are logged on       */
/*                                                                         */
/***************************************************************************/

bool      ControlDoc::AllLoggedOn () {
 for (long i=0; i<MAX_PLAYERS; i++)
  if (players[i].isrequired && !players[i].isloggedon) return false;
 return true;
}

/***************************************************************************/
/*                                                                         */
/* LogonPlayer: chains new player into controls and gives its doc access   */
/*                                                                         */
/***************************************************************************/

void      ControlDoc::LogonPlayer (long id) {
 CDocument *ptrcdoc = theApp.ptrplayertmp->OpenDocumentFile (game.bkgrndfile);
 if (ptrcdoc) {
  if (ptrcdoc && ptrcdoc->IsKindOf (RUNTIME_CLASS (GameDoc))) {
   GameDoc *ptrdoc = (GameDoc*) ptrcdoc;
   CView *ptrcview;
   POSITION pos = ptrdoc->GetFirstViewPosition ();
   if (pos != NULL) {
    ptrcview = ptrdoc->GetNextView (pos);
    if (ptrcview && ptrcview->IsKindOf (RUNTIME_CLASS (PlayerView))) {
     PlayerView *ptrview = (PlayerView*) ptrcview;
     players[id].ptrplayerview = ptrview;
     players[id].ptrgamedoc  = ptrdoc;
     players[id].isloggedon    = true;
     ptrdoc->PostControlAddrs (this, id, descr, GetControlView ());
     if ((all_logged_on = AllLoggedOn ()) == true) {
      game.current_player = game.first_player;
      for (long i=0; i<MAX_PLAYERS; i++) {  // start game
       if (players[i].isrequired) current_action[i].done = 0;
       else current_action[i].done = 1;
       if (i != game.current_player && players[i].isloggedon) ShowPlayer (i, false);
      }
      GetControlView ()->GetParentFrame ()->ShowWindow (SW_MINIMIZE);
      GetControlView ()->Invalidate ();
      return;
     }
     return;
    }
   }
  }
 }
 CString msg1, msg2;
 msg1.LoadString (IDS_BATTLCTRL_ERRH);
 msg2.LoadString (IDS_BATTLCTRL_ERR1);
 MessageBox (0, msg2, msg1, MB_OK);
}

/***************************************************************************/
/*                                                                         */
/* CmLogonXXXX: 8 logons for the possible players                          */
/*                                                                         */
/***************************************************************************/

void      ControlDoc::CmLogonGerman () {
 if (!players[GermanPlayer].isloggedon) LogonPlayer (GermanPlayer);
}

void      ControlDoc::CmLogonItaly () {
 if (!players[ItalianPlayer].isloggedon) LogonPlayer (ItalianPlayer);
}

void      ControlDoc::CmLogonJapan () {
 if (!players[JapanesePlayer].isloggedon) LogonPlayer (JapanesePlayer);
}

void      ControlDoc::CmLogonBritian () {
 if (!players[BritishPlayer].isloggedon) LogonPlayer (BritishPlayer);
}

void      ControlDoc::CmLogonFrance () {
 if (!players[FrenchPlayer].isloggedon) LogonPlayer (FrenchPlayer);
}

void      ControlDoc::CmLogonUSA () {
 if (!players[USAPlayer].isloggedon) LogonPlayer (USAPlayer);
}

void      ControlDoc::CmLogonRussia () {
 if (!players[RussianPlayer].isloggedon) LogonPlayer (RussianPlayer);
}

void      ControlDoc::CmLogonMinor () {
 if (!players[MinorsPlayer].isloggedon) LogonPlayer (MinorsPlayer);
}

/***************************************************************************/
/*                                                                         */
/* CmEnableLogonXXXX: command enablers for the possible 8 players          */
/*                                                                         */
/***************************************************************************/
 
void      ControlDoc::CmEnableLogonGerman (CCmdUI *ptrui) {
 ptrui->Enable (players[GermanPlayer].isrequired ? 
                players[GermanPlayer].isloggedon ? 0 : 1 : 0);
}
 
void      ControlDoc::CmEnableLogonItaly (CCmdUI *ptrui) {
 ptrui->Enable (players[ItalianPlayer].isrequired ?
                players[ItalianPlayer].isloggedon ?  0 : 1 : 0);
}
 
void      ControlDoc::CmEnableLogonJapan (CCmdUI *ptrui) {
 ptrui->Enable (players[JapanesePlayer].isrequired ?
                players[JapanesePlayer].isloggedon ?  0 : 1 : 0);
}
 
void      ControlDoc::CmEnableLogonBritian (CCmdUI *ptrui) {
 ptrui->Enable (players[BritishPlayer].isrequired ?
                players[BritishPlayer].isloggedon ?  0 : 1 : 0);
}
 
void      ControlDoc::CmEnableLogonFrance (CCmdUI *ptrui) {
 ptrui->Enable (players[FrenchPlayer].isrequired ?
                players[FrenchPlayer].isloggedon ?  0 : 1 : 0);
}
 
void      ControlDoc::CmEnableLogonUSA (CCmdUI *ptrui) {
 ptrui->Enable (players[USAPlayer].isrequired ?
                players[USAPlayer].isloggedon ?  0 : 1 : 0);
}
 
void      ControlDoc::CmEnableLogonRussia (CCmdUI *ptrui) {
 ptrui->Enable (players[RussianPlayer].isrequired ?
                players[RussianPlayer].isloggedon ?  0 : 1 : 0);
}
 
void      ControlDoc::CmEnableLogonMinor (CCmdUI *ptrui) {
 ptrui->Enable (players[MinorsPlayer].isrequired ?
                players[MinorsPlayer].isloggedon ?  0 : 1 : 0);
}

/***************************************************************************/
/*                                                                         */
/* LogoffPlayer: remove player for active status and suspend game          */
/*                                                                         */
/***************************************************************************/

void      ControlDoc::LogoffPlayer (long id) {
 if (id < 0 || id >= MAX_PLAYERS) return;

 players[id].isloggedon = false;
 players[id].ptrplayerview = NULL;
 players[id].ptrgamedoc = NULL;

 if (all_logged_on) all_logged_on = false;
 GetControlView ()->GetParentFrame ()->ShowWindow (SW_RESTORE);
 GetControlView ()->Invalidate ();
 if (game.current_player == id) {
  game.current_player = -1;
  id = NextPlayerIs ();
  if (id != -1) {
   ShowPlayer (game.current_player, true);
   players[id].ptrplayerview->GetParentFrame ()->ActivateFrame ();
  }
 }
 UpdateAllViews (NULL, 1, NULL);
}

/***************************************************************************/
/*                                                                         */
/* LogoffAll: log all players off the battle                               */
/*                                                                         */
/***************************************************************************/

void      ControlDoc::LogoffAll () {
 for (long id=0; id<MAX_PLAYERS; id++) {
 if (players[id].isloggedon) players[id].ptrgamedoc->OnCloseDocument ();
  players[id].isloggedon = false;
  players[id].ptrplayerview = NULL;
  players[id].ptrgamedoc = NULL;
 }
 if (IsModified ()) {
  SaveBattle (controlfile);
  SetModifiedFlag (false);
 }
 all_logged_on = false;
 if (ptrheaddoc && ptrheaddoc->IsKindOf (RUNTIME_CLASS (HeadDoc)))
  ptrheaddoc->LogoffBattle (head_index);
 head_index = -1;
 ptrheaddoc = NULL;
 // halt game 
}


/***************************************************************************/
/*                                                                         */
/* ServerClosesAdmin: the server requests closure of the game - log all off*/
/*                                                                         */
/***************************************************************************/

void      ControlDoc::ServerClosesAdmin () {
 CString msg1, msg2;
 msg1.LoadString (IDS_BATTLCTRL_SRVH);
 msg2.LoadString (IDS_BATTLCTRL_SRV1);
 MessageBox (0, msg2, msg1, MB_OK);
 OnCloseDocument ();
}

/***************************************************************************/
/*                                                                         */
/* OnFileClose: close the document which closes down all players           */
/*                                                                         */
/***************************************************************************/

void      ControlDoc::OnFileClose () {
 OnCloseDocument ();
}

/***************************************************************************/
/*                                                                         */
/* OnCloseDocument: fast close of the document and all players beneath     */
/*                                                                         */
/***************************************************************************/

void      ControlDoc::OnCloseDocument () {
 LogoffAll ();
 CDocument::OnCloseDocument ();
}

/***************************************************************************/
/*                                                                         */
/* CanCloseFrame: prompt user about shutting whole game down               */
/*                                                                         */
/***************************************************************************/

BOOL      ControlDoc::CanCloseFrame (CFrameWnd *ptrwin) {
 if (!ptrwin) return TRUE;
 if (!ptrwin->IsKindOf (RUNTIME_CLASS (ControlFrame))) return FALSE;
 CView *ptrview = ((ControlFrame*) ptrwin)->GetActiveView ();
 if (ptrview->IsKindOf (RUNTIME_CLASS (ControlView))) {
  CString msg1, msg2;
  msg1.LoadString (IDS_BATTLCTRL_CLOSE);
  msg2.LoadString (IDS_BATTLCTRL_QUERY);
  if (MessageBox (0, msg2, msg1, MB_YESNO | MB_ICONQUESTION) == IDYES) {
   LogoffAll ();
   return TRUE;
  }
  return FALSE;
 }
 return FALSE;
} 


/***************************************************************************/
/*                                                                         */
/* RemoveObjects: remove all allocated objects                             */
/*                                                                         */
/***************************************************************************/

void       ControlDoc::RemoveObjects () {
}

/***************************************************************************/
/*                                                                         */
/* OnOpenDocument: open an existing battle                                 */
/*                                                                         */
/***************************************************************************/

BOOL      ControlDoc::OnOpenDocument (LPCTSTR filename) {
 ClearControlValues ();
 RemoveObjects (); 
 if (!CDocument::OnOpenDocument (filename)) return FALSE;
 return LoadBattle (filename);
}

/***************************************************************************/
/*                                                                         */
/* OnSaveDocument: save this battle                                        */
/*                                                                         */
/***************************************************************************/

BOOL      ControlDoc::OnSaveDocument (LPCTSTR filename) {
 return SaveBattle (filename); 
}

/***************************************************************************/
/*                                                                         */
/* LoadBattle: loads in a battle to play                                   */
/*                                                                         */
/***************************************************************************/

bool      ControlDoc::LoadBattle (LPCTSTR filename) {
 strcpy_s (controlfile, sizeof(controlfile), filename);
 long i, j, k;
 char buf[MAX_PATH];            // the input buffer
 long count;               // 0 at EOF
 CFile file;              // the file, will be binary I/O still
 CFileException file_err; // used if more info on errors is desired

 i = 0;
 file.Open (filename, CFile::modeRead, &file_err);
 while (true) { // read all file bytes, checking for new line codes
  count = file.Read (&buf[i], 1);   // reads one byte
  if (buf[i] == '\n') {
   buf[i]=0;                         // insert NULL terminator
   break;
  }
  else if (count==0) {
   buf[i] = 0;
   break;           // also end when no more bytes
  }
  else i++;                           // get next byte in the line of data
 }
 sscanf_s (buf, "%ld", &k); // convert into proper data types
 if (k < 2) {
  MessageBox (0, "Must have at least two players for a battle", "error", MB_OK);
  return false;
 }
 game.num_players_required = k;
 for (k=0; k<game.num_players_required; k++) {
  i = 0;
  while (true) { // read all file bytes, checking for new line codes
   count = file.Read (&buf[i], 1);   // reads one byte
   if (buf[i] == '\n') {
    buf[i]=0;                         // insert NULL terminator
    break;
   }
   else if (count==0) {
    buf[i] = 0;
    break;           // also end when no more bytes
   }
   else i++;                           // get next byte in the line of data
  }
  sscanf_s (buf, "%ld", &j);
  if (j < 0 || j > 7) {
   MessageBox (0, "bad data in battle file","error", MB_OK);
   return false;
  }
  players[j].isrequired = true;
  if (game.first_player == -1) game.first_player = j;
 }
 i = 0;
 while (true) { // read all file bytes, checking for new line codes
  count = file.Read (&buf[i], 1);   // reads one byte
  if (buf[i] == '\n') {
   buf[i]=0;                         // insert NULL terminator
   break;
  }
  else if (count==0) {
   buf[i] = 0;
   break;           // also end when no more bytes
  }
  else i++;                           // get next byte in the line of data
 }
 if (i>0 && buf[i-1] == '\r') buf[i-1]=0;
 strcpy_s (game.bkgrndbasefile, sizeof(game.bkgrndbasefile), buf);
 strcpy_s (game.bkgrndfile, sizeof(game.bkgrndfile), theApp.mainpath);
 strcat_s (game.bkgrndfile, sizeof(game.bkgrndfile), buf);
 file.Close ();
 return true;
}

/***************************************************************************/
/*                                                                         */
/* SaveBattle: saves a battle to disk                                      */
/*                                                                         */
/***************************************************************************/

bool      ControlDoc::SaveBattle (LPCTSTR filename) {
 MessageBox (0, "Saving control battle file to be implemented", filename, MB_OK);
 return true;
}

/***************************************************************************/
/*                                                                         */
/* DeleteContents: remove this document's contents                         */
/*                                                                         */
/***************************************************************************/

void      ControlDoc::DeleteContents () {                
 RemoveObjects ();
}

#ifdef _DEBUG
void ControlDoc::AssertValid() const { CDocument::AssertValid(); }
void ControlDoc::Dump(CDumpContext& dc) const { CDocument::Dump(dc); }
#endif //_DEBUG
