#pragma once
class PlayerView;
class OtherView;
class GameDoc;
class HeadDoc;
class ControlView;

const int MAX_PLAYERS = 8;

struct Players {
 bool        isrequired;     // true when this player has forces
 bool        isloggedon;     // true when player is logged on
 PlayerView *ptrplayerview;  // the player's view
 GameDoc    *ptrgamedoc;     // the player's game doc
};

struct Game {
 long  active_phase;
 long  num_players_required;
 long  first_player;
 long  current_player;
 long  current_turn;
 char  bkgrndfile[MAX_PATH];
 char  bkgrndbasefile[MAX_PATH];
};

struct CurrentAction {
 BYTE done;
 BYTE type;
};

/***************************************************************************/
/*                                                                         */
/* ControlDoc: Container for one complete battle                           */
/*                                                                         */
/***************************************************************************/

class ControlDoc : public CDocument {
public:
          ControlDoc();
 virtual ~ControlDoc();
	DECLARE_DYNCREATE(ControlDoc)

 /***************************************************************************/
 /*                                                                         */
 /* Data Members                                                            */
 /*                                                                         */
 /***************************************************************************/
 
public:

HeadDoc  *ptrheaddoc;             // our head document
long      head_index;             // head index

char      descr[MAX_PATH];        // our battle description
char      controlfile[MAX_PATH];  // our filename to save into
bool      all_logged_on;          // true when all required are on

Players   players[MAX_PLAYERS];   // array of players to control
Game      game;

CurrentAction  current_action[MAX_PLAYERS];

virtual BOOL OnOpenDocument (LPCTSTR);       // open existing battle
virtual void OnCloseDocument ();
virtual void OnFileClose ();
virtual BOOL OnSaveDocument (LPCTSTR);       // save a battle
virtual void DeleteContents ();              // remove this battle     
virtual void ServerClosesAdmin ();
virtual BOOL CanCloseFrame (CFrameWnd*);

virtual ControlView* GetControlView ();

virtual void LogonPlayer (long);
virtual void LogoffPlayer (long);
virtual void LogoffAll ();

virtual void SetHeaderAddr (HeadDoc*, long, char*);
virtual void ShowPlayer (long, bool);

virtual void ActionDone (long);

protected:

virtual long NextPlayerIs ();
virtual bool LoadBattle (LPCTSTR);           // load in a battle - setup view
virtual bool SaveBattle (LPCTSTR);           // save this battle
virtual void ClearControlValues ();
virtual void RemoveObjects ();               // removes all allocated objects
virtual bool AllLoggedOn ();
virtual void CmLogonGerman ();
virtual void CmLogonItaly ();
virtual void CmLogonJapan ();
virtual void CmLogonBritian ();
virtual void CmLogonFrance ();
virtual void CmLogonUSA ();
virtual void CmLogonRussia ();
virtual void CmLogonMinor ();
virtual void CmEnableLogonGerman (CCmdUI*);
virtual void CmEnableLogonItaly (CCmdUI*);
virtual void CmEnableLogonJapan (CCmdUI*);
virtual void CmEnableLogonBritian (CCmdUI*);
virtual void CmEnableLogonFrance (CCmdUI*);
virtual void CmEnableLogonUSA (CCmdUI*);
virtual void CmEnableLogonRussia (CCmdUI*);
virtual void CmEnableLogonMinor (CCmdUI*);

public:
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif
	DECLARE_MESSAGE_MAP()
};
