#include "stdafx.h"
#include "Pgm11a.h"
#include "ControlView.h"
#include "ControlDoc.h"   // access to document
#include "PlayerView.h"
#include "resource.h"             // pick up resource id numbers

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(ControlView, CView)

ControlView::ControlView() {
 ptrdoc  = NULL;
 ptrgame = NULL;
 action  = NULL;
 players = NULL;
}

ControlView::~ControlView() {
 RemoveObjects ();
}

BEGIN_MESSAGE_MAP(ControlView, CView)
 ON_WM_CREATE ()
 ON_MESSAGE (CM_NOTIFY_BATDOC_ACTDONE,   CmActionDone)
END_MESSAGE_MAP()

/***************************************************************************/
/*                                                                         */
/* RemoveObjects: delete all allocated objects                             */
/*                                                                         */
/***************************************************************************/

void      ControlView::RemoveObjects () {
 // place holder
}

/***************************************************************************/
/*                                                                         */
/* OnCreate: build the view with lt grey background brush                  */
/*                                                                         */
/***************************************************************************/

int      ControlView::OnCreate (LPCREATESTRUCT lpCS) {
 int retcd = CView::OnCreate (lpCS); // pass along to base class
 // install new background color
 ::SetClassLong (m_hWnd, GCL_HBRBACKGROUND,
                 (long) (HBRUSH) ::GetStockObject (LTGRAY_BRUSH));
 return retcd;
}

/***************************************************************************/
/*                                                                         */
/* OnInitialUpdate:                                                        */
/*                                                                         */
/***************************************************************************/

void      ControlView::OnInitialUpdate () {
 ptrdoc = GetDocument ();
 if (ptrdoc) {
  ptrgame = &ptrdoc->game;
  action = ptrdoc->current_action;
  players = ptrdoc->players;
 }
 GetAvgCharDims (); // set average character dimensions
}

/***************************************************************************/
/*                                                                         */
/* CmActionDone: active player done, so let other player do something      */
/*                                                                         */
/***************************************************************************/

LRESULT   ControlView::CmActionDone (WPARAM, LPARAM id) {
 ptrdoc->ActionDone (id);
 return 0;
}

/***************************************************************************/
/*                                                                         */
/* OnUpdate: respond to game status change                                 */
/*                                                                         */
/***************************************************************************/

void      ControlView::OnUpdate (CView*, LPARAM hint, CObject*) {
 if (hint == 1) { // change in current player
  if (ptrdoc->all_logged_on) {
   players[ptrgame->current_player].ptrplayerview->PostMessage (
                    CM_NOTIFY_BATDOC_NEWACT, 0, 0);
  }
 }
 Invalidate (TRUE);
}

/***************************************************************************/
/*                                                                         */
/* GetAvgCharDims: retrieve current font's average dimensions              */
/*                                                                         */
/***************************************************************************/

void      ControlView::GetAvgCharDims () {
 TEXTMETRIC  tm;  // set the system font's characteristics in tm

 CClientDC *ptrdc = new CClientDC (this);    // acquire a DC
 ptrdc->GetTextMetrics (&tm);                // get the font information
 delete ptrdc;                               // delete the dc

 // calculate average character parameters
 char_width = tm.tmAveCharWidth;
 char_height = tm.tmHeight + tm.tmExternalLeading;
}

/***************************************************************************/
/*                                                                         */
/* OnDraw: Render the image onto the off screen DIB Section and Blit to dc */
/*                                                                         */
/***************************************************************************/
    
void      ControlView::OnDraw (CDC *ptrdc) {
 ptrdc->SetBkMode (TRANSPARENT);
 ptrdc->TextOut (char_width, char_height, ptrdoc->descr);
 char turnmsg[80], turnstr[80] = "Current Turn Number: %d";
 wsprintf (turnmsg, turnstr, ptrgame->current_turn);
 ptrdc->TextOut (char_width, char_height*3, turnmsg);
 ptrdc->TextOut (char_width, char_height*5, "Players in the Battle");
 long linect = char_height *7;
 for (long i=0; i<MAX_PLAYERS; i++) {
  if (players[i].isrequired) {
   CString msg;
   msg.LoadString (IDS_PLAYER_GERMAN + i);
   ptrdc->TextOut (char_width * 2, linect, msg);
   linect += char_height; 
   if (players[i].isloggedon) {
    if (ptrgame->current_player == i)
     ptrdc->TextOut (char_width * 4, linect, "Is Logged On   Is Currently Active Player");
    else ptrdc->TextOut (char_width * 4, linect, "Is Logged On");
    linect += char_height;
   }
   else {
    ptrdc->TextOut (char_width * 4, linect, "Not Logged On");
    linect += char_height;  
   }
   linect += char_height * 3;
  }
 }  
}

/***************************************************************************/
/*                                                                         */
/* GetDocument: returns ptr to the associated CDocument                    */
/*                                                                         */
/***************************************************************************/

ControlDoc*  ControlView::GetDocument () {
 ControlDoc* ptrbdoc = (ControlDoc*) m_pDocument;
 ASSERT (ptrbdoc);
 ASSERT (ptrbdoc->IsKindOf (RUNTIME_CLASS (ControlDoc)));
 if (ptrbdoc && ptrbdoc->IsKindOf (RUNTIME_CLASS (ControlDoc)))
  return ptrbdoc;
 else return NULL;
}

#ifdef _DEBUG
void ControlView::AssertValid() const { CView::AssertValid(); }
void ControlView::Dump(CDumpContext& dc) const { CView::Dump(dc); }
#endif //_DEBUG
