#include "stdafx.h"
#include "Pgm11a.h"
#include "GameDoc.h"
#include "GameView.h"
#include "PlayerView.h"
#include "OtherView.h"
#include "ControlDoc.h"
#include "ControlView.h"
#include "resource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(GameDoc, CDocument)

GameDoc::GameDoc() {
 ptrworldDC     = NULL;
 ptrpalette     = NULL;               
 ptrcontroldoc  = NULL;
 ptrcontrolview = NULL;
 playerid      = -1;
 world_size.cx = world_size.cy = 0;
 world_rect.SetRect (0, 0, 0, 0);
 descr[0] = 0;
 canundo  = FALSE;
}

GameDoc::~GameDoc() {
 RemoveObjects (); 
}

BEGIN_MESSAGE_MAP(GameDoc, CDocument)
END_MESSAGE_MAP()

/***************************************************************************/
/*                                                                         */
/* RemoveObjects: remove all allocated objects                             */
/*                                                                         */
/***************************************************************************/

void       GameDoc::RemoveObjects () {
 if (ptrworldDC) delete ptrworldDC;
 if (ptrpalette) delete ptrpalette;
 ptrworldDC = NULL;
 ptrpalette = NULL;
 ptrcontroldoc = NULL;
 ptrcontrolview = NULL;
 playerid = -1;
 world_size.cx = world_size.cy = 0;
 world_rect.SetRect (0, 0, 0, 0);
 descr[0] = 0;
}

/***************************************************************************/
/*                                                                         */
/* PostControlAddrs: game controller installs player id and description    */
/*                   which is passed to our views and their titlebars      */
/*                                                                         */
/***************************************************************************/

void      GameDoc::PostControlAddrs (ControlDoc *ptrcdoc, long id,
                                       char *desc, ControlView *ptrv) {
 ASSERT (ptrcdoc);
 ASSERT (ptrv); 
 ASSERT (ptrcdoc->IsKindOf (RUNTIME_CLASS (ControlDoc)));
 ASSERT (ptrv->IsKindOf (RUNTIME_CLASS (ControlView)));
 if (ptrcdoc && ptrcdoc->IsKindOf (RUNTIME_CLASS (ControlDoc)) &&
     ptrv    && ptrv->IsKindOf (RUNTIME_CLASS (ControlView))) {
  playerid = id;
  ptrcontroldoc = ptrcdoc;
  ptrcontrolview = ptrv;
  strcpy_s (descr, sizeof(descr), desc);
  POSITION pos = GetFirstViewPosition ();
  CView *ptrcview;
  while (pos != NULL) {
   ptrcview = GetNextView (pos);
   if (ptrcview && ptrcview->IsKindOf (RUNTIME_CLASS (PlayerView))) {
    PlayerView *ptrpview = (PlayerView*) ptrcview;
    ptrpview->PostPlayerIdAndAddr (id, ptrcontrolview);
    CString msg;
    msg.LoadString (IDS_PLAYER_GERMAN + id);
    msg += descr;
    ptrpview->GetParentFrame ()->SetWindowText (msg);
    break;
   }
  }
 }
}

/***************************************************************************/
/*                                                                         */
/* GetPlayerId: returns the current player ID number                       */
/*                                                                         */
/***************************************************************************/

long      GameDoc::GetPlayerId () {
 return playerid;
}

/***************************************************************************/
/*                                                                         */
/* ShowPlayer: when TRUE, shows all windows, FALSE hides all windows       */
/*                                                                         */
/***************************************************************************/

void      GameDoc::ShowPlayer (bool which) {
 POSITION pos = GetFirstViewPosition ();
 CView *ptrview;
 while (pos != NULL) {
  ptrview = GetNextView (pos);
  if (ptrview && ptrview->IsKindOf (RUNTIME_CLASS (GameView))) 
   if (which) (GameView*)ptrview->GetParentFrame()->ShowWindow (SW_SHOW);
   else (GameView*)ptrview->GetParentFrame()->ShowWindow (SW_HIDE);
 }  
 isvisible = which;
}

/***************************************************************************/
/*                                                                         */
/* ActionDone: clear canundo flag                                          */
/*                                                                         */
/***************************************************************************/

void      GameDoc::ActionDone () {
 canundo = false;
}

/***************************************************************************/
/*                                                                         */
/* GetControlDoc: returns the ptr to the control doc                       */
/*                                                                         */
/***************************************************************************/

ControlDoc*   GameDoc::GetControlDoc () {
 if (ptrcontroldoc && 
     ptrcontroldoc->IsKindOf (RUNTIME_CLASS (ControlDoc)))
  return ptrcontroldoc;
 else return NULL;
}

/***************************************************************************/
/*                                                                         */
/* LoginOtherView: notify other view of player id and sets it titlebar     */
/*                                                                         */
/***************************************************************************/

void      GameDoc::LoginOtherView (OtherView *ptrview, long &id) {
 id = playerid;
 CString msg;
 msg.LoadString (IDS_OTHER_GERMAN + id);
 msg += descr;
 ptrview->GetParentFrame ()->SetWindowText (msg);
}

/***************************************************************************/
/*                                                                         */
/* LogoffPlayer: notify game controller of player departure                */
/*                                                                         */
/***************************************************************************/

void      GameDoc::LogoffPlayer () {
 if (IsModified ()) {
  SaveBattle (docfilename);
  SetModifiedFlag (FALSE);
 }
 ptrcontroldoc->LogoffPlayer (playerid);
}

/***************************************************************************/
/*                                                                         */
/* LogoffOther: notification of closure of other view                      */
/*                                                                         */
/***************************************************************************/

void      GameDoc::LogoffOther () {
}

/***************************************************************************/
/*                                                                         */
/* OnFileClose: handle file close messages - should not be needed          */
/*                                                                         */
/***************************************************************************/

void      GameDoc::OnFileClose () {
 LogoffPlayer ();
 CDocument::OnCloseDocument ();
}

/***************************************************************************/
/*                                                                         */
/* OnCloseDocument: usually called from game controller document to quit   */
/*                                                                         */
/***************************************************************************/

void      GameDoc::OnCloseDocument () {
 if (IsModified ()) {
  SaveBattle (docfilename);
  SetModifiedFlag (FALSE);
 }
 CDocument::OnCloseDocument ();
}

/***************************************************************************/
/*                                                                         */
/* GetWorldDC: returns a ptr to the background DIB Section DC              */
/*                                                                         */
/***************************************************************************/

BitmapDC*  GameDoc::GetWorldDC () {
 return ptrworldDC;
}

/***************************************************************************/
/*                                                                         */
/* GetWorldSizeAndPalette: give view world size and current palette        */
/*                                                                         */
/***************************************************************************/

Palette*  GameDoc::GetWorldSizeAndPalette (CSize &viewworld) {
 viewworld = world_size;
 return ptrpalette;
}


/***************************************************************************/
/*                                                                         */
/* OnOpenDocument: open an existing battle                                 */
/*                                                                         */
/***************************************************************************/

BOOL      GameDoc::OnOpenDocument (LPCTSTR filename) {
 if (!CDocument::OnOpenDocument (filename)) return FALSE;
 return LoadBattle (filename);
}

/***************************************************************************/
/*                                                                         */
/* OnSaveDocument: save this battle                                        */
/*                                                                         */
/***************************************************************************/

BOOL      GameDoc::OnSaveDocument (LPCTSTR filename) {
 return SaveBattle (filename); 
}

/***************************************************************************/
/*                                                                         */
/* LoadBattle: loads in a battle to play                                   */
/*                                                                         */
/***************************************************************************/

bool      GameDoc::LoadBattle (LPCTSTR filename) {
 SetModifiedFlag (TRUE);
 strcpy_s (docfilename, sizeof(docfilename), filename);
 RemoveObjects (); 
 BOOL ok; 
 
 ptrpalette = new Palette (ok);
 ptrworldDC = new BitmapDC ();
 if (!ptrpalette || !ok || !ptrworldDC) {
  CString msg1, msg2;
  msg1.LoadString (IDS_GAMEDOC_FILERRH);
  msg1 += filename;
  msg2.LoadString (IDS_GAMEDOC_FILERR1);
  MessageBox (0, msg2, msg1, MB_OK);
  return false;
 }

 if (!ptrworldDC->Read (filename, ptrpalette)) return false;
 
 world_rect.SetRect (0, 0, ptrworldDC->Width (), ptrworldDC->Height ());
 world_size.cx = ptrworldDC->Width ();
 world_size.cy = ptrworldDC->Height ();

 return true;
}

/***************************************************************************/
/*                                                                         */
/* SaveBattle: saves a battle to disk                                      */
/*                                                                         */
/***************************************************************************/

bool      GameDoc::SaveBattle (LPCTSTR filename) {
 MessageBox (0, "Saving a battle", filename, MB_OK);
 return true;
}

/***************************************************************************/
/*                                                                         */
/* DeleteContents: remove this document's contents                         */
/*                                                                         */
/***************************************************************************/

void      GameDoc::DeleteContents () {                 
 RemoveObjects ();
}

#ifdef _DEBUG
void GameDoc::AssertValid() const { CDocument::AssertValid(); }
void GameDoc::Dump(CDumpContext& dc) const { CDocument::Dump(dc); }
#endif //_DEBUG
