#pragma once
#include "Palette.h"   // access to the palette
#include "BitmapDC.h"  // access to DIB Section memory DC

class PlayerView;
class OtherView;
class ControlDoc;
class ControlView;

/***************************************************************************/
/*                                                                         */
/* GameDoc: Container for one complete game                                */
/*                                                                         */
/***************************************************************************/

class GameDoc : public CDocument {
protected:
	GameDoc(); // protected constructor used by dynamic creation
	DECLARE_DYNCREATE(GameDoc)

 /***************************************************************************/
 /*                                                                         */
 /* Data Members                                                            */
 /*                                                                         */
 /***************************************************************************/
 
public:

ControlDoc  *ptrcontroldoc;     // our controlling document
ControlView *ptrcontrolview;

BitmapDC *ptrworldDC;           // the world bitmap
Palette  *ptrpalette;           // the player's palette

long      bmpw;                 // width and height of DIB Section
long      bmph;

CSize     world_size;           // dimensions of the world battle
CRect     world_rect;           // the world rectangle

long      playerid;             // the player id
char      descr[MAX_PATH];      // description of the battle for titlebar
char      docfilename[MAX_PATH];// the filename to save into

BOOL      isvisible;
BOOL      canundo;

public:

virtual BOOL OnOpenDocument (LPCTSTR);       // open existing battle
virtual BOOL OnSaveDocument (LPCTSTR);       // save a battle
virtual void DeleteContents ();              // remove this battle     
virtual void OnCloseDocument ();             // closes this player doc down
virtual void OnFileClose ();                 // handles closing this doc
virtual void LogoffPlayer ();                // notify game control when done
virtual long GetPlayerId ();                 // returns player id

virtual BitmapDC* GetWorldDC ();             // provide access to world DC
virtual Palette*  GetWorldSizeAndPalette (CSize&);

             // sets our game control doc address
virtual void PostControlAddrs (ControlDoc*, long, char*, ControlView*);

virtual void LoginOtherView (OtherView*, long&);
virtual void LogoffOther ();

virtual void ShowPlayer (bool);              // true, show views, false, hide
virtual void ActionDone ();
virtual ControlDoc*  GetControlDoc ();
virtual ~GameDoc();

protected:

virtual bool LoadBattle (LPCTSTR);           // load in a battle - setup view
virtual bool SaveBattle (LPCTSTR);           // save this battle
virtual void RemoveObjects ();               // removes all allocated objects

public:
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif
	DECLARE_MESSAGE_MAP()
};
