#include "stdafx.h"
#include "Pgm11a.h"
#include "GameView.h"
#include "resource.h"  // pick up resource id numbers
#include "BitmapDC.h"  // the DIB Section DC
#include "Palette.h"   // the palette class
#include "GameDoc.h"   // access to the doc

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(GameView, CView)

GameView::GameView() {
 ptroffscreenDC     = NULL;
 ptrpalette         = NULL;
 lbtndown           = false;
 must_render        = false;
 world_size.cx      = world_size.cy = 0;
 offscreen_size.cx  = offscreen_size.cy  = 0;
 offscreen_offset.x = offscreen_offset.y = 0;
 offscreen_rect.SetRect (0, 0, 0, 0);
 ptrdoc             = NULL;
}

GameView::~GameView() {
 RemoveObjects ();
}

BEGIN_MESSAGE_MAP(GameView, CView)
 ON_WM_GETMINMAXINFO ()
 ON_WM_CREATE ()
 ON_WM_PALETTECHANGED ()
 ON_WM_QUERYNEWPALETTE ()
 ON_WM_HSCROLL ()
 ON_WM_VSCROLL ()
 ON_WM_SIZE ()
END_MESSAGE_MAP()

/***************************************************************************/
/*                                                                         */
/* RemoveObjects: delete all allocated objects                             */
/*                                                                         */
/***************************************************************************/

void      GameView::RemoveObjects () {
 if (ptroffscreenDC) delete ptroffscreenDC;
 ptroffscreenDC = NULL;
 ptrpalette     = NULL;
 world_size.cx  = world_size.cy = 0;
 offscreen_rect.SetRect (0, 0, 0, 0);
 offscreen_size.cx = offscreen_size.cy = 0;
 offscreen_offset.x = offscreen_offset.y = 0;
}

/***************************************************************************/
/*                                                                         */
/* OnCreate: build the view with lt grey background brush                  */
/*                                                                         */
/***************************************************************************/

int      GameView::OnCreate (LPCREATESTRUCT lpCS) {
 int retcd = CView::OnCreate (lpCS); // pass along to base class
 // install new background color
  ::SetClassLong (m_hWnd, GCL_HBRBACKGROUND,
                 (long) (HBRUSH) ::GetStockObject (LTGRAY_BRUSH));
 return retcd;
}

/***************************************************************************/
/*                                                                         */
/* OnInitialUpdate: initial create of the DIB Section off screen buffer    */
/*                                                                         */
/***************************************************************************/

void      GameView::OnInitialUpdate () {
 CFrameWnd *ptrframe = GetParentFrame ();
 CRect rf, rc, rq;
 ptrframe->GetClientRect (&rc);
 ptrframe->RepositionBars (AFX_IDW_CONTROLBAR_FIRST, AFX_IDW_CONTROLBAR_LAST,
                            NULL, reposQuery, &rf, NULL);
 long menuht = GetSystemMetrics (SM_CYMENU);
 long xsb = GetSystemMetrics (SM_CXVSCROLL);
 long ysb = GetSystemMetrics (SM_CYHSCROLL);
 long ht = rc.Height () - rf.Height () + menuht;
 rq.SetRect (0, 0, world_size.cx + xsb, world_size.cy + ht + ysb);
 if (rq.Width () > 600) rq.right = 600;
 if (rq.Height() > 600) rq.bottom = 600;
 ptrframe->CalcWindowRect (&rq, 0);
 ptrframe->SetWindowPos (NULL, 0, 0, rq.Width (), rq.Height (),
                         SWP_NOZORDER | SWP_NOACTIVATE);
 CView::OnInitialUpdate ();
}

/***************************************************************************/
/*                                                                         */
/* CreateOffScreenBuffer: create the DIB Section and save the Palette      */
/*                                                                         */
/***************************************************************************/

bool      GameView::CreateOffScreenBuffer () {
 if (!ptrpalette) {
  CreateFailure (1); // no palette
  return false;
 }
 if (ptroffscreenDC) delete ptroffscreenDC;
 ptroffscreenDC = new BitmapDC ();
 if (ptroffscreenDC)
  if (ptroffscreenDC->Create (offscreen_size.cx, offscreen_size.cy,
      ptrpalette)) {
   offscreen_rect.SetRect (0, 0, ptroffscreenDC->Width (),
                                 ptroffscreenDC->Height ());
   offscreen_size.cx = ptroffscreenDC->Width ();
   offscreen_size.cy = ptroffscreenDC->Height ();
   if (offscreen_offset.x + offscreen_size.cx > world_size.cx) {
    offscreen_offset.x = world_size.cx - offscreen_size.cx;
    if (offscreen_offset.x < 0) offscreen_offset.x = 0;
   }
   if (offscreen_offset.y + offscreen_size.cy > world_size.cy) {
    offscreen_offset.y = world_size.cy - offscreen_size.cy;
    if (offscreen_offset.y < 0) offscreen_offset.y = 0;
   }
   must_render = true;
   SetOurScrollRange ();
   Invalidate(FALSE);              // force repainting of window  
  } 
  else {
   CreateFailure (2);
   return false;
  }
 else {
  CreateFailure (3);
  return false;
 }
 return true;
}

/***************************************************************************/
/*                                                                         */
/* CreateFailure: Display why cannot create the offscreen buffer - set NULL*/
/*                                                                         */
/***************************************************************************/

void      GameView::CreateFailure (long why) {
 CString msg1, msg2;
 msg1.LoadString (IDS_GAMEVIEW_ERRHD);
 if (why == 1) msg2.LoadString (IDS_GAMEVIEW_ERR1);
 else if (why == 2) msg2.LoadString (IDS_GAMEVIEW_ERR2);
 else if (why == 3) msg2.LoadString (IDS_GAMEVIEW_ERR3);
 MessageBox (msg2, msg1, MB_OK);
 RemoveObjects ();
 must_render = TRUE;
 SetOurScrollRange ();
 Invalidate (FALSE);
}

/***************************************************************************/
/*                                                                         */
/* OnSize: respond to new size of the view window - adjust size of DIB buf */
/*                                                                         */
/***************************************************************************/

void      GameView::OnSize (UINT a, int w, int h) { 
 CView::OnSize    (a, w, h);
 w = w % 4 > 0 ? 4 - w % 4 + w : w; // force width onto 32-bit boundary
 // first avoid doing anything for initial call with no size
 if (ptroffscreenDC && w == offscreen_size.cx &&
     h == offscreen_size.cy) return;
 // if not yet attached to document, save the current size for later use
 if (!ptrpalette) {
  offscreen_rect.SetRect (0, 0, w, h); // save for real build
  offscreen_size.cx = w;
  offscreen_size.cy = h;
  offscreen_offset.x = offscreen_offset.y = 0;
  return;
 }
 else { // all ok, so build the off screen buffer
  offscreen_size.cx = w;
  offscreen_size.cy = h;
  CreateOffScreenBuffer ();
 }
}

/***************************************************************************/
/*                                                                         */
/* OnGetMinMaxInfo: force window to not exceed size of world coordinates   */
/*                                                                         */
/***************************************************************************/

void      GameView::OnGetMinMaxInfo (MINMAXINFO *ptrinfo) {
 if (world_size.cx == 0 || world_size.cy == 0) return;
 ptrinfo->ptMaxTrackSize.x = world_size.cx;
 ptrinfo->ptMaxTrackSize.y = world_size.cy;
}

/***************************************************************************/
/*                                                                         */
/* SetOurScrollRange: sets up the new scroll range in World coordinates    */
/*                                                                         */
/***************************************************************************/

void      GameView::SetOurScrollRange () {
 long x = world_size.cx - offscreen_size.cx;
 if (x < 0) x = 0;
 long y = world_size.cy - offscreen_size.cy;
 if (y < 0) y = 0;
 SetScrollRange (SB_VERT, 0, y, FALSE);
 SetScrollPos (SB_VERT, offscreen_offset.y, TRUE);
 SetScrollRange (SB_HORZ, 0, x, FALSE);
 SetScrollPos (SB_HORZ, offscreen_offset.x, TRUE);
}

/***************************************************************************/
/*                                                                         */
/* Draw: override to display background DIB Section to screen              */
/*                                                                         */
/***************************************************************************/

void      GameView::Draw (CDC *ptrdc) {
}

/***************************************************************************/
/*                                                                         */
/* Render: override to copy world map and draw sprites onto offscreen DIB  */
/*                                                                         */
/***************************************************************************/
  
void      GameView::Render (CRect*) {
}

/***************************************************************************/
/*                                                                         */
/* OnDraw: Render the image onto the off screen DIB Section and Blit to dc */
/*                                                                         */
/***************************************************************************/
    
void      GameView::OnDraw (CDC *ptrdc) {
}

/***************************************************************************/
/*                                                                         */
/* OnVScroll: process world Y direction scroll                             */
/*                                                                         */
/***************************************************************************/

void      GameView::OnVScroll (UINT type, UINT pos, CScrollBar *ptrsb) {
 long max_pels = world_size.cy - offscreen_size.cy;
 if (max_pels < 0) max_pels = 0;
 long page = offscreen_size.cy / 2 > max_pels ? max_pels : offscreen_size.cy / 2;
 long at = offscreen_offset.y;

 if (type == SB_LINEUP) {     // scroll up 1 pel
  if (at > 0) at--;
 }
 else if (type == SB_LINEDOWN) {   // scroll 1 pel down
  if (at <= max_pels) at++;
 }
 else if (type == SB_PAGEUP) {     // scroll 1 page up
  if (at > 0) 
   if (at > page) at -= page;
   else at = 0;
 }
 else if (type == SB_PAGEDOWN) {   // scroll 1 page down
  if (at <= max_pels) {
   at += page;
   if (at > max_pels) at = max_pels;
  }
 }
 else if (type == SB_THUMBTRACK) { // follow thumb bar
  at = pos;
 }

 if (at != offscreen_offset.y) {
  SetScrollPos (SB_VERT, at, TRUE); // set new scroll position
  offscreen_offset.y = at;
  must_render = TRUE;
  Invalidate (FALSE);
 }
}

/***************************************************************************/
/*                                                                         */
/* OnHScroll: process world X direction scroll                             */
/*                                                                         */
/***************************************************************************/

void      GameView::OnHScroll (UINT type, UINT pos, CScrollBar *ptrsb) {
 long max_pels = world_size.cx - offscreen_size.cx;
 if (max_pels < 0) max_pels = 0;
 long page = offscreen_size.cx / 2 > max_pels ? max_pels : offscreen_size.cx / 2;
 long at = offscreen_offset.x;

 if (type == SB_LINEUP) {          // scroll left 1 pel
  if (at > 0) at--;
 }
 else if (type == SB_LINEDOWN) {   // scroll right 1 pel
  if (at <= max_pels) at++;
 }
 else if (type == SB_PAGEUP) {     // scroll 1 page left
  if (at > 0) 
   if (at > page) at -= page;
   else at = 0;
 }
 else if (type == SB_PAGEDOWN) {   // scroll 1 page right
  if (at <= max_pels) {
   at += page;
   if (at > max_pels) at = max_pels;
  }
 }
 else if (type == SB_THUMBTRACK) { // follow thumb bar
  at = pos;
 }

 if (at != offscreen_offset.x) {
  SetScrollPos (SB_HORZ, at, TRUE); // set new scroll position
  offscreen_offset.x = at;
  must_render = TRUE;
  Invalidate (FALSE);
 }
}

/***************************************************************************/
/*                                                                         */
/* GetDocument: returns ptr to the associated CDocument                    */
/*                                                                         */
/***************************************************************************/

GameDoc*  GameView::GetDocument () {
 GameDoc* ptrbdoc = (GameDoc*) m_pDocument;
 ASSERT (ptrbdoc);
 if (ptrbdoc && ptrbdoc->IsKindOf (RUNTIME_CLASS (GameDoc))) return ptrbdoc;
 else return NULL;
}

/***************************************************************************/
/*                                                                         */
/* GetOffScreenBuffer: returns a ptr to the off screen BitmapDC            */
/*                                                                         */
/***************************************************************************/

BitmapDC*  GameView::GetOffScreenBuffer () {
 return ptroffscreenDC;
}

/***************************************************************************/
/*                                                                         */
/* OnPaletteChanged: we are inactive, so make the best of it               */
/*                                                                         */
/***************************************************************************/

void      GameView::OnPaletteChanged (CWnd *ptrwin) {
 if (ptrwin != this) OnQueryNewPalette ();
}

/***************************************************************************/
/*                                                                         */
/* OnQueryNewPalette: we are going active so realize our palette           */
/*                                                                         */
/***************************************************************************/

BOOL      GameView::OnQueryNewPalette () {
 if (ptrpalette) {
  CDC *ptrdc = GetDC ();
  CPalette *ptroldpal = ptrdc->SelectPalette (ptrpalette->GetPalette(), FALSE);
  UINT num = ptrdc->RealizePalette ();
  ReleaseDC (ptrdc);
  if (num) {
   InvalidateRect (NULL, TRUE);
   return TRUE;
  }
 }
 return FALSE;
}

#ifdef _DEBUG
void GameView::AssertValid() const { CView::AssertValid(); }
void GameView::Dump(CDumpContext& dc) const { CView::Dump(dc); }
#endif //_DEBUG
