#pragma once
#include "BitmapDC.h"
#include "Palette.h"
#include "GameDoc.h"

/***************************************************************************/
/*                                                                         */
/* GameView: Basic Offscreen Buffer Scrolled View                          */
/*                                                                         */
/***************************************************************************/

class GameView : public CView {
protected:
 GameView();           // protected constructor used by dynamic creation
 DECLARE_DYNCREATE(GameView)

public:
BitmapDC *ptroffscreenDC;  // the offscreen DIB Section bitmap
Palette  *ptrpalette;      // the palette in use
bool      must_render;     // TRUE when view must redraw its BitmapDC from map
CSize     world_size;      // the dimensions of the world

protected:
CRect     offscreen_rect;  // the offscreen DIB Section bitmap rect
CPoint    offscreen_offset;// the offscreen World X,Y offset
CSize     offscreen_size;  // the dimensions of the off screen buffer
bool      lbtndown;        // TRUE when left button is down
GameDoc  *ptrdoc;

virtual void  OnDraw(CDC* pDC);      // overridden to draw this view
virtual bool  CreateOffScreenBuffer ();      // create off the screen buffer
virtual void  Draw (CDC *ptrdc);             // place holder for derived class
virtual void  Render (CRect *ptrclip = NULL);// place holder for derived class
virtual void  OnInitialUpdate ();            // place holder for derived class 
afx_msg int   OnCreate (LPCREATESTRUCT);     // install gray background brush
afx_msg void  OnGetMinMaxInfo (MINMAXINFO*);
afx_msg void  OnSize (UINT, int, int);       // recreate new sized buffer
afx_msg void  OnVScroll (UINT, UINT, CScrollBar*); // scroll screen in X dir
afx_msg void  OnHScroll (UINT, UINT, CScrollBar*); // scroll screen in Y dir
virtual void  SetOurScrollRange ();                // sets the scrolling range
virtual void  CreateFailure (long);                // show why and set to NULL
virtual void  RemoveObjects ();                    // remove offscreen objects
virtual      ~GameView();
afx_msg void  OnPaletteChanged (CWnd*);            // respond to palette change
afx_msg BOOL  OnQueryNewPalette ();                // respond to palette change

public:
virtual GameDoc*  GetDocument ();          // get access to our document
virtual BitmapDC* GetOffScreenBuffer ();   // provide access to our buffer

#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif
	DECLARE_MESSAGE_MAP()
};
