#include "stdafx.h"
#include "Pgm11a.h"
#include "HeadData.h"
#include "HeadDoc.h"
#include "HeadView.h"
#include "HeadFrame.h"
#include "ControlDoc.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(HeadDoc, CDocument)

BEGIN_MESSAGE_MAP(HeadDoc, CDocument)
 ON_COMMAND           (CM_NEXT_BATTLE,     CmNextBattle)
 ON_COMMAND           (CM_PREV_BATTLE,     CmPrevBattle)
 ON_COMMAND           (CM_ACTIVATE_BATTLE, CmActivateBattle)
 ON_COMMAND           (CM_CLOSE_BATTLE,    CmCloseBattle)
 ON_UPDATE_COMMAND_UI (CM_NEXT_BATTLE,     CmEnableNextBattle)
 ON_UPDATE_COMMAND_UI (CM_PREV_BATTLE,     CmEnablePrevBattle)
 ON_UPDATE_COMMAND_UI (CM_ACTIVATE_BATTLE, CmEnableActivateBattle)
 ON_UPDATE_COMMAND_UI (CM_CLOSE_BATTLE,    CmEnableCloseBattle)
END_MESSAGE_MAP()

HeadDoc::HeadDoc() {
  theApp.ptrheaddoc = this;
}

/***************************************************************************/
/*                                                                         */
/* GetNumBattles: returns the number of battles objects in the collection  */
/*                                                                         */
/***************************************************************************/

long      HeadDoc::GetNumBattles () {
 return battle_array.GetSize ();
}

/***************************************************************************/
/*                                                                         */
/* GetHeadView: returns ptr to header view for minimize/normal msgs        */
/*                                                                         */
/***************************************************************************/

HeadView* HeadDoc::GetHeadView () {
 POSITION pos = GetFirstViewPosition ();
 CView *ptrview;
 while (pos != NULL) {
  ptrview = GetNextView (pos);
  if (ptrview && ptrview->IsKindOf (RUNTIME_CLASS (HeadView)))
   return (HeadView*) ptrview;
 }
 return NULL;
}

/***************************************************************************/
/*                                                                         */
/* GetThisBattle: returns a copy of the current Battle Header object       */
/*                                                                         */
/***************************************************************************/

void      HeadDoc::GetThisBattle (char *msg_descr,  char *msg_file,
                                  char *msg_status, char *msg_active, long *ptrsel, long *ptrnum) {
 if (battle_array.GetSize () == 0) {
  strcpy_s (msg_descr, _MAX_PATH, "No Battles are in this group");
  strcpy_s (msg_file, _MAX_PATH, "No file");
  strcpy_s (msg_status, 80, "None");
  strcpy_s (msg_active, 80, "None");
  *ptrsel = *ptrnum = 0;
 }
 else {
  HeadData *ptrdata = (HeadData*) battle_array.GetAt (selection);
  if (ptrdata && ptrdata->IsKindOf (RUNTIME_CLASS (HeadData))) {
   strcpy_s (msg_descr, _MAX_PATH, ptrdata->battle_descr);
   strcpy_s (msg_file, _MAX_PATH,  ptrdata->battle_file);
   if (ptrdata->battle_status == Battle_NotStarted)
    strcpy_s (msg_status, 80, "Battle has not been started");
   else if (ptrdata->battle_status == Battle_InProgress)
    strcpy_s (msg_status, 80, "Battle is in progress");
   else if (ptrdata->battle_status == Battle_Completed)
    strcpy_s (msg_status, 80, "Battle has been fully completed");
   else strcpy_s (msg_status, 80, "Unknown");
   if (ptrdata->ptrctldoc)
    strcpy_s (msg_active, 80, "Battle Administration Active - Battle can be played");
   else
    strcpy_s (msg_active, 80, "Battle Not Logged On Yet");
   *ptrsel = selection;
   *ptrnum = battle_array.GetSize ();
  }
 }
}

/***************************************************************************/
/*                                                                         */
/* DeleteContents: remove the array of BattleHeaderData objects            */
/*                                                                         */
/***************************************************************************/

void     HeadDoc::DeleteContents () {
 long i;
 selection = 0;
 if (battle_array.GetSize () !=0) {
  for (i=0; i<battle_array.GetSize (); i++)    // for each data object
   delete (HeadData*) battle_array.GetAt (i); // get at it and delete it
  battle_array.RemoveAll ();                   // now empty the array container
 }
}

/***************************************************************************/
/*                                                                         */
/* OnNewDocument: launch a new BattleHeader document                       */
/*                                                                         */
/***************************************************************************/

BOOL     HeadDoc::OnNewDocument () {
 selection = 0;
 if (!CDocument::OnNewDocument ()) return FALSE;
 HeadData *ptrdata = new HeadData ();
 strcpy_s (ptrdata->battle_descr, sizeof(ptrdata->battle_descr),
          "Denmark Battle between Germans and French");
 strcpy_s (ptrdata->battle_file, sizeof(ptrdata->battle_file),
          "D:\\Windows MFC Programming II\\Pgm11a\\Game\\BattleM.btl");
 ptrdata->battle_status = Battle_NotStarted;
 battle_array.Add (ptrdata);  // add the new object to the array
 ptrdata = new HeadData ();
 strcpy_s (ptrdata->battle_descr, sizeof(ptrdata->battle_descr),
          "Small Denmark Battle between Germans and French");
 strcpy_s (ptrdata->battle_file, sizeof(ptrdata->battle_file),
          "D:\\Windows MFC Programming II\\Pgm11a\\Game\\BattleS.btl");
 ptrdata->battle_status = Battle_InProgress;
 battle_array.Add (ptrdata); // add the new object to the array
 return TRUE;
}

/***************************************************************************/
/*                                                                         */
/* OnOpenDocument: open existing BattleHeader document                                  */
/*                                                                         */
/***************************************************************************/

BOOL     HeadDoc::OnOpenDocument (const char *filename) {
 selection = 0;
 CString path (filename);
 long i = path.ReverseFind ('\\');
 const char *pathc = (LPCTSTR) path;
 strcpy_s (theApp.mainpath, sizeof(theApp.mainpath), pathc);
 theApp.mainpath [i+1] = 0;
 BOOL retcd = CDocument::OnOpenDocument (filename);
 if (!retcd) {
  CFile file;              // the file, will be binary I/O still
  CFileException file_err; // used if more info on errors is desired
  file.Open (filename, CFile::modeRead, &file_err);
  HeadData *ptrhead;
  ptrhead = new HeadData ();
  while (ptrhead->Read (file)) {
   battle_array.Add (ptrhead);   // and add it to the array
   ptrhead = new HeadData ();
  }
  delete ptrhead;
  file.Close ();
  if (battle_array.GetSize () > 0) {
   retcd = TRUE;
   SetModifiedFlag (TRUE);
   CDocument::SetPathName (filename, TRUE);
  }
 }
 SetModifiedFlag (TRUE);
 return retcd;
}

/***************************************************************************/
/*                                                                         */
/* Serialize: input or output the whole document - I/O binary data only    */
/*                                                                         */
/***************************************************************************/

void      HeadDoc::Serialize (CArchive &ar) {
 if (ar.IsStoring ()) {                    // save to .BTH file
  if (battle_array.GetSize () == 0) return;// none to save
  battle_array.Serialize (ar);             // output to container's serializer
 }
 else battle_array.Serialize (ar);         // input from container's serializer
}

/***************************************************************************/
/*                                                                         */
/* CmNextBattle: scroll to next battle group to examine                    */
/*                                                                         */
/***************************************************************************/

void      HeadDoc::CmNextBattle () {
 if (selection + 1 >= battle_array.GetSize ()) {
  MessageBeep (0);
  return;
 }
 selection++;
 UpdateAllViews (NULL, 0, NULL);
}

/***************************************************************************/
/*                                                                         */
/* CmPrevBattle: scroll to previous battle to examine                      */
/*                                                                         */
/***************************************************************************/

void      HeadDoc::CmPrevBattle () {
 if (selection - 1 < 0) {
  MessageBeep (0);
  return;
 }
 selection--;
 UpdateAllViews (NULL, 0, NULL);
}

/***************************************************************************/
/*                                                                         */
/* CmActivateBattle: open this battle for logon and playing                */
/*                                                                         */
/***************************************************************************/

void      HeadDoc::CmActivateBattle () {
 if (battle_array.GetSize ()) {
  HeadData *ptrdata = (HeadData*) battle_array.GetAt (selection);
  if (ptrdata->ptrctldoc == NULL) {
   CDocument *ptrdoc = theApp.ptrcontroltmp->
                       OpenDocumentFile (ptrdata->battle_file);
   ASSERT (ptrdoc->IsKindOf (RUNTIME_CLASS (ControlDoc)));
   if (ptrdoc && ptrdoc->IsKindOf (RUNTIME_CLASS (ControlDoc))) {
    ptrdata->ptrctldoc = (ControlDoc*) ptrdoc;
    ptrdata->ptrctldoc->SetHeaderAddr (this, selection, ptrdata->battle_descr);
    UpdateAllViews (NULL, 0, NULL);
    GetHeadView ()->GetParentFrame ()->ShowWindow (SW_MINIMIZE);
    return;
   }
  }
 }
 MessageBeep (0);
}

/***************************************************************************/
/*                                                                         */
/* LogoffBattle: BattleControlDoc view is terminating - kill whole battle  */
/*                                                                         */
/***************************************************************************/

void      HeadDoc::LogoffBattle (long id) {
 long num = battle_array.GetSize ();
 if (num && id >=0 && id < num) {
  HeadData *ptrdata = (HeadData*) battle_array.GetAt (id);
  if (ptrdata && ptrdata->IsKindOf (RUNTIME_CLASS (HeadData))) {
   ptrdata->ptrctldoc = NULL;
   GetHeadView ()->GetParentFrame ()->ShowWindow (SW_RESTORE);
   UpdateAllViews (NULL, 0, NULL);
  }
 }
}

/***************************************************************************/
/*                                                                         */
/* LogoffAllBattles: BattleHeaderDoc view is terminating - kill all battles*/
/*                                                                         */
/***************************************************************************/

void      HeadDoc::LogoffAllBattles () {
 long num = battle_array.GetSize ();
 for (long i = 0; i<num; i++) {
  HeadData *ptrdata = (HeadData*) battle_array.GetAt (i);
  if (ptrdata && ptrdata->IsKindOf (RUNTIME_CLASS (HeadData))) {
   ControlDoc *ptrdoc = ptrdata->ptrctldoc;
   if (ptrdoc && ptrdoc->IsKindOf (RUNTIME_CLASS (ControlDoc))) {
    ptrdoc->ServerClosesAdmin ();
    UpdateAllViews (NULL, 0, NULL);
   }
  }
 }
}

/***************************************************************************/
/*                                                                         */
/* OnFileClose: close all server docs and those below it                   */
/*                                                                         */
/***************************************************************************/

void      HeadDoc::OnFileClose () {
 OnCloseDocument ();
}

/***************************************************************************/
/*                                                                         */
/* OnCloseDocument: close the server connection and all documents below it */
/*                                                                         */
/***************************************************************************/

void      HeadDoc::OnCloseDocument () {
 LogoffAllBattles ();
 theApp.ptrheaddoc = NULL;
 CString s = GetPathName ();
 if (!s.IsEmpty ()) {
  // update registry with this bth file
  ((Pgm11aApp*) AfxGetApp ())->SavePath (s);
 }
 if (IsModified ()) {
  OnSaveDocument (s);
 }
 CDocument::OnCloseDocument ();
}

/***************************************************************************/
/*                                                                         */
/* CanCloseFrame: respond to termination request                           */
/*                                                                         */
/***************************************************************************/

BOOL      HeadDoc::CanCloseFrame (CFrameWnd *ptrwin) {
 if (!ptrwin) return TRUE;
 if (!ptrwin->IsKindOf (RUNTIME_CLASS (HeadFrame))) return FALSE;
 CView *ptrview = ((HeadFrame*) ptrwin)->GetActiveView ();
 if (ptrview && ptrview->IsKindOf (RUNTIME_CLASS (HeadView))) {
  CString msg1, msg2;
  msg1.LoadString (IDS_BATTLEHEAD_CLOSE);
  msg2.LoadString (IDS_BATTLEHEAD_QUERY);
  if (MessageBox (0, msg2, msg1, MB_YESNO | MB_ICONQUESTION) == IDYES) {
   LogoffAllBattles ();
   return CDocument::CanCloseFrame (ptrwin);//TRUE;
  }
  return FALSE;
 }
 return FALSE;
} 

/***************************************************************************/
/*                                                                         */
/* CmCloseBattle: terminate this battle and logoff all players             */
/*                                                                         */
/***************************************************************************/

void      HeadDoc::CmCloseBattle () {
 if (battle_array.GetSize ()) {
  HeadData *ptrdata = (HeadData*) battle_array.GetAt (selection);
  if (ptrdata &&
      ptrdata->IsKindOf (RUNTIME_CLASS (HeadData)) &&
      ptrdata->ptrctldoc &&
      ptrdata->ptrctldoc->IsKindOf (RUNTIME_CLASS (ControlDoc))) {
   ptrdata->ptrctldoc->ServerClosesAdmin ();
   ptrdata->ptrctldoc = NULL;
   UpdateAllViews (NULL, 0, NULL);
  }
  else MessageBeep (0);
 }
 else MessageBeep (0);
}

/***************************************************************************/
/*                                                                         */
/* CmEnableNextBattle: enable or disable Next Battle menuitem              */
/*                                                                         */
/***************************************************************************/

void      HeadDoc::CmEnableNextBattle (CCmdUI *ptrui) {
 ptrui->Enable (selection + 2 > battle_array.GetSize () ? 0 : 1);
}

/***************************************************************************/
/*                                                                         */
/* CmEnablePrevBattle: enable or disable Previous Battle menuitem          */
/*                                                                         */
/***************************************************************************/

void      HeadDoc::CmEnablePrevBattle (CCmdUI *ptrui) {
 ptrui->Enable (selection > 0 ? 1 : 0);
}

/***************************************************************************/
/*                                                                         */
/* CmEnableActivateBattle: enable or disable Activate Battle menuitem      */
/*                                                                         */
/***************************************************************************/

void      HeadDoc::CmEnableActivateBattle (CCmdUI *ptrui) {
 ptrui->Enable (battle_array.GetSize () == 0 ? 0 :
               ((HeadData*) battle_array.GetAt (selection))->ptrctldoc
                ? 0 : 1);
}

/***************************************************************************/
/*                                                                         */
/* CmEnableCloseBattle: enable or disable Close Battle menuitem            */
/*                                                                         */
/***************************************************************************/

void      HeadDoc::CmEnableCloseBattle (CCmdUI *ptrui) {
 ptrui->Enable (battle_array.GetSize () == 0 ? 0 :
               ((HeadData*) battle_array.GetAt (selection))->ptrctldoc
                ? 1 : 0);
}

HeadDoc::~HeadDoc() { }

#ifdef _DEBUG
void HeadDoc::AssertValid() const { CDocument::AssertValid(); }
void HeadDoc::Dump(CDumpContext& dc) const { CDocument::Dump(dc); }
#endif //_DEBUG
