#include "stdafx.h"
#include "Pgm11a.h"
#include "MainFrame.h"
#include "Pgm11a.h"
#include "HeadData.h"
#include "HeadDoc.h"
#include "GameDoc.h"
#include "PlayerView.h"
#include "OtherView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(MainFrame, CMDIFrameWnd)

BEGIN_MESSAGE_MAP(MainFrame, CMDIFrameWnd)
	ON_WM_CREATE()
	ON_WM_CLOSE()
 ON_WM_PALETTECHANGED ()
 ON_WM_QUERYNEWPALETTE ()
 ON_COMMAND (CM_SERVER_CONNECT,   CmServerConnect)
 ON_UPDATE_COMMAND_UI (CM_SERVER_CONNECT,  CmEnableServerConnect)
 ON_COMMAND (CM_MAKE_OTHER_VIEW,  CmMakeOtherView)
 ON_COMMAND (CM_FILE_CLOSE_OTHER, CmCloseOtherView)
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

MainFrame::MainFrame() { }
MainFrame::~MainFrame() { }

int MainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct) {
	if (CMDIFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	if (!m_wndToolBar.Create(this) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME)) 	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT))) 	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	m_wndToolBar.SetBarStyle(m_wndToolBar.GetBarStyle() |
		CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);

	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

	return 0;
}

/***************************************************************************/
/*                                                                         */
/* OnClose: query user about termination of whole battle                   */
/*                                                                         */
/***************************************************************************/

void      MainFrame::OnClose () {
 CString msg1, msg2;
 msg1.LoadString (IDS_BATTLEMAIN_CLOSE);
 msg2.LoadString (IDS_BATTLEMAIN_QUERY);
 if (MessageBox (msg2, msg1, MB_YESNO | MB_ICONQUESTION) == IDYES) {
  if (theApp.ptrheaddoc && 
      theApp.ptrheaddoc->IsKindOf (RUNTIME_CLASS (HeadDoc))) {
   theApp.ptrheaddoc->OnCloseDocument ();
   theApp.ptrheaddoc = NULL;
  }
  CMDIFrameWnd::OnClose ();
 }
}

/***************************************************************************/
/*                                                                         */
/* CmEnableServerConnect: enabler for menu server connect item             */
/*                                                                         */
/***************************************************************************/

void      MainFrame::CmEnableServerConnect (CCmdUI *ptrui) {
 ptrui->Enable (theApp.ptrheaddoc ? 0 : 1);
}

/***************************************************************************/
/*                                                                         */
/* CmServerConnect: reconnect to the server to logon to battle games       */
/*                                                                         */
/***************************************************************************/

void      MainFrame::CmServerConnect () {
 if (!theApp.ptrheaddoc) {
  CString bthfile = theApp.RetrievePath ();
  if (bthfile.IsEmpty()) bthfile = "Game\\Battle.bth";
  theApp.ptrheadtmp->OpenDocumentFile (LPCSTR(bthfile));
 }
}

/***************************************************************************/
/*                                                                         */
/* CmMakeOtherView: create opponent view                                   */
/*                                                                         */
/***************************************************************************/

void      MainFrame::CmMakeOtherView () {
 CMDIChildWnd *ptrMDIActive = MDIGetActive ();
 ASSERT(ptrMDIActive != NULL);
 if (!ptrMDIActive) return;
 
 CDocument *ptrdoc = ptrMDIActive->GetActiveDocument ();
 ASSERT(ptrdoc != NULL);
 if (!ptrdoc) return;

 CView *ptrview;
 POSITION pos = ptrdoc->GetFirstViewPosition ();
 while (pos != NULL) {
  ptrview = ptrdoc->GetNextView (pos);
  if (ptrview && ptrview->IsKindOf (RUNTIME_CLASS (OtherView))) {
   ptrview->GetParentFrame ()->ActivateFrame ();
   return;
  }
 }

 if (ptrdoc->IsKindOf (RUNTIME_CLASS (GameDoc))) {
  GameDoc *ptrbdoc = (GameDoc*) ptrdoc;
  long id = ptrbdoc->GetPlayerId ();
  if (id == -1) return;
  CMDIChildWnd *ptrNewFrame;
  ptrNewFrame = (CMDIChildWnd*) (theApp.ptrothertmp->CreateNewFrame (ptrdoc, NULL));
  if (!ptrNewFrame) return;     // not created
  if (ptrNewFrame->IsKindOf (RUNTIME_CLASS (CMDIChildWnd))) 
   theApp.ptrothertmp->InitialUpdateFrame (ptrNewFrame, ptrdoc);

  pos = ptrdoc->GetFirstViewPosition ();
  while (pos != NULL) {
   ptrview = ptrdoc->GetNextView (pos);
   if (ptrview && ptrview->IsKindOf (RUNTIME_CLASS (PlayerView))) {
    ptrview->GetParentFrame ()->ActivateFrame ();
    break;
   }
  }
  MDITile (MDITILE_VERTICAL);
 }
}

/***************************************************************************/
/*                                                                         */
/* CmCloseOtherView: close opponent view                                   */
/*                                                                         */
/***************************************************************************/

void      MainFrame::CmCloseOtherView () {
 CMDIChildWnd *ptrMDIActive = MDIGetActive ();
 ASSERT(ptrMDIActive != NULL);
 if (!ptrMDIActive) return;
 
 CDocument *ptrdoc = ptrMDIActive->GetActiveDocument ();
 ASSERT(ptrdoc != NULL);
 if (!ptrdoc) return;

 CView *ptrview;
 POSITION pos = ptrdoc->GetFirstViewPosition ();
 while (pos != NULL) {
  ptrview = ptrdoc->GetNextView (pos);
  if (ptrview && ptrview->IsKindOf (RUNTIME_CLASS (OtherView))) {
   ptrview->GetParentFrame ()->PostMessage (WM_CLOSE, 0, 0L);
   return;
  }
 }
}

/***************************************************************************/
/*                                                                         */
/* OnPaletteChanged: we are inactive, so let views make the best of it     */
/*                                                                         */
/***************************************************************************/

void      MainFrame::OnPaletteChanged (CWnd *ptrwin) {
 CView *ptrview = GetActiveView ();
 if (ptrview) ptrview->SendMessage (WM_PALETTECHANGED, 
                             (WPARAM) (ptrwin->GetSafeHwnd ()), (LPARAM) 0);
}

/***************************************************************************/
/*                                                                         */
/* OnQueryNewPalette: we are going active, so let all views realize our pal*/
/*                                                                         */
/***************************************************************************/

BOOL      MainFrame::OnQueryNewPalette () {
 CView *ptrview = GetActiveView ();
 if (ptrview) return ptrview->SendMessage (WM_QUERYNEWPALETTE, 0, 0L);
 return FALSE;
} 

BOOL MainFrame::PreCreateWindow(CREATESTRUCT& cs) {
	return CMDIFrameWnd::PreCreateWindow(cs);
}

#ifdef _DEBUG
void MainFrame::AssertValid() const { CMDIFrameWnd::AssertValid(); }
void MainFrame::Dump(CDumpContext& dc) const { CMDIFrameWnd::Dump(dc); }
#endif //_DEBUG
