#include "stdafx.h"
#include "Pgm11a.h"
#include "MainFrame.h"
#include "HeadData.h"
#include "HeadDoc.h"
#include "HeadView.h"
#include "HeadFrame.h"
#include "ControlDoc.h"
#include "ControlView.h"
#include "ControlFrame.h"
#include "GameDoc.h"
#include "GameFrame.h"
#include "PlayerView.h"
#include "OtherView.h"
#include "Palette.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

BEGIN_MESSAGE_MAP(Pgm11aApp, CWinApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
//	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
//	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
END_MESSAGE_MAP()

Pgm11aApp::Pgm11aApp() { }
Pgm11aApp theApp;

BOOL Pgm11aApp::InitInstance() {
 ptrheaddoc = NULL;
 SetRegistryKey ("WindowsMFCII"); // setup the higher level reg key
 LoadStdProfileSettings (10);     // load most recently used files
 
 ptrheadtmp = new CMultiDocTemplate(IDR_HEADFRAME,
                                    RUNTIME_CLASS(HeadDoc),
                                    RUNTIME_CLASS(HeadFrame),
                                    RUNTIME_CLASS(HeadView));
 AddDocTemplate (ptrheadtmp);

 ptrcontroltmp = new CMultiDocTemplate(IDR_CONTROLFRAME,
                                      RUNTIME_CLASS(ControlDoc),
                                      RUNTIME_CLASS(ControlFrame),
                                      RUNTIME_CLASS(ControlView));
 AddDocTemplate (ptrcontroltmp);

 ptrplayertmp = new CMultiDocTemplate (IDR_PLAYERFRAME,
                                        RUNTIME_CLASS(GameDoc),
                                        RUNTIME_CLASS(GameFrame),
                                        RUNTIME_CLASS(PlayerView));
 AddDocTemplate (ptrplayertmp);

 ptrothertmp  = new CMultiDocTemplate (IDR_OTHERFRAME,
                                        RUNTIME_CLASS(GameDoc),
                                        RUNTIME_CLASS(GameFrame),
                                        RUNTIME_CLASS(OtherView));
 AddDocTemplate (ptrothertmp);

 MainFrame *ptrmainframe = new MainFrame;
 if (!ptrmainframe->LoadFrame (IDR_MAINFRAME)) return FALSE;
 ptrmainframe->ShowWindow (SW_SHOWMAXIMIZED);

 ptrmainframe->UpdateWindow ();
 m_pMainWnd = ptrmainframe;

 // automatic open of the game bth file
 CString bthfile = RetrievePath ();
 if (bthfile.IsEmpty()) {
  // here it was not found, so try the Game subfolder
  bthfile = "Game\\Battle.bth";
 }
 // build the filename for the Palette file
 CString palfile = GetPaletteFile (bthfile);
 if (!Palette::InitPalette (LPCSTR(palfile))) {
  AfxMessageBox ("Master.pal file failed to initialize", MB_OK);
  return FALSE;
 }
 OpenDocumentFile (LPCSTR(bthfile));
 
 // comment out the above series and uncomment this series if you want
 // to use the find dialog box
/* CWinApp::OnFileOpen();
 char pfile[MAX_PATH];
 strcpy (pfile, mainpath);
 strcat (pfile, "Master.pal");
 if (!Palette::InitPalette (pfile)) return FALSE;
 */
 return TRUE;
}

/***************************************************************************/
/*                                                                         */
/* RetrievePath: obtain the installed path from the system registry        */
/*                                                                         */
/***************************************************************************/

CString   Pgm11aApp::RetrievePath () {
 CString subkey = "SOFTWARE\\WindowsMFCII\\Pgm11a\\Settings";
 CString entry = "BthFile";
 CString filename = "";
 HKEY hkey;
 LONG ok;
 // attempt to make the Settings key, if it is not there yet
 ok = RegCreateKeyEx (HKEY_CURRENT_USER, subkey, 0, 0, 0, KEY_CREATE_SUB_KEY, 0, &hkey, 0);
 if (ok != ERROR_SUCCESS) return filename;

 // attempt to open the current user Settings key with our value
 ok = RegOpenKeyEx (HKEY_CURRENT_USER, subkey, 0, KEY_READ, &hkey);
 if (ok == ERROR_SUCCESS) {
  DWORD dwType, dwCount;
  // key exists, now verify BthFile entry is a string
  ok = RegQueryValueEx (hkey, entry, NULL, &dwType, NULL, &dwCount);
  if (ok == ERROR_SUCCESS && dwType == REG_SZ) {
   // now retrieve the actual path string
			ok = RegQueryValueEx (hkey, entry, NULL, &dwType,
                         (LPBYTE) filename.GetBuffer (dwCount/sizeof(TCHAR)),
                         &dwCount);
			filename.ReleaseBuffer(); // release the CString buffer locked by GetBuffer
  }
  RegCloseKey (hkey);
 }
 return filename;
}

/***************************************************************************/
/*                                                                         */
/* SavePath: save path of bth file in registry                             */
/*                                                                         */
/***************************************************************************/

bool   Pgm11aApp::SavePath (CString& filename) {
 CString subkey = "SOFTWARE\\WindowsMFCII\\Pgm11a\\Settings";
 CString entry = "BthFile";
 HKEY hkey;
 LONG ok;
 // build Settings key, if it is not there yet
 ok = RegCreateKeyEx (HKEY_CURRENT_USER, subkey, 0, 0, 0, KEY_CREATE_SUB_KEY, 0, &hkey, 0);
 if (ok != ERROR_SUCCESS) return false;
 // attempt to open the current user Settings key with our value
 ok = RegOpenKeyEx (HKEY_CURRENT_USER, subkey, 0, KEY_WRITE, &hkey);
 if (ok == ERROR_SUCCESS) {
  // key exists, now store the value in BthFile
  ok = RegSetValueEx (hkey, entry, 0, REG_SZ, (const BYTE*) filename.GetBuffer(), filename.GetLength ()+1);
  if (ok == ERROR_SUCCESS) {
			filename.ReleaseBuffer(); // release the CString buffer locked by GetBuffer
   RegCloseKey (hkey);
   return true;
  }
  RegCloseKey (hkey);
 }
 return false;
}

/***************************************************************************/
/*                                                                         */
/* GetPaletteFile: make the palette filename from the bth filename         */
/*                                                                         */
/***************************************************************************/

CString Pgm11aApp::GetPaletteFile (CString& bthfile) {
 char drive[_MAX_DRIVE];
 char dir[_MAX_PATH];
 char file[_MAX_PATH];
 char ext[_MAX_EXT];
 _splitpath_s ((LPCSTR) bthfile, drive, _MAX_DRIVE, dir, _MAX_PATH, file, _MAX_PATH, ext, _MAX_EXT);
 char answer[_MAX_PATH];
 _makepath_s (answer, _MAX_PATH, drive, dir, "Master", ".pal");
 return CString (answer);
}

/***************************************************************************/
/*                                                                         */
/* DisplayMsg: display a message box and return results                    */
/*                                                                         */
/***************************************************************************/

long      Pgm11aApp::DisplayMsg (long id1, long id2, UINT flags) {

 CString msg1, msg2;
 msg1.LoadString (id1);
 msg2.LoadString (id2);
 return ::MessageBox (0, msg2, msg1, flags);
}

/***************************************************************************/
/*                                                                         */
/* ExitInstance: delete any palette, if running on older systems           */
/*                                                                         */
/***************************************************************************/

int Pgm11aApp::ExitInstance() {
 Palette::DeletePalette ();
	return CWinApp::ExitInstance();
}

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog {
public:
	CAboutDlg();
	enum { IDD = IDD_ABOUTBOX };

	protected:
	virtual void DoDataExchange(CDataExchange* pDX);
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD) { }

void CAboutDlg::DoDataExchange(CDataExchange* pDX) {
	CDialog::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
END_MESSAGE_MAP()

void Pgm11aApp::OnAppAbout() {
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

