#include "stdafx.h"
#include "Pgm11a.h"
#include "PlayerView.h"
#include "ControlView.h"
#include "GameDoc.h" 
#include "resource.h"
#include "MainFrame.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(PlayerView, GameView)

PlayerView::PlayerView() : GameView () {
 playerid = -1; // indicates no player id yet installed
 ptrcontrolview = NULL;
}

PlayerView::~PlayerView() { }

BEGIN_MESSAGE_MAP(PlayerView, GameView)
 ON_WM_CHAR ()
 ON_WM_KEYDOWN ()
 ON_COMMAND (CM_BATTLEDOC_ACTIONDONE,    CmActionDone)
 ON_COMMAND (CM_BATTLEDOC_UNDO,          CmUndo)
 ON_UPDATE_COMMAND_UI(CM_BATTLEDOC_UNDO, CmEnableUndo)
END_MESSAGE_MAP()

/***************************************************************************/
/*                                                                         */
/* PostPlayerIdAndAddr: call back from document to install player number   */
/*                      and game controller view for message posting       */
/*                                                                         */
/***************************************************************************/

void      PlayerView::PostPlayerIdAndAddr (long id, ControlView *ptrv) {
 playerid = id;
 ptrcontrolview = ptrv;
}

/***************************************************************************/
/*                                                                         */
/* OnInitialUpdate: set the initial size of the view                       */
/*                                                                         */
/***************************************************************************/

void      PlayerView::OnInitialUpdate () {
 GameDoc *ptrbdoc = GetDocument ();
 if (ptrbdoc && ptrbdoc->IsKindOf (RUNTIME_CLASS (GameDoc))) {
  ptrdoc = ptrbdoc;
  ptrpalette = ptrdoc->GetWorldSizeAndPalette (world_size);
  GameView::OnInitialUpdate ();
  CreateOffScreenBuffer (); // let base handle errors
 }
}

/***************************************************************************/
/*                                                                         */
/* OnUpdate: Process hints from document                                   */
/*                                                                         */
/***************************************************************************/

void      PlayerView::OnUpdate(CView *ptrsender, LPARAM lhint,
                                     CObject* /*ptrhint*/) {
 ASSERT(ptrsender != this);               // avoid repaint if it's us
 if (lhint == 0) Invalidate (TRUE);       // force repaint, erase background
 else if (lhint == 1) Invalidate (FALSE); // force repaint, no erase on scroll
}

/***************************************************************************/
/*                                                                         */
/* Draw:                                                                   */
/*                                                                         */
/***************************************************************************/

void      PlayerView::Draw (CDC *ptrdc) {
 if (!ptroffscreenDC) return;

 CRect drawrect = offscreen_rect;
 CRect cliprect;
 ptrdc->GetClipBox (cliprect);
 CPalette *ptroldpal = NULL;  // install our palette
 if (ptrpalette) {
  ptroldpal = ptrdc->SelectPalette (ptrpalette->GetPalette (), FALSE);
  ptrdc->RealizePalette ();
 }
 // copy our offscreen buffer to the screen
 BitBlt (ptrdc->GetSafeHdc (), drawrect.left, drawrect.top, drawrect.Width(),
         drawrect.Height (), ptroffscreenDC->hbmpdc, drawrect.left,
         drawrect.top, SRCCOPY);
 if (ptroldpal) ptrdc->SelectPalette (ptroldpal, 0);
}

/***************************************************************************/
/*                                                                         */
/* Render:                                                                 */
/*                                                                         */
/***************************************************************************/

void      PlayerView::Render (CRect*) {
 GameDoc *ptrdoc = GetDocument ();
 BitmapDC *ptrmemdc = ptrdoc->GetWorldDC (); // get access to the world map DC
 if (!ptrmemdc || !ptroffscreenDC) return;
 // copy the portion of the world map to our offscreen buffer
 BitBlt (ptroffscreenDC->hbmpdc, 0, 0, offscreen_rect.Width (),
         offscreen_rect.Height (), ptrmemdc->hbmpdc, offscreen_offset.x,
         offscreen_offset.y, SRCCOPY);
 CDC dc;
 dc.Attach (ptroffscreenDC->hbmpdc);
 CPalette *ptroldpal = NULL;  // install our palette
 if (ptrpalette) {
  ptroldpal = dc.SelectPalette (ptrpalette->GetPalette (), FALSE);
  dc.RealizePalette ();
 }
 if (ptroldpal) dc.SelectPalette (ptroldpal, 0);
 dc.Detach ();
}

/***************************************************************************/
/*                                                                         */
/* OnDraw:                                                                 */
/*                                                                         */
/***************************************************************************/

void      PlayerView::OnDraw (CDC *ptrdc) {
 if (must_render) Render ();
 Draw (ptrdc);
}

/***************************************************************************/
/*                                                                         */
/* OnChar: get hot key actions                                             */
/*                                                                         */
/***************************************************************************/

void      PlayerView::OnChar (UINT c, UINT a, UINT b) {
 switch (c) {
  case 0x1b: // escape
  case 26:   // ^Z
   // undo
   break;
  default:
   GameView::OnChar (c, a, b);
 };
}
  
/***************************************************************************/
/*                                                                         */
/* OnSysKeyDown: get hot key actions                                       */
/*                                                                         */
/***************************************************************************/

void      PlayerView::OnKeyDown (UINT c, UINT a, UINT b) {
 switch (c) {
  case VK_F2: 
   CmActionDone ();
   break;
  default:
   GameView::OnKeyDown (c, a, b);
 };
}

/***************************************************************************/
/*                                                                         */
/* CmActionDone: post done to control doc                                  */
/*                                                                         */
/***************************************************************************/

void      PlayerView::CmActionDone () {
 if (ptrdoc) ptrdoc->ActionDone ();
 if (ptrcontrolview)
  ptrcontrolview->PostMessage (CM_NOTIFY_BATDOC_ACTDONE, 0, playerid);
}

/***************************************************************************/
/*                                                                         */
/* CmUndo: undo last action                                                */
/*                                                                         */
/***************************************************************************/

void      PlayerView::CmUndo () {
 if (ptrdoc && ptrdoc->canundo) {
  //...
 }
}

/***************************************************************************/
/*                                                                         */
/* CmEnableUndo: enable undo menuitem                                      */
/*                                                                         */
/***************************************************************************/

void      PlayerView::CmEnableUndo (CCmdUI *ptrui) {
 ptrui->Enable (ptrdoc ? ptrdoc->canundo ? 1 : 0 : 0);
}

#ifdef _DEBUG
void PlayerView::AssertValid() const { GameView::AssertValid(); }
void PlayerView::Dump(CDumpContext& dc) const { GameView::Dump(dc); }
#endif //_DEBUG
