#include "stdafx.h"
#include "Pgm14c.h"
#include "RYGWindow.h"

IMPLEMENT_DYNAMIC (RYGWindow, CWnd)

RYGWindow::RYGWindow() {
 whichFocus = -1;   // say none has the focus
}

RYGWindow::~RYGWindow() { }

BOOL RYGWindow::PreCreateWindow(CREATESTRUCT& cs) {
 if (!CWnd::PreCreateWindow(cs))
  return FALSE;
 // register our the fancy window
 cs.lpszClass = AfxRegisterWndClass (
                  CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS, 
                  ::LoadCursor(NULL, IDC_ARROW),
                  (HBRUSH) ::GetStockObject (LTGRAY_BRUSH),
                  NULL);
 return TRUE;
}

BEGIN_MESSAGE_MAP(RYGWindow, CWnd)
 ON_WM_CHAR()
 ON_WM_CREATE()
 ON_WM_LBUTTONDOWN()
 ON_WM_PAINT()
 ON_WM_SIZE()
END_MESSAGE_MAP()

void RYGWindow::SetText (const char* txt) {
 text = txt;
}

void RYGWindow::SetChecked (int which) {
 if (which < 0 || which > 2) {
  MessageBox ("Requested radio button is out of range (0-2)", "Error", MB_OK);
  return;
 }
 whichChecked = which; // set the requested button as on
}

int RYGWindow::GetChecked () const {
 return whichChecked;  // return which button is on
}

void RYGWindow::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) {
 if (nChar == VK_TAB) { // tabbing through the buttons
  whichFocus++;         // increment to the next button
  if (whichFocus > 2)   // past the last one, so
   whichFocus = -1;     // indicate none and force a repaint
  Invalidate ();
 }
}

bool RYGWindow::StillHasFocus () const{
 return whichFocus != -1; // let dialog caller know we do or do not have focus
}

int RYGWindow::OnCreate(LPCREATESTRUCT lpCreateStruct) {
 if (CWnd::OnCreate(lpCreateStruct) == -1)
  return -1;
 // create our three colored brushes
 brushRed.CreateSolidBrush (RGB (255,0,0));
 brushYellow.CreateSolidBrush (RGB (255,255,0));
 brushGreen.CreateSolidBrush (RGB (0,255,0));
 return 0;
}

void RYGWindow::OnLButtonDown(UINT nFlags, CPoint point) {
 // sort out which button was clicked and set both
 // the integer indicator and force a repaint showing that one in
 // its color, while undoing the others colors
 if (red.PtInRect (point)) {
  whichChecked = 0;
  Invalidate ();
 }
 else if (yellow.PtInRect (point)) {
  whichChecked = 1;
  Invalidate ();
 }
 else if (green.PtInRect (point)) {
  whichChecked = 2;
  Invalidate ();
 }
}

// paint the buttons and any focus rectangle, along with the text
void RYGWindow::OnPaint() {
 CPaintDC dc(this);
 dc.SetBkMode (TRANSPARENT);
 GetClientRect (client);
 dc.SelectStockObject (LTGRAY_BRUSH);
 dc.Rectangle (client);  // erase our window
 // draw the three outer radio button circles
 dc.Ellipse (&outerRed);
 dc.Ellipse (&outerYellow);
 dc.Ellipse (&outerGreen);
 
 // for each button, if it is selected, insert the color brush
 // draw the center circle
 // if it has the focus, draw the focus rectangle
 if (whichChecked == 0) dc.SelectObject (brushRed);
 else dc.SelectObject (::GetStockObject (LTGRAY_BRUSH));
 dc.Ellipse (red);
 if (whichFocus == 0) dc.DrawFocusRect (outerRed);
 
 if (whichChecked == 1) dc.SelectObject (brushYellow);
 else dc.SelectObject (::GetStockObject (LTGRAY_BRUSH));
 dc.Ellipse (yellow);
 if (whichFocus == 1) dc.DrawFocusRect (outerYellow);
 
 if (whichChecked == 2) dc.SelectObject (brushGreen);
 else dc.SelectObject (::GetStockObject (LTGRAY_BRUSH));
 dc.Ellipse (green);
 if (whichFocus == 2) dc.DrawFocusRect (outerGreen);

 // show the text line
 dc.TextOut (textStart.x, textStart.y, text);
}

void RYGWindow::OnSize(UINT nType, int cx, int cy) {
 if (cx == 0) return;
 // layout the three radio buttons and text locations
 // each button has two parts: outer circle and inner circle
 // outer circle are 15 pixels in diameter or 45 total
 // place the three buttons centered vertically within the window's
 // height and 10 pixels from the left edge
 // place text 2 pixels above the start of the center button
 GetClientRect (client);
 int top = (client.Height () - 45)/2;
 outerRed.top = top;
 outerRed.bottom = outerRed.top + 15;
 outerYellow.top = outerRed.bottom;
 outerYellow.bottom = outerYellow.top + 15;
 outerGreen.top = outerYellow.bottom;
 outerGreen.bottom = outerGreen.top + 15;
 outerRed.left = outerYellow.left = outerGreen.left = 10;
 outerRed.right = outerYellow.right = outerGreen.right = outerRed.left + 15;

 textStart.x = outerYellow.right + 4;
 textStart.y = outerYellow.top - 2;
 
 // inner colored circles are 4 pixels less in diameter
 red = outerRed;
 yellow = outerYellow;
 green = outerGreen;
 red.DeflateRect (2, 2);
 yellow.DeflateRect (2, 2);
 green.DeflateRect (2, 2);
}
