#include "StdAfx.h"
#include "FrameWin.h"
#include "FontEnumDlg.h"
#include "resource.h"

/***************************************************************************/
/*                                                                         */
/* FontEnumDlg Events Response Table                                       */
/*                                                                         */
/***************************************************************************/

BEGIN_MESSAGE_MAP(FontEnumDlg, CDialog)
 ON_NOTIFY (LVN_ITEMCHANGED, IDC_FONTENUMLIST, OnItemSelected)
END_MESSAGE_MAP()

/***************************************************************************/
/*                                                                         */
/* FontEnumDlg: save the transfer LOGFONT buffer address                   */
/*                                                                         */
/***************************************************************************/

          FontEnumDlg::FontEnumDlg (CWnd *ptrparent, LOGFONT *ptrf) 
                      : CDialog (IDD_FONTENUM, ptrparent) {

 ptrlf = ptrf; // save transfer buffer - parent's LOGFONT
}

/***************************************************************************/
/*                                                                         */
/* OnInitDialog: build the list control with enumerated fonts              */
/*                                                                         */
/***************************************************************************/

BOOL      FontEnumDlg::OnInitDialog () {

 CDialog::OnInitDialog ();

 // gain access to the CListCtrl itself
 ptrlistview = (CListCtrl*) (GetDlgItem (IDC_FONTENUMLIST));
 // create the small icons for the images - 0 = TT font, 1 = printer font
 bmplist.Create (IDB_FONTBMP, 20, 0, RGB (128, 128, 128));
 bmplist.SetBkColor (GetSysColor (COLOR_WINDOW));
 // install the image list
 ptrlistview->SetImageList (&bmplist, LVSIL_SMALL);
 // set one report style column with no header
 ptrlistview->InsertColumn (0, "", LVCFMT_LEFT, 200);

 static char   printer[80];
 char         *device, *driver, *output, *next;
 CDC          *ptrprinterDC = new CDC;

 // get access to a printer DC from the ini file installed printer
 GetProfileString ("windows", "device", "...", printer, 80);
 if ((device = strtok_s (printer, ",", &next)) != NULL &&
     (driver = strtok_s (NULL,    ",", &next)) != NULL &&
     (output = strtok_s (NULL,    ",", &next)) != NULL) {
  
  // a default printer exists, so get a DC for it
  ptrprinterDC->CreateDC (driver, device, output, NULL);
  // build font list by enumerating fonts, adding to CPtrArray and to list view
  BuildFontList (ptrprinterDC);
  delete ptrprinterDC;
 }
 return TRUE;
}

/***************************************************************************/
/*                                                                         */
/* ~FontEnumDlg: remove the array of enumerated fonts                      */
/*                                                                         */
/***************************************************************************/

          FontEnumDlg::~FontEnumDlg () {
 
 int num, i;
 num = fonts_array.GetSize ();
 for (i=0; i<num; i++) delete (FONTITEM*) fonts_array.GetAt (i);
 fonts_array.RemoveAll ();
}

/***************************************************************************/
/*                                                                         */
/* BuildFontList: enumerates fonts and adds to the list view and array     */
/*                                                                         */
/***************************************************************************/

void      FontEnumDlg::BuildFontList (CDC *ptrprinterDC) {

 // determine whether running under Windows 95 or NT
 DWORD version = ::GetVersion ();
 BOOL  win95   = (BYTE) version >= 4; // TRUE for Win 95; FALSE for Win NT

 // construct the model LOGFONT to enumerate - use only DEFAULT_CHARSET
 LOGFONT lf;
 memset (&lf, 0, sizeof (LOGFONT));
 lf.lfCharSet = DEFAULT_CHARSET;

 // enumerate all fonts on either Win 95 or Win NT system
 if (win95) ::EnumFontFamiliesEx (ptrprinterDC->GetSafeHdc(), &lf, 
                        (FONTENUMPROC) EnumPrinterFonts, (LPARAM) this, NULL);
 else ::EnumFontFamilies (ptrprinterDC->GetSafeHdc(), NULL, 
                          (FONTENUMPROC) EnumPrinterFonts, (LPARAM) this);
 // now add the array of fonts to the list view
 AddFontsToList ();
}

/***************************************************************************/
/*                                                                         */
/* EnumPrinterFonts: call back function to handle each font                */
/*                                                                         */
/***************************************************************************/

BOOL CALLBACK AFX_EXPORT  FontEnumDlg::EnumPrinterFonts (ENUMLOGFONT *ptrelf, 
                                                NEWTEXTMETRICEX* /* ptrntm*/,
                                                int fonttype, LPVOID pthis) {
 // eliminate unwanted fonts, such as Turkish and MAC fonts
 if (ptrelf->elfLogFont.lfCharSet == DEFAULT_CHARSET ||
     ptrelf->elfLogFont.lfCharSet == ANSI_CHARSET ||
     ptrelf->elfLogFont.lfCharSet == SYMBOL_CHARSET ||
     ptrelf->elfLogFont.lfCharSet == OEM_CHARSET) {
  // if TrueType, construct our copy of the enumerated font and add to array
  if (fonttype & TRUETYPE_FONTTYPE) {
   FONTITEM *ptrf = new FONTITEM;
   memcpy (&ptrf->lf, &ptrelf->elfLogFont, sizeof (LOGFONT));
   ptrf->ttFont = TRUE;
   ((FontEnumDlg*) pthis)->fonts_array.Add (ptrf);
  }
  // if printer font, construct our copy of the enumerated font & add to array
  else if (fonttype & DEVICE_FONTTYPE) {
   FONTITEM *ptrf = new FONTITEM;
   memcpy (&ptrf->lf, &ptrelf->elfLogFont, sizeof (LOGFONT));
   ptrf->ttFont = FALSE;
   ((FontEnumDlg*) pthis)->fonts_array.Add (ptrf);
  }
 }
 return 1;
}

/***************************************************************************/
/*                                                                         */
/* AddFontsToList: adds all of the fonts to the list view control          */
/*                                                                         */
/***************************************************************************/

void      FontEnumDlg::AddFontsToList () {

 int num, i;
 FONTITEM *ptrf;

 num = fonts_array.GetSize (); // get total number of fonts to add
 LV_ITEM lv;
 // initialize the constant information
 lv.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM | LVIF_STATE;
 lv.iSubItem = 0;
 lv.state = 0;
 lv.iItem = 0;
 for (i=0; i<num; i++) {
  // access the current font
  ptrf = (FONTITEM*) fonts_array.GetAt (i);
  // use the face name in the list view
  lv.pszText = ptrf->lf.lfFaceName;
  lv.cchTextMax = lstrlen (lv.pszText);
  lv.lParam = (LPARAM) ptrf;
  // set which image to use 0 = TT, 1 = printer
  lv.iImage = ptrf->ttFont? 0 : 1;
  // add this item to the list view
  ptrlistview->InsertItem (&lv);
 }
}

/***************************************************************************/
/*                                                                         */
/* OnSelectedItem: end dialog and pass log font to parent                  */
/*                                                                         */
/***************************************************************************/

void      FontEnumDlg::OnItemSelected (NMHDR *ptrhdr, LRESULT *ptrresult) {

 NM_LISTVIEW *ptrlv = (NM_LISTVIEW*) ptrhdr;   // gain access to header
 if (ptrlv->uNewState & LVIS_SELECTED) {       // handle if its a selected item
  FONTITEM *ptrf = (FONTITEM*) ptrlv->lParam;  // extract the font item
  memcpy (ptrlf, &ptrf->lf, sizeof (LOGFONT)); // copy the log font to parent
  PostMessage (WM_COMMAND, IDOK, 0);           // send terminate message
 }
 *ptrresult = 0;
}
