#include "StdAfx.h"
#include "FrameWin.h"
#include "LogFontDlg.h"
#include "resource.h"

/***************************************************************************/
/*                                                                         */
/* FrameWin Events Response Table                                          */
/*                                                                         */
/***************************************************************************/

BEGIN_MESSAGE_MAP(FrameWin, CFrameWnd)
 ON_WM_CREATE () 
 ON_WM_PAINT ()
 ON_COMMAND(CM_MAPFONT, CmMapFont)
END_MESSAGE_MAP()

/***************************************************************************/
/*                                                                         */
/* FrameWin: create the frame window                                       */
/*                                                                         */
/***************************************************************************/

          FrameWin::FrameWin (CWnd* ptrparent, const char *title)
                  : CFrameWnd () {

 ptrdlg = NULL;
 ptrprinter = NULL;
 isprinter = FALSE;             // set for screen DC

 DWORD style = WS_OVERLAPPEDWINDOW;  // set window styles
 CRect rect (10, 10, 600, 550);      // set init pos and size

 Create (AfxRegisterWndClass (
              CS_VREDRAW | CS_HREDRAW,       // register window style UINT
              AfxGetApp()->LoadStandardCursor (IDC_ARROW),     // use arrow cur
              (HBRUSH) GetStockObject (LTGRAY_BRUSH),          // gray brush
              AfxGetApp()->LoadStandardIcon (IDI_APPLICATION)),// set min icon
          title,               // window caption
          style,               // wndclass DWORD style
          rect,                // set initial window position
          ptrparent,           // the parent window, here none
          NULL);               // assign the main menu
}

/***************************************************************************/
/*                                                                         */
/* OnCreate: create initial font, create and launch the modeless dialog    */
/*                                                                         */
/***************************************************************************/

int      FrameWin::OnCreate (LPCREATESTRUCT lpCS) {

 int retcd = CFrameWnd::OnCreate (lpCS); // pass along to base class

 // get access to a printer DC from the copy maintained by CWinApp
 PRINTDLG pd;                       
 AfxGetApp ()->GetPrinterDeviceDefaults (&pd);
 ptrprinter->FromHandle (pd.hDC); 

 CClientDC dc (this);                // acquire a DC for char dims
 dc.SelectStockObject (SYSTEM_FONT); // use the default system font
 memset (&lf, 0, sizeof(LOGFONT));   // clear the main LOGFONT data
 GetAvgCharDims (dc);                // get system font metrics

 // allocate, Create, and Show the modeless dialog box
 ptrdlg = new LogfontDlg (this, &lf, &tm, &isprinter);
 ptrdlg->Create (MAPDIALOG, this);
 ptrdlg->ShowWindow (SW_SHOW);

 // force the dialog box to be located at the top of our client window
 CRect r;
 GetClientRect (&r);
 ptrdlg->SetWindowPos (NULL, 0, GetSystemMetrics (SM_CYCAPTION), 0, 0,
                       SWP_NOSIZE | SWP_NOZORDER);
 return retcd;
}

/***************************************************************************/
/*                                                                         */
/* ~FrameWin: destructor - remove modeless dialog and printer instances    */
/*                                                                         */
/***************************************************************************/

          FrameWin::~FrameWin () {

 if (ptrdlg) {
  ptrdlg->DestroyWindow (); // modeless dialogs: destroy windows element
  delete ptrdlg;            // and delete the C++ windows object
 }
 delete ptrprinter;
}

/***************************************************************************/
/*                                                                         */
/* GetAvgCharDims: retrieve current font's average dimensions              */
/*                                                                         */
/***************************************************************************/

void      FrameWin::GetAvgCharDims (CDC &dc) {

 dc.GetTextMetrics (&tm);                  // get the font information

 // calculate average character parameters
 avg_char_width  = tm.tmAveCharWidth;
 avg_char_height = tm.tmHeight + tm.tmExternalLeading;
 avg_caps_width  = (tm.tmPitchAndFamily & 1 ? 3 : 2) * avg_char_width / 2;
}

/***************************************************************************/
/*                                                                         */
/* OnPaint: create and install font and get its metrics, then paint sample */
/*                                                                         */
/***************************************************************************/

void      FrameWin::OnPaint () {

 CPaintDC  dc (this);

 // first handle the printer DC case
 if (isprinter && ptrprinter) {
  CFont pfont, *ptroldpfont;
  pfont.CreateFontIndirect (&lf);          // construct the font
  // realize the font on the printer DC - MATCH FONT
  ptroldpfont = ptrprinter->SelectObject (&pfont);
  GetAvgCharDims (*ptrprinter);            // get its physical metrics
  pfont.GetObject (sizeof(LOGFONT), &lf);  // and any LOGFONT updates
  ptrprinter->SelectObject (ptroldpfont);  // reinstall original font
 }

 // now construct the screen version of the font
 CFont font, *ptroldfont;
 font.CreateFontIndirect (&lf);        // construct the font
 // realize the font on the screen DC - MATCH FONT
 ptroldfont = dc.SelectObject (&font);
 if (!isprinter) {    // if not the printer, then obtain metrics
  GetAvgCharDims (dc);
  font.GetObject (sizeof(LOGFONT), &lf);   // and updated font data
 }

 // display a test sample in a white window in the bottom of the client window
 CRect r;
 GetClientRect (&r);  // get window's dimensions
 // make a clipping region for the sample display
 CRgn reg;
 CRect irect = CRect (1, r.bottom - 49, r.right - 1, r.bottom -1);
 reg.CreateRectRgnIndirect (&irect);
 // draw a rectangle forming the sample window
 dc.Rectangle (0, r.bottom - 50, r.right, r.bottom);
 dc.SelectClipRgn (&reg);     // install the clipping region - preserve the frame
 dc.SetBkMode (TRANSPARENT);
 // display as much of the sample text as fits - clipping region handles details
 CString facename;
 dc.GetTextFace (facename);
 dc.TextOut (2, r.bottom - 48, facename + 
          ": AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz1234567890");
 dc.SelectObject (ptroldfont);

 // make the dialog the active window once more and reset its controls
 ptrdlg->SetActiveWindow (); // make the dialog the active window
 ptrdlg->Reset ();           // and force the metrics to be displayed
}

/***************************************************************************/
/*                                                                         */
/* CmMapFont: make a new font and map it                                   */
/*                                                                         */
/***************************************************************************/

void      FrameWin::CmMapFont () {

 Invalidate ();
}
