#include "StdAfx.h"
#include "FrameWin.h"
#include "LogFontDlg.h"
#include "resource.h"

/***************************************************************************/
/*                                                                         */
/* LogfontDlg Response Table - pass on the Match Font request              */
/*                                                                         */
/***************************************************************************/

BEGIN_MESSAGE_MAP(LogfontDlg, CDialog)
  ON_COMMAND(IDC_MATCHFONT, CmMatchFont)
END_MESSAGE_MAP()

/***************************************************************************/
/*                                                                         */
/* LogfontDlg constructor - save passed addresses, alloc controls          */
/*                                                                         */
/***************************************************************************/

          LogfontDlg::LogfontDlg (FrameWin *ptrpar, LOGFONT *ptrlog,
                                  TEXTMETRIC *ptrtext, BOOL *ptrpr)
                     : CDialog () {

 // save addresses of the real transfer buffers
 ptrlf = ptrlog;
 ptrtm = ptrtext;
 ptrparent = ptrpar;
 ptrisprinter = ptrpr;
}

/***************************************************************************/
/*                                                                         */
/* LogfontFld::DoExchange: actually transfer data in and out of controls   */
/*                                                                         */
/***************************************************************************/
     
void      LogfontDlg::DoExchange (CDataExchange *ptrdata) {

 // transfers all settings of the series of buttons beginning with the first
 // be sure that only the first radio btn of the group has the WS_GROUP style
 // and that there is a group control after the last in the group
 // the value in the DDX member is the index of the button in the group that's
 // on - taken from the order that the btns are defined in the dialog box
 // in the resource file
 
 DDX_Text  (ptrdata, IDC_LFHEIGHT, height);
 DDX_Text  (ptrdata, IDC_LFWIDTH, width);
 DDX_Text  (ptrdata, IDC_LFFACENAME, facename);
 DDX_Radio (ptrdata, IDC_DEFAULT_PITCH, pitch);
 DDX_Radio (ptrdata, IDC_FF_DONTCARE, family);
 DDX_Radio (ptrdata, IDC_FW_DONTCARE, weight);
 DDX_Radio (ptrdata, IDC_ANSI_CHARSET, charset);
 DDX_Radio (ptrdata, IDC_DEFAULT_QUALITY, quality);
 DDX_Check (ptrdata, IDC_LFITALIC, italic);
 DDX_Check (ptrdata, IDC_LFUNDERLINE, underline);
 DDX_Check (ptrdata, IDC_LFSTRIKEOUT, strikeout);
 DDX_Radio (ptrdata, IDC_CLIP_DEFAULT_PRECIS, clip);
 DDX_Radio (ptrdata, IDC_OUT_DEFAULT_PRECIS, output);
 DDX_Radio (ptrdata, IDC_SCREEN, screen);
}

/***************************************************************************/
/*                                                                         */
/* LogfontFld::DoDataExchange: transfer data in and out of controls        */
/*                                                                         */
/***************************************************************************/
     
void      LogfontDlg::DoDataExchange (CDataExchange *ptrdata) {

 CDialog::DoDataExchange (ptrdata);

 // separate the transfer cases
 if (!ptrdata->m_bSaveAndValidate) {
  // set the data in the transfer buffer according to LOGFONT & TEXTMETRICS
  // set the desired DC radio button group index
  screen = *ptrisprinter ? 1 : 0;

  // fill in the edit boxes for height, width and facename
  char fill[10];
  height.Empty ();
  _ltoa_s (ptrlf->lfHeight, fill, sizeof(fill), 10);
  height = fill;
  width.Empty ();
  _ltoa_s (ptrlf->lfWidth, fill, sizeof(fill), 10);
  width = fill;
  facename.Empty ();
  facename = ptrlf->lfFaceName;

  // extract the pitch portion from lfPitchAndFamily - the low order nibble
  BYTE pit = ptrlf->lfPitchAndFamily;
  pit = (BYTE) (pit & (BYTE) 0x0f); // mask out the family in the high nibble
  // set the pitch radio buttons
  if (pit == FIXED_PITCH) pitch = 1;
  else if (pit == VARIABLE_PITCH) pitch = 2;
  else if (pit == TMPF_TRUETYPE) pitch = 3;
  else pitch = 0;
 
  // extract the family portion - the high order nibble
  BYTE fam = ptrlf->lfPitchAndFamily;
  fam = (BYTE) (fam & (BYTE) 0xf0);  // mask out the pitch in the low nibble
  // set the family radio buttons
  if (fam == FF_MODERN) family = 1;
  else if (fam == FF_ROMAN) family = 2;
  else if (fam == FF_SWISS) family = 3;
  else if (fam == FF_SCRIPT) family = 4;
  else if (fam == FF_DECORATIVE) family = 5;
  else family = 0;

  // set the font weight radio buttons
  switch (ptrlf->lfWeight) {
   case FW_THIN:       weight = 1; break;
   case FW_EXTRALIGHT: weight = 2; break;
   case FW_LIGHT:      weight = 3; break;
   case FW_NORMAL:     weight = 4; break;
   case FW_MEDIUM:     weight = 5; break;
   case FW_SEMIBOLD:   weight = 6; break;
   case FW_BOLD:       weight = 7; break;
   case FW_EXTRABOLD:  weight = 8; break;
   case FW_HEAVY:      weight = 9; break;
   case FW_DONTCARE:   weight = 0; break;
  };

  // set the charset radio buttons
  if (ptrlf->lfCharSet == ANSI_CHARSET) charset = 0;
  else if (ptrlf->lfCharSet == SYMBOL_CHARSET) charset = 2;
  else if (ptrlf->lfCharSet == OEM_CHARSET) charset = 3;
  else charset = 1;

  // set the quality radio buttons
  if (ptrlf->lfQuality == DRAFT_QUALITY) quality = 1;
  else if (ptrlf->lfQuality == PROOF_QUALITY) quality = 2;
  else quality = 0;

  // set the check boxes for italic, underline and strikeout
  italic = ptrlf->lfItalic == 0 ? FALSE : TRUE;
  underline = ptrlf->lfUnderline == 0 ? FALSE : TRUE;
  strikeout = ptrlf->lfStrikeOut == 0 ? FALSE : TRUE;

  // set the clipping radio buttons
  if (ptrlf->lfClipPrecision == CLIP_CHARACTER_PRECIS) clip = 1;
  else if (ptrlf->lfClipPrecision == CLIP_STROKE_PRECIS) clip = 2;
  else clip = 0;

  // set the output radio buttons
  if (ptrlf->lfOutPrecision == OUT_TT_PRECIS) output = 1;
  else if (ptrlf->lfOutPrecision == OUT_TT_ONLY_PRECIS) output = 2;
  else if (ptrlf->lfOutPrecision == OUT_RASTER_PRECIS) output = 3;
  else if (ptrlf->lfOutPrecision == OUT_STRING_PRECIS) output = 4;
  else if (ptrlf->lfOutPrecision == OUT_STROKE_PRECIS) output = 5;
  else if (ptrlf->lfOutPrecision == OUT_CHARACTER_PRECIS) output = 6;
  else if (ptrlf->lfOutPrecision == OUT_DEVICE_PRECIS) output = 7;
  else output = 0;

  // fill up the static controls with the character version of the TEXTMETRICS
  wsprintf (text01, "%5d", ptrtm->tmHeight);
  wsprintf (text02, "%5d", ptrtm->tmAscent);
  wsprintf (text03, "%5d", ptrtm->tmDescent);
  wsprintf (text04, "%5d", ptrtm->tmInternalLeading);
  wsprintf (text05, "%5d", ptrtm->tmExternalLeading);
  wsprintf (text06, "%5d", ptrtm->tmAveCharWidth);
  wsprintf (text07, "%5d", ptrtm->tmMaxCharWidth);
  wsprintf (text08, "%5d", ptrtm->tmWeight);
  // note that italic, underline and strikeout values are non-zero if enabled
  if (ptrtm->tmItalic) strcpy_s (text09, sizeof(text09), "Yes");
  else strcpy_s (text09, sizeof(text09), " No");
  if (ptrtm->tmUnderlined) strcpy_s (text10,sizeof(text10),  "Yes");
  else strcpy_s (text10, sizeof(text10), " No");
  if (ptrtm->tmStruckOut) strcpy_s (text11, sizeof(text11), "Yes");
  else strcpy_s (text11, sizeof(text11), " No");
  wsprintf (text12, "%5d", ptrtm->tmFirstChar);
  wsprintf (text13, "%5d", ptrtm->tmLastChar);
  wsprintf (text14, "%5d", ptrtm->tmDefaultChar);
  wsprintf (text15, "%5d", ptrtm->tmBreakChar);
  // while the family portion uses the same values as LOGFONT's family,
  // the pitch is quite differently handled
  pit = fam = ptrtm->tmPitchAndFamily;
  pit = (BYTE) (pit & (BYTE) 0x0fU);
  fam = (BYTE) (fam & (BYTE) 0xf0);
  // if the TMPF_FIXED_PITCH bit is ON, it is a variable font - OFF, it's fixed
  if (pit & TMPF_FIXED_PITCH) strcpy_s (text16, sizeof(text16), "Var ");
  else strcpy_s (text16, sizeof(text16), "Fix ");
  // Note: some fonts can turn on several of these attributes
  if (pit & TMPF_VECTOR) strcat_s (text16, sizeof(text16), "Vec ");
  if (pit & TMPF_DEVICE) strcat_s (text16, sizeof(text16), "Dev ");
  if (pit & TMPF_TRUETYPE) strcat_s (text16, sizeof(text16), "TT ");
  strcat_s (text16, sizeof(text16), "- ");
  if (fam == FF_MODERN) strcat_s (text16, sizeof(text16), "Modern");
  else if (fam == FF_ROMAN) strcat_s (text16, sizeof(text16), "Roman");
  else if (fam == FF_SWISS) strcat_s (text16, sizeof(text16), "Swiss");
  else if (fam == FF_SCRIPT) strcat_s (text16, sizeof(text16), "Script");
  else strcat_s (text16, sizeof(text16), "Decor");
  wsprintf (text17, "%5d", ptrtm->tmCharSet);
  wsprintf (text18, "%5d", ptrtm->tmOverhang);
  wsprintf (text19, "%5d", ptrtm->tmDigitizedAspectX);
  wsprintf (text20, "%5d", ptrtm->tmDigitizedAspectY);

  // now actually transfer the data into the dialog
  DoExchange (ptrdata);

  // and set the static results controls
  SetDlgItemText (IDC_TM01, text01);
  SetDlgItemText (IDC_TM02, text02);
  SetDlgItemText (IDC_TM03, text03);
  SetDlgItemText (IDC_TM04, text04);
  SetDlgItemText (IDC_TM05, text05);
  SetDlgItemText (IDC_TM06, text06);
  SetDlgItemText (IDC_TM07, text07);
  SetDlgItemText (IDC_TM08, text08);
  SetDlgItemText (IDC_TM09, text09);
  SetDlgItemText (IDC_TM10, text10);
  SetDlgItemText (IDC_TM11, text11);
  SetDlgItemText (IDC_TM12, text12);
  SetDlgItemText (IDC_TM13, text13);
  SetDlgItemText (IDC_TM14, text14);
  SetDlgItemText (IDC_TM15, text15);
  SetDlgItemText (IDC_TM16, text16);
  SetDlgItemText (IDC_TM17, text17);
  SetDlgItemText (IDC_TM18, text18);
  SetDlgItemText (IDC_TM19, text19);
  SetDlgItemText (IDC_TM20, text20);
 }
 else {
  // Match Font has been pressed, so retrieve user's selections and convert into
  // the parent's LOGFONT structure
 
  // transfer front he controls to the XFER structure
  DoExchange (ptrdata);

  // set the BOOL indicating the DC desired
  *ptrisprinter = screen ? TRUE : FALSE;

  // now convert to LOGFONT members
  ptrlf->lfHeight = atoi (height);
  ptrlf->lfWidth = atoi (width);
  strcpy_s (ptrlf->lfFaceName, sizeof(ptrlf->lfFaceName), facename);
  ptrlf->lfPitchAndFamily = 0;
 
  if (pitch == 1) ptrlf->lfPitchAndFamily |= FIXED_PITCH;
  else if (pitch == 2) ptrlf->lfPitchAndFamily |=VARIABLE_PITCH;
  else if (pitch == 3) ptrlf->lfPitchAndFamily |= TMPF_TRUETYPE;
  else ptrlf->lfPitchAndFamily |= DEFAULT_PITCH;
 
  if (family == 1) ptrlf->lfPitchAndFamily |= FF_MODERN;
  else if (family == 2) ptrlf->lfPitchAndFamily |= FF_ROMAN;
  else if (family == 3) ptrlf->lfPitchAndFamily |= FF_SWISS;
  else if (family == 4) ptrlf->lfPitchAndFamily |= FF_SCRIPT;
  else if (family == 5) ptrlf->lfPitchAndFamily |= FF_DECORATIVE;
  else ptrlf->lfPitchAndFamily |= FF_DONTCARE;
 
  if (weight == 1) ptrlf->lfWeight = FW_THIN;
  else if (weight == 2) ptrlf->lfWeight = FW_EXTRALIGHT;
  else if (weight == 3) ptrlf->lfWeight = FW_LIGHT;
  else if (weight == 4) ptrlf->lfWeight = FW_NORMAL;
  else if (weight == 5) ptrlf->lfWeight = FW_MEDIUM;
  else if (weight == 6) ptrlf->lfWeight = FW_SEMIBOLD;
  else if (weight == 7) ptrlf->lfWeight = FW_BOLD;
  else if (weight == 8) ptrlf->lfWeight = FW_EXTRABOLD;
  else if (weight == 9) ptrlf->lfWeight = FW_HEAVY;
  else if (weight == 0) ptrlf->lfWeight = FW_DONTCARE;
 
  if (charset == 0) ptrlf->lfCharSet = ANSI_CHARSET;
  else if (charset == 2) ptrlf->lfCharSet = SYMBOL_CHARSET;
  else if (charset == 3) ptrlf->lfCharSet = OEM_CHARSET;
  else ptrlf->lfCharSet = DEFAULT_CHARSET;
 
  if (quality == 1) ptrlf->lfQuality = DRAFT_QUALITY;
  else if (quality == 2) ptrlf->lfQuality = PROOF_QUALITY;
  else ptrlf->lfQuality = DEFAULT_QUALITY;
 
  ptrlf->lfItalic    = italic ? (BYTE) 1 : (BYTE) 0;
  ptrlf->lfUnderline = underline ? (BYTE) 1 : (BYTE) 0;
  ptrlf->lfStrikeOut = strikeout ? (BYTE) 1 : (BYTE) 0;

  if (clip == 1) ptrlf->lfClipPrecision = CLIP_CHARACTER_PRECIS;
  else if (clip == 2) ptrlf->lfClipPrecision = CLIP_STROKE_PRECIS;
  else ptrlf->lfClipPrecision = CLIP_DEFAULT_PRECIS;

  if (output == 1) ptrlf->lfOutPrecision = OUT_TT_PRECIS;
  else if (output == 2) ptrlf->lfOutPrecision = OUT_TT_ONLY_PRECIS;
  else if (output == 3) ptrlf->lfOutPrecision = OUT_RASTER_PRECIS;
  else if (output == 4) ptrlf->lfOutPrecision = OUT_STRING_PRECIS;
  else if (output == 5) ptrlf->lfOutPrecision = OUT_STROKE_PRECIS;
  else if (output == 6) ptrlf->lfOutPrecision = OUT_CHARACTER_PRECIS;
  else if (output == 7) ptrlf->lfOutPrecision = OUT_DEVICE_PRECIS;
  else ptrlf->lfOutPrecision = OUT_DEFAULT_PRECIS;
 }
}

/***************************************************************************/
/*                                                                         */
/* CmMatchFont: Signals Parent to match LOGFONT and get TEXTMETRICS        */
/*                                                                         */
/***************************************************************************/

void      LogfontDlg::CmMatchFont () {

 UpdateData (TRUE);        // obtain current LOGFONT data from the dialog
 ptrparent->CmMapFont ();  // notify parent to construct font
}

/***************************************************************************/
/*                                                                         */
/* Reset: once font is matched, transfer TEXTMETRICS and LOGFONT data back */
/*        into the dialog controls                                         */
/*                                                                         */
/***************************************************************************/

void     LogfontDlg::Reset () {

 UpdateData (FALSE);
}
