#include "stdafx.h"
#include "Pgm12a.h"
#include "Function.h"
#include <cmath>
using namespace std;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/***************************************************************************/
/*                                                                         */
/* Function: base class for all graph functions                            */
/*                                                                         */
/***************************************************************************/

/***************************************************************************/
/*                                                                         */
/* Function: constructor saves parms and NULLs the array pointers          */
/*                                                                         */
/***************************************************************************/

Function::Function (int num, double &p1, double &p2) {
 num_pts = num;                // save initial calues
 parm1 = p1;
 parm2 = p2;

 use_hatch_brush = true;       // use hatch marks under curve

 x = NULL;                     // set no arrays allocated yet
 y = NULL;
 pts = NULL;
 xtics_x = NULL;
 xtics_msg = NULL;
 ytics_y = NULL;
 ytics_msg = NULL;

 max_quad_size  = 16000;       // full size quadrant
 draw_quad_size = 14000;       // drawing's reduced size
 border_size = (max_quad_size - draw_quad_size) /2; // border size
 font_height = 0;              // indicate no font yet installed
 pointsztens = 160;            // default point size in tens for titles
 strcpy_s (title, sizeof(title), "No Graph");   // indicate no graph yet done
 strcpy_s (caption, sizeof(caption), "No Graph");
 title_on_right = false;       // place title left justified
 ticksize = 400;               // length of ticks on both axis
}

/***************************************************************************/
/*                                                                         */
/* ~Function: deletes any arrays                                           */
/*                                                                         */
/***************************************************************************/
  
          Function::~Function () {
 if (x)         delete [] x;
 if (y)         delete [] y;
 if (pts)       delete [] pts;
 if (xtics_x)   delete [] xtics_x;
 if (ytics_y)   delete [] ytics_y;
 int i;
 if (xtics_msg) {
  for (i=0; i<num_xtics_tot; i++) delete [] xtics_msg[i];
  delete [] xtics_msg;
 }
 if (ytics_msg) {
  for (i=0; i<num_ytics_tot; i++) delete [] ytics_msg[i];
  delete [] ytics_msg;
 }
}

/***************************************************************************/
/*                                                                         */
/* GetTitle: returns graph's title                                         */
/*                                                                         */
/***************************************************************************/

char*     Function::GetTitle () {
 return caption;
}

/***************************************************************************/
/*                                                                         */
/* GetGraphSize: returns page space dimensions                             */
/*                                                                         */
/***************************************************************************/

void      Function::GetGraphSize (int &xx, int &yy) {
 xx = window_ext_x;
 yy = window_ext_y;
}

/***************************************************************************/
/*                                                                         */
/* Render: display function on a DC                                        */
/*                                                                         */
/***************************************************************************/
  
void      Function::Render (CDC *ptrdc, CRect &view_rect) {
 HDC hdc = ptrdc->GetSafeHdc ();
 CPen black_pen (PS_SOLID, 0, RGB (0,0,0));

 // now set the device mapping mode for screen or printer
 ptrdc->SetMapMode (MM_ANISOTROPIC);
 ptrdc->SetWindowExt (window_ext_x, window_ext_y);
 ptrdc->SetViewportExt (view_rect.Width (), -view_rect.Height ());
 ptrdc->SetWindowOrg (window_org_x, window_org_y);
 ptrdc->SetViewportOrg (view_rect.left, view_rect.top);
   
 // if not printing, let's use some color so we can differentiate some
 // drawn items for better understanding of the coordinate mappings
 if (!ptrdc->IsPrinting ()) {
  CPen p1 (PS_SOLID, 0, RGB (255, 0, 0));
  ptrdc->SelectObject (&p1);
 }
 else ptrdc->SelectObject (&black_pen);

 // first, draw a rectangle around the complete total display area
 ptrdc->Rectangle (window_org_x, window_org_y, window_ext_x + window_org_x,
                   window_org_y - window_ext_y);
 ptrdc->SelectObject (&black_pen);

 // if using hatch shading, draw curve first and overlay with ticks and axis
 // otherwise, show ticks and then draw curve
 if (use_hatch_brush) {
  CBrush br (HS_DIAGCROSS, RGB (0,0,0));
  ptrdc->SelectObject (&br);
  ptrdc->Polygon (pts, num_pts);
 }

 // now draw the two axis
 ptrdc->MoveTo (window_org_x, 0);
 ptrdc->LineTo (window_org_x + window_ext_x, 0);
 ptrdc->MoveTo (0, window_org_y);
 ptrdc->LineTo (0, window_org_y - window_ext_y);

 // construct a reasonable scaleable font based on a point size
 // but really scaled to the drawing by the double scale
 CFont *ptrfont = new CFont ();
 if (font_height == 0) {
  double luperinch = ((double) window_ext_y) * GetDeviceCaps (hdc, LOGPIXELSY)
                     / view_rect.Height () ;
  double scale = ((double) view_rect.Height ()) / (GetDeviceCaps (hdc, LOGPIXELSY) *10);
  font_height = (int) (pointsztens * luperinch / 720 * scale);
 }
 ptrfont->CreateFont (font_height, font_height / 3, 0, 0, FW_NORMAL, 0, 0, 0, 
                  DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, 
                  PROOF_QUALITY, TMPF_TRUETYPE, "Arial");
 ptrdc->SelectObject (ptrfont);

 // obtain text metrics for text display actions
 TEXTMETRIC  tm;
 GetTextMetrics (hdc, &tm);
 int avg_char_width  = (int) tm.tmAveCharWidth;
 int avg_char_height = (int) ((tm.tmHeight + tm.tmExternalLeading));

 // display the title in the upper border zone, right or left justified
 if (title_on_right) {
  CSize sz;
  ::GetTextExtentPoint32 (hdc, title, strlen (title), &sz);
  ptrdc->TextOut (window_org_x + window_ext_x - avg_char_width - sz.cx, 
                  window_org_y - avg_char_height / 6, title);
 }
 else ptrdc->TextOut (window_org_x + avg_char_width, 
                      window_org_y - avg_char_height / 6, title);

 // display x axis ticks that are visible
 int i;
 for (i=0; i<num_xtics; i++) {
  ptrdc->MoveTo (xtics_x[i], xtics_ys);
  ptrdc->LineTo (xtics_x[i], xtics_ye);
  ptrdc->TextOut (
   xtics_x[i] - avg_char_width / 2
              - avg_char_width * (strlen (xtics_msg[i]) -1) / 2,
   xtics_ye - avg_char_height / 6, xtics_msg[i]);
 }
 
 // display all the y axis ticks that are visible
 for (i=0; i<num_ytics; i++) {
  ptrdc->MoveTo (ytics_xs, ytics_y[i]);
  ptrdc->LineTo (ytics_xe, ytics_y[i]);
  ptrdc->TextOut (ytics_xs + avg_char_width / 6,
                  ytics_y[i] + avg_char_height / 2, ytics_msg[i]);
 }

 CSize sz;
 // now display the y units scale in the lower right corner or upper right
 if (yscale_msg_up) {
  ::GetTextExtentPoint32 (hdc, yscale_msg, strlen (yscale_msg), &sz);
  ptrdc->TextOut (window_org_x + window_ext_x - avg_char_width - sz.cx, 
                  window_org_y - avg_char_height / 6 - avg_char_height,
                  yscale_msg);
 }
 else {
  ::GetTextExtentPoint32 (hdc, yscale_msg, strlen (yscale_msg), &sz);
  ptrdc->TextOut (window_org_x + window_ext_x - avg_char_width - sz.cx, 
                  window_org_y - window_ext_y + avg_char_height * 2,
                  yscale_msg);
 }

 // now draw the graph - it may overlay some ticks, axis, and text
 if (!use_hatch_brush) ptrdc->Polyline (pts, num_pts);
 delete ptrfont;
}

/***************************************************************************/
/*                                                                         */
/* ScaleYAxis: calculate uniform Y axis scale for tick marks               */
/*                                                                         */
/***************************************************************************/

 void      Function::ScaleYAxis () {
 double yrange = ymax > fabs (ymin) ? ymax : fabs (ymin);
 // extract the exponent and convert back into a power of ten
 // if range was 9000, ybase becomes 1000
 // the number of ticks would then be 9
 ybase = pow (10., log10 (yrange));
 num_ytics_tot = (int) (yrange / ybase + .5);
 // however, if the range was 1000, ybase is also 1000 with one tick
 // so if only one tick, divide into 5 2-tenths units
 istenths = false;
 if (num_ytics_tot == 1) {
  ybase /= 5;
  num_ytics_tot = 5;
  istenths = true;
 }
 // further, if the number of ticks is 0, the range is between 0 and 1
 // reduce the base by 10 and recalculate
 else if (num_ytics_tot == 0) {
  ybase /= 10;
  num_ytics_tot = (int) (yrange / ybase + .5);
 }
 num_ytics_tot *= 2;
}

// MUST derive from this abstract base class and provide for the 
// pure virtual function: Create

