#pragma once

/***************************************************************************/
/*                                                                         */
/* Function: Abstract Base Class for all four Function classes             */
/*                                                                         */
/***************************************************************************/

class Function {

 /***************************************************************************/
 /*                                                                         */
 /* Class Data:                                                             */
 /*                                                                         */
 /***************************************************************************/

public:

char    caption[80];     // the graph's title to be used in the caption
char    title[80];       // the graph's title to be used in the metafile
bool    title_on_right;  // when TRUE, right justify title
bool    use_hatch_brush; // when TRUE, hatch brush highlights area under curve

double  parm1;           // user range variable 1
double  parm2;           // user range variable 2, if any

int     num_pts;         // the total number of points to plot

// world space values
double  *x;              // array of plotted x values in world space
double  *y;              // array of plotted y values in world space

double  ymax;            // y max value in world space
double  ymin;            // y min value in world space
double  xs;              // x start or min value
double  xe;              // x end or max value

double  dx;              // x axis increment between x points

// page space definition values
int     max_quad_size;   // full size quadrant
int     draw_quad_size;  // drawing's reduced size
int     border_size;     // border size on all sides
double  xscale;          // scaling factors to convert from world to page space
double  yscale;     

CPoint *pts;             // the graph of plotted points in page space
int     xpmin;           // the x,y min and max plotted values in page space
int     xpmax;
int     ypmin;
int     ypmax;

// device space conversion values
int     window_ext_x; // the window's extent in page space
int     window_ext_y;
int     window_org_x; // the window origin - that point in page space
int     window_org_y; // that is to map to the upper left corner of device space

bool    xvisible;     // TRUE when the x axis is visible in device space
bool    yvisible;     // TRUE when the y axis is visible in device space

// font data
int     font_height;  // rendered font height
int     pointsztens;  // point size in tens for titles

// x axis definition values in page space
int     ticksize;     // length/width of ticks
int     num_xtics_tot;// array size for ticks, some might be clipped
int     num_xtics;    // actual number of ticks on x axis to draw
int    *xtics_x;      // array of x axis tick points
char  **xtics_msg;    // caption for each tick
int     xtics_ys;     // x axis ticks start y position
int     xtics_ye;     // x axis ticks end y position

// y axis definition values in page space
double  ybase;        // y axis base values for ticks
bool    istenths;     // TRUE when using tenths units for scale
bool    yscale_msg_up;// TRUE when y scale message in in upper rt corner
char    yscale_msg[50]; // the scale on the y axis message
int     num_ytics_tot;// array size for ticks, some might be clipped
int     num_ytics;    // actual number of ticks on y axis to draw
int    *ytics_y;      // array of y axis tick points
char  **ytics_msg;    // caption for each tick
int     ytics_xs;     // y axis ticks start x position
int     ytics_xe;     // y axis ticks end x position

 /***************************************************************************/
 /*                                                                         */
 /* Class Functions:                                                        */
 /*                                                                         */
 /***************************************************************************/

public:
              Function (int, double&, double&); // set defaults
virtual      ~Function ();               // deletes the arrays
virtual char* GetTitle ();               // returns the graph's title
virtual void  GetGraphSize (int&, int&); // return graph's page space size

virtual bool  Create () = 0;             // calculates the plot
virtual void  Render (CDC*, CRect&);     // draws the plot
virtual void  ScaleYAxis ();             // calc Y axis scale for tick marks
};

const int YSTRLEN = 10; // length of y axis messages
