#include "stdafx.h"
#include "Pgm12a.h"
#include "GraphDoc.h"
#include "ChooseFunction.h"
#include "ChooseResolutionDlg.h"
#include "RadiationFunction.h"
#include "PlaneFunction.h"
#include "SpringFunction.h"
#include "SincFunction.h"
#include "GetAnglesDlg.h"
#include "GetPlaneDlg.h"
#include "GetSpringDlg.h"
#include "GetTempDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(GraphDoc, CDocument)

BEGIN_MESSAGE_MAP(GraphDoc, CDocument)
END_MESSAGE_MAP()

/***************************************************************************/
/*                                                                         */
/* GraphDoc: set initial defaults                                          */
/*                                                                         */
/***************************************************************************/

GraphDoc::GraphDoc () {
 ptrfn = NULL;
 ptrmargins = NULL;
 is_open_title_call = false;
 hMetafile = 0;
}

/***************************************************************************/
/*                                                                         */
/* ~GraphDoc: remove allocated objects                                     */
/*                                                                         */
/***************************************************************************/

GraphDoc::~GraphDoc () {
 if (ptrfn) delete ptrfn;
 if (hMetafile) ::DeleteEnhMetaFile (hMetafile);
 if (ptrmargins) delete ptrmargins;
}

/***************************************************************************/
/*                                                                         */
/* DeleteContents: remove all allocated objects                            */
/*                                                                         */
/***************************************************************************/

void      GraphDoc::DeleteContents () {
 if (ptrfn) delete ptrfn;
 if (hMetafile) ::DeleteEnhMetaFile (hMetafile);
 if (ptrmargins) delete ptrmargins;
 hMetafile = 0;
 ptrfn = NULL;
 ptrmargins = NULL;
 if (isnew) DeleteFile (tempname);
	CDocument::DeleteContents();
}

/***************************************************************************/
/*                                                                         */
/* SetMargins: copy our saved margins to PageSetup Dlg                     */
/*                                                                         */
/***************************************************************************/

void      GraphDoc::SetMargins (PAGESETUPDLG *ptrpd) {
 if (ptrmargins) ptrpd->rtMargin = *ptrmargins;
}

/***************************************************************************/
/*                                                                         */
/* GetMargins: returns the margins                                         */
/*                                                                         */
/***************************************************************************/

RECT*     GraphDoc::GetMargins () {
 return ptrmargins;
}

/***************************************************************************/
/*                                                                         */
/* SaveMargins: save page setup margins                                    */
/*                                                                         */
/***************************************************************************/

void      GraphDoc::SaveMargins (RECT *ptrrect) {
 if (ptrmargins) delete ptrmargins;
 ptrmargins = new RECT;
 *ptrmargins = *ptrrect;
}

/***************************************************************************/
/*                                                                         */
/* OnNewDocument: Construct a new graph from user's choice                 */
/*                                                                         */
/***************************************************************************/

BOOL      GraphDoc::OnNewDocument () {
 DeleteContents ();
 isnew              = true;  // indicate new file, create as a temporary file
 use_metafile_only  = false; // indicate graph function can render
 metafile_available = false; // indocate no metafile exists yet

 which_graph = 0;            // set the default graph type
 num_pts = 1000;             // number of points to plot
 ptrfn = NULL;               // no graph class instance yet

 int resolution = 0;         // set for Page Space huge resolution
 // get user's choice of resolution with which to construct the metafile
 ChooseResolutionDlg cdlg (NULL, &resolution);
 cdlg.DoModal ();

 // get user's choice of function
 ChooseFunction dlg (NULL, &which_graph);
 if (dlg.DoModal () == IDOK) {
  // have a choice, so construct a document
  if (!CDocument::OnNewDocument()) return FALSE;
  if (which_graph == 0) { // car shock absorber case
   parm1 = 3000.0;        // set dafault car weight in pounds
   parm2 = 10.;           // set default depth of pot hole in inches
   // get user's choices for weight and depth
   GetSpringDlg dlg (NULL, &parm1, &parm2);
   if (dlg.DoModal () == IDOK) {
    // got user's choices, so allocate the spring instance
    ptrfn = new SpringFunction (num_pts, parm1, parm2);
    if (!ptrfn) return FALSE;
   }
   else return FALSE;
  }
  else if (which_graph == 1) { // plane lift pressure case
   parm1 = 8.0;                // set default wing width in feet
   parm2 = 40.0;               // set default wing length in feet
   // get user's wing dimension choices
   GetPlaneDlg dlg (NULL, &parm1, &parm2);
   if (dlg.DoModal () == IDOK) {
    // got the user's choices, so allocate the plane instance
    ptrfn = new PlaneFunction (num_pts, parm1, parm2);
    if (!ptrfn) return FALSE;
   }
   else return FALSE;
  }
  else if (which_graph == 2) { // blackbody radiation case
   parm1 = 2000.0;             // set default temperature in Celsius
   // get user's temperature
   GetTempDlg dlg (NULL, &parm1);
   if (dlg.DoModal () == IDOK) {
    // got the user's choice, so allocate the blackbody instance
    ptrfn = new RadiationFunction (num_pts, parm1, parm2);
    if (!ptrfn) return FALSE;
   }
   else return FALSE;
  }
  else {            // the sinc function case
   parm1 = -6.0;    // set default angle range in PI radian units
   parm2 = +6.0;
   GetAnglesDlg dlg (NULL, &parm1, &parm2);
   if (dlg.DoModal () == IDOK) {
    // got user's choices, so allocate the sinc instance
    ptrfn = new SincFunction (num_pts, parm1, parm2);
    if (!ptrfn) return FALSE;
   }
   else return FALSE;
  }

  // here one of the four is allocated, so actually go create the plot
  if (!ptrfn->Create ()) {
   delete ptrfn;  // plot failed, so remove attempt
   ptrfn = NULL;
   return FALSE;  // set doc new failed
  }
  // install the graph's title
  SetTitle (ptrfn->GetTitle ());

  // make reference DC for Enhanced Metafile creation to model
  // and fill in the two rects defining the metafile mapping
  CDC dcref;
  CRect rclFrame (0, 0, 0, 0);  // the rclFrame for CreateEnhanced
  CRect viewFrame (0, 0, 0, 0); // the viewport rect for render to metafile

  static char   printer[80];    // for printer DC acquisition
  char         *device, *driver, * output, *next;

  if (resolution == 0) {      // use full Page Space for the resolution
   // first find the highest resolution device - default printer or screen
   bool use_printer = TRUE;   // assume it is the printer
   // get access to a printer DC from the ini file installed printer
   GetProfileString ("windows", "device", "...", printer, 80);
   if ((device = strtok_s (printer, ",", &next)) != NULL &&
       (driver = strtok_s (NULL,    ",", &next)) != NULL &&
       (output = strtok_s (NULL,    ",", &next)) != NULL) {
    // a default printer exists, so get a DC for it
    CDC *ptrprdc = new CDC;
    ptrprdc->CreateDC (driver, device, output, NULL);
    CDC *ptrscrdc = new CDC;
    ptrscrdc->CreateDC ("DISPLAY", NULL, NULL, NULL);
    if (ptrprdc->GetDeviceCaps (LOGPIXELSX) < 
        ptrscrdc->GetDeviceCaps (LOGPIXELSX)) use_printer = false;
    delete ptrprdc;
    delete ptrscrdc;
   }
   else use_printer = false;
   if (use_printer) dcref.CreateDC (driver, device, output, NULL);
   else dcref.CreateDC ("DISPLAY", NULL, NULL, NULL);
   // construct total graph bounding rectangle in HIMETRIC .01 mm units
   // by getting the graph's width and height in pixels and converting
   int w, h;
   ptrfn->GetGraphSize (w, h);
   CSize sz (w, h);
   dcref.DPtoHIMETRIC (&sz);   // convert to .01 mm units
   rclFrame.right   = sz.cx;   // bounding rect in .01 mm units
   rclFrame.bottom  = sz.cy;
   viewFrame.right  = w;       // render viewport in pixels 
   viewFrame.bottom = h;
  }
  else if (resolution == 1) { // use the default printer as a model
   // get access to a printer DC from the ini file installed printer
   GetProfileString ("windows", "device", "...", printer, 80);
   if ((device = strtok_s (printer, ",", &next)) != NULL &&
       (driver = strtok_s (NULL,    ",", &next)) != NULL &&
       (output = strtok_s (NULL,    ",", &next)) != NULL) 
    // a default printer exists, so get a DC for it
    dcref.CreateDC (driver, device, output, NULL);
   else dcref.CreateDC ("DISPLAY", NULL, NULL, NULL);
   rclFrame.right   = dcref.GetDeviceCaps (HORZSIZE) * 100;
   rclFrame.bottom  = dcref.GetDeviceCaps (VERTSIZE) * 100;
   viewFrame.right  = dcref.GetDeviceCaps (HORZRES);
   viewFrame.bottom = dcref.GetDeviceCaps (VERTRES);
  }
  else {
   dcref.CreateDC ("DISPLAY", NULL, NULL, NULL);
   rclFrame.right   = dcref.GetDeviceCaps (HORZSIZE) * 100;
   rclFrame.bottom  = dcref.GetDeviceCaps (VERTSIZE) * 100;
   viewFrame.right  = dcref.GetDeviceCaps (HORZRES);
   viewFrame.bottom = dcref.GetDeviceCaps (VERTRES);
  }

  // make the description string: file\0descr\0\0 format
  char desc[_MAX_PATH*2];
  strcpy_s (desc, sizeof(desc), "\\WindowsMFCII\\Pgm12a\\Debug\\Pgm12a.exe");
  int i = strlen (desc);
  char des[100];
  strcpy_s (des, sizeof(des), ptrfn->GetTitle ());
  strcpy_s (&desc[strlen (desc) + 1], sizeof(des)-1, des);
  desc[i + 1 + strlen(des) + 1]=0;
 
  CString emfsPath = ((Pgm12aApp*) AfxGetApp ())->RetrievePath ();
  // have Windows build a unique temporary file name
  // if it is not saved, when the doc is closed, the temp file is deleted
  ::GetTempFileName(emfsPath, "TMP", 0, tempname);

  // build the metafile based on the ref dc and the bound rect in .01mm units
  if (dc.CreateEnhanced (&dcref, tempname, rclFrame, desc)) {
   // to draw on it, must use a viewport size of the bound rect in pixels
   ptrfn->Render (&dc, viewFrame);  // draw the graph on the metafile
   hMetafile = dc.CloseEnhanced (); // finish drawing, get a handle to it
   metafile_available = true;       // flag as usable if desired
   SetModifiedFlag (TRUE);          // indicate must be saved
  }
  else {
   MessageBeep (0);
   AfxMessageBox ("Can't create metafile", MB_OK | MB_ICONSTOP);
   ::DeleteFile (tempname);
  }
  return TRUE;
 }
 return FALSE;
}

/***************************************************************************/
/*                                                                         */
/* OnOpenDocument: open ANY .EMF file as the document                      */
/*                                                                         */
/***************************************************************************/

BOOL      GraphDoc::OnOpenDocument (LPCTSTR lpszPathName) {
 DeleteContents ();            // remove any existing items - if used in SDI
 isnew = FALSE;                // set not a new document
 metafile_available = false;   // assume the worst and assume not valid emf
 use_metafile_only = true;     // set must use the metafile - no graph object
 is_open_title_call = true;    // handle extra set title call from framework
 num_calls = 0;
 
 // attempt to open the metafile
 hMetafile = ::GetEnhMetaFile (lpszPathName);
 if (hMetafile) {
  // it exists, so flag as available for use
  metafile_available = true;
  strcpy_s (tempname, sizeof(tempname), lpszPathName); // save the filename
  SetModifiedFlag (FALSE);         // indicate not altered
  // now see if this is one of our 4 types of graphs
  // by retrieving the documentation string, if any is present
  char desc[200];
  int len = GetEnhMetaFileDescription (hMetafile, 200, desc);
  if (len > 0 ) {
   // a description string is present
   // skip over the program string and use the description portion if it's there
   int i = strlen (desc);
   if (len > i + 1) SetTitle (&desc[i+1]);
   else SetTitle (desc); // not there, so use application name
  }
  else SetTitle (lpszPathName); // nothing present, so use filename
  return TRUE;
 }
 else {
  MessageBeep (0);
  AfxMessageBox ("Unable to open requested metafile", MB_OK | MB_ICONSTOP);
  return FALSE;
 }
}

/***************************************************************************/
/*                                                                         */
/* SetTitle: override to prevent framework's second call to set filename   */
/*           as the title; instead use our names as titles                 */
/*                                                                         */
/***************************************************************************/

void      GraphDoc::SetTitle(LPCTSTR lpszTitle) {
 if (is_open_title_call) {
  num_calls ++;
  // only prevent the second call by the framework from replacing our title
  if (num_calls != 2) CDocument::SetTitle(lpszTitle);
 }
 else CDocument::SetTitle(lpszTitle);
}

/***************************************************************************/
/*                                                                         */
/* OnSaveDocument: save document by copying the metafile                   */
/*                                                                         */
/***************************************************************************/

BOOL      GraphDoc::OnSaveDocument (LPCTSTR lpszPathName) {
 BOOL ok = CDocument::OnSaveDocument(lpszPathName);
 if (metafile_available) {
  CopyFile (tempname, lpszPathName, FALSE);
  SetTitle (lpszPathName);
 }
 else {
  MessageBeep (0);
  AfxMessageBox ("No metafile exists to save", MB_OK | MB_ICONSTOP);
 }
 return ok;
}

#ifdef _DEBUG
void      GraphDoc::AssertValid () const { CDocument::AssertValid(); }
void      GraphDoc::Dump (CDumpContext& dc) const { CDocument::Dump(dc); }
#endif //_DEBUG
