#include "stdafx.h"
#include "Pgm12a.h"
#include "GraphDoc.h"
#include "GraphView.h"
#include "Function.h"
#include "GraphSizeDlg.h"
#include "ViewHeaderDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(GraphView, CView)

BEGIN_MESSAGE_MAP(GraphView, CView)
 ON_COMMAND(CM_PAGESETUP,               OnFilePageSetup)
	ON_COMMAND(CM_VIEW_FROM_EMF,           OnViewFromEmf)
	ON_UPDATE_COMMAND_UI(CM_VIEW_FROM_EMF, OnUpdateViewFromEmf)
	ON_COMMAND(CM_SET_GRAPH_SIZE,          OnSetGraphSize)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_UPDATE_COMMAND_UI(CM_VIEWHEADER, OnUpdateViewHeader)
	ON_COMMAND(CM_VIEWHEADER, OnViewHeader)
	ON_COMMAND(ID_FILE_PRINT,              CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_DIRECT,       CView::OnFilePrint)
	ON_COMMAND(ID_FILE_PRINT_PREVIEW,      CView::OnFilePrintPreview)
END_MESSAGE_MAP()

/***************************************************************************/
/*                                                                         */
/* GraphView: initialization                                               */
/*                                                                         */
/***************************************************************************/

GraphView::GraphView () {
 useMetafile = false;
 ptrgraph_rect = NULL;
 dragging = false;
}

/***************************************************************************/
/*                                                                         */
/* ~GraphView: remove any graph size rectangle in use                      */
/*                                                                         */
/***************************************************************************/

GraphView::~GraphView () {
 if (ptrgraph_rect) delete ptrgraph_rect;
}

/***************************************************************************/
/*                                                                         */
/* OnDraw: display graph from its Render or from EMF directly              */
/*                                                                         */
/***************************************************************************/

void      GraphView::OnDraw (CDC* pDC) {
 GraphDoc* pDoc = GetDocument();
 ASSERT_VALID(pDoc);
 // set up the drawing rect when not printing by using client area or graph size
 if (!pDC->IsPrinting ())
  if (!ptrgraph_rect) GetClientRect (draw_rect);
  else draw_rect = *ptrgraph_rect;

 if (!pDoc->use_metafile_only && !useMetafile && pDoc->ptrfn)
  pDoc->ptrfn->Render (pDC, draw_rect);
 else if (pDoc->metafile_available)
  pDC->PlayMetaFile (pDoc->hMetafile, &draw_rect);
}

/***************************************************************************/
/*                                                                         */
/* OnPreparePrinting: set the page numbers to 1 for the Print dlg and prevw*/
/*                                                                         */
/***************************************************************************/

BOOL      GraphView::OnPreparePrinting (CPrintInfo* pInfo) {
 if (pInfo->m_bPreview) pInfo->m_nNumPreviewPages = 1;
 pInfo->SetMaxPage (1);
	return DoPreparePrinting(pInfo);
}

/***************************************************************************/
/*                                                                         */
/* OnPrint: apply any user margins to the page rectangle                   */
/*                                                                         */
/***************************************************************************/

void      GraphView::OnPrint (CDC* pDC, CPrintInfo* pInfo) {
 OnPrepareDC (pDC, pInfo);        // fix up the DC for rendering

 // retrieve any user margins
 RECT *ptrmargins = GetDocument ()->GetMargins ();

 // construct the working rectangle by applying margins to full page rect
 draw_rect = pInfo->m_rectDraw;

 // apply margins, if present
 if (ptrmargins) {
  // retrieve the printer's pixels per inch for scaling margins
  int logpixelsx = pDC->GetDeviceCaps (LOGPIXELSX);
  int logpixelsy = pDC->GetDeviceCaps (LOGPIXELSX);
  // I am ignoring the physical margins effect - see Pgm06a
  // scale and add in margins top form final drawing rectangle
  draw_rect.left   += MulDiv (ptrmargins->left,   logpixelsx, 1000);
  draw_rect.right  -= MulDiv (ptrmargins->right,  logpixelsx, 1000);
  draw_rect.top    += MulDiv (ptrmargins->top,    logpixelsx, 1000);
  draw_rect.bottom -= MulDiv (ptrmargins->bottom, logpixelsx, 1000);
 }
 CView::OnPrint(pDC, pInfo);      // go print the page
}

/***************************************************************************/
/*                                                                         */
/* OnFilePageSetup: setup the page margins for printing                    */
/*                                                                         */
/***************************************************************************/

void      GraphView::OnFilePageSetup () {
 CPageSetupDialog pd (PSD_DEFAULTMINMARGINS | PSD_MARGINS | 
                      PSD_INTHOUSANDTHSOFINCHES, this);
 GetDocument ()->SetMargins (&pd.m_psd);            // install saved margins
 if (pd.DoModal () == IDOK)                         // get user choices
  ((Pgm12aApp*)AfxGetApp ())->SavePrinterDefaults (
                                 pd.m_psd.hDevMode, pd.m_psd.hDevNames);
  GetDocument ()->SaveMargins (&pd.m_psd.rtMargin); // save user margins
}

/***************************************************************************/
/*                                                                         */
/* OnViewFromEmf: toggle between viewing from EMF and from Render          */
/*                                                                         */
/***************************************************************************/

void      GraphView::OnViewFromEmf () {
 GraphDoc* pDoc = GetDocument();
 ASSERT_VALID(pDoc);
	if (!pDoc->use_metafile_only && pDoc->metafile_available) {
  useMetafile = !useMetafile;
  Invalidate ();
 }
}

/***************************************************************************/
/*                                                                         */
/* OnUpdateViewFromEmf: enable or checkmark View from EMF menu option      */
/*                                                                         */
/***************************************************************************/

void      GraphView::OnUpdateViewFromEmf (CCmdUI* pCmdUI) {
 GraphDoc* pDoc = GetDocument();
 ASSERT_VALID(pDoc);
 if (pDoc->use_metafile_only || !pDoc->metafile_available)
  pCmdUI->Enable (FALSE);
 else
  pCmdUI->SetCheck (useMetafile);
}

/***************************************************************************/
/*                                                                         */
/* OnSetGraphSize: get user's choice for display graph size                */
/*                                                                         */
/***************************************************************************/

void      GraphView::OnSetGraphSize () {
 CClientDC dc (this);
 // get the pixels per inch
 double logpixelsx = dc.GetDeviceCaps (LOGPIXELSX);
 double logpixelsy = dc.GetDeviceCaps (LOGPIXELSY);
 // get the current window size and convert to inches
 GetClientRect (draw_rect);
 GRAPHSIZE gsz;
 char msg[20];
 sprintf_s (msg, sizeof(msg), "%.2f", draw_rect.Width () / logpixelsx);
 gsz.scrn_width   = msg;
 sprintf_s (msg, sizeof(msg), "%.2f", draw_rect.Height () / logpixelsy);
 gsz.scrn_height  = msg;
 gsz.revert_screen = FALSE;
 // if there is a previous graph size, use its values; otherwise use window size
 if (ptrgraph_rect) {
  gsz.graph_width   = ptrgraph_rect->Width ()  / logpixelsx;
  gsz.graph_height  = ptrgraph_rect->Height () / logpixelsy;
 }
 else {
  gsz.graph_width  = draw_rect.Width () / logpixelsx;
  gsz.graph_height = draw_rect.Height () / logpixelsy;
 }
	GraphSizeDlg dlg (this, &gsz);
 if (dlg.DoModal () == IDOK) {
  // implement user's choices
  if (!ptrgraph_rect) ptrgraph_rect = new CRect (0,0,0,0);
  ptrgraph_rect->right  = ptrgraph_rect->left + (int) (gsz.graph_width * logpixelsx);
  ptrgraph_rect->bottom = ptrgraph_rect->top +(int) (gsz.graph_height * logpixelsy);
  // if revert is requested, delete the graph size rectangle so graph follows
  // the window resizing operations once more
  if (gsz.revert_screen) {
   delete ptrgraph_rect;
   ptrgraph_rect = NULL;
  }
  Invalidate ();
 }
}

/***************************************************************************/
/*                                                                         */
/* OnLButtonDown: begin user dragging of the graph around the client area  */
/*                                                                         */
/***************************************************************************/

void      GraphView::OnLButtonDown (UINT nFlags, CPoint point) {
 CView::OnLButtonDown(nFlags, point);
	if (ptrgraph_rect && !dragging) {
  dragging = true;  // enable dragging
  init_pt = point;  // save current mouse location
 }
}

/***************************************************************************/
/*                                                                         */
/* OnLButtonUp: halt moving the graph around the client area               */
/*                                                                         */
/***************************************************************************/

void      GraphView::OnLButtonUp (UINT nFlags, CPoint point) {
	dragging = false;
 CView::OnLButtonUp(nFlags, point);
}

/***************************************************************************/
/*                                                                         */
/* OnMouseMove: move graph image to new location within client window      */
/*                                                                         */
/***************************************************************************/

void      GraphView::OnMouseMove (UINT nFlags, CPoint point) {
 CView::OnMouseMove(nFlags, point);
	if (ptrgraph_rect && dragging) {
  CPoint pt (point - init_pt);    // calculate the offset for this movement
  ptrgraph_rect->OffsetRect (pt); // offset the drawing rect this amount
  init_pt = point;                // save current mouse location for next move
  Invalidate ();                  // force painting at this spot
 }
}

/***************************************************************************/
/*                                                                         */
/* OnUpdateViewHeader: enable view enhanced metafile header menuitem       */
/*                                                                         */
/***************************************************************************/

void      GraphView::OnUpdateViewHeader (CCmdUI* pCmdUI) {
 GraphDoc* pDoc = GetDocument();
 ASSERT_VALID(pDoc);
 pCmdUI->Enable (pDoc->metafile_available);
}

/***************************************************************************/
/*                                                                         */
/* OnViewHeader: view the contents of the metafile header record           */
/*                                                                         */
/***************************************************************************/

void      GraphView::OnViewHeader () {
 GraphDoc* pDoc = GetDocument();
 ASSERT_VALID(pDoc);
 if (!pDoc->metafile_available) return; // avoid doing nothing
 ENHMETAHEADER eh;
 if (GetEnhMetaFileHeader (pDoc->hMetafile, sizeof (eh), &eh) == 0) return;
 char title[256];
 char desc[256];
 int len = GetEnhMetaFileDescription (pDoc->hMetafile, sizeof(title), title);
 if (len > 0 ) { // a description string is present
  // skip over the program string and use the description portion if it's there
  int i = strlen (title);
  if (len > i + 1) strcpy_s (desc, sizeof(desc), &title[i+1]);
  else strcpy_s (desc, sizeof(desc), "No Description");
 }
 else {
  strcpy_s (desc, sizeof(desc), "No description present");
  strcpy_s (title, sizeof(title), "No title present");
 }
 ViewHeaderDlg dlg (this, &eh, title, desc);
 dlg.DoModal ();
}

#ifdef _DEBUG
void      GraphView::AssertValid () const { CView::AssertValid(); }
void      GraphView::Dump (CDumpContext& dc) const { CView::Dump(dc); }
GraphDoc* GraphView::GetDocument () { // non-debug version is inline
 ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(GraphDoc)));
 return (GraphDoc*)m_pDocument;
}
#endif //_DEBUG
