#include "stdafx.h"
#include "Pgm12a.h"
#include "MainFrm.h"
#include "ChildFrm.h"
#include "GraphDoc.h"
#include "GraphView.h"
#include <direct.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

BEGIN_MESSAGE_MAP(Pgm12aApp, CWinApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
	ON_COMMAND(ID_FILE_PRINT_SETUP, CWinApp::OnFilePrintSetup)
END_MESSAGE_MAP()

Pgm12aApp::Pgm12aApp() { }

Pgm12aApp theApp;

BOOL Pgm12aApp::InitInstance() {
 SetRegistryKey ("WindowsMFCII"); // setup the higher level reg key
	LoadStdProfileSettings(10);  // Load standard INI file options (including MRU)

	CMultiDocTemplate* pDocTemplate;
	pDocTemplate = new CMultiDocTemplate(
		IDR_GRAPHTYPE,
		RUNTIME_CLASS(GraphDoc),
		RUNTIME_CLASS(ChildFrame), // custom MDI child frame
		RUNTIME_CLASS(GraphView));
	AddDocTemplate(pDocTemplate);

	// create main MDI Frame window
	MainFrame* pMainFrame = new MainFrame;
	if (!pMainFrame->LoadFrame(IDR_MAINFRAME))
		return FALSE;
	m_pMainWnd = pMainFrame;

 // find and/or make the temp folder for emf files
 CString emfs = LocateEmfsFolder ();
 SavePath (emfs);
 
	// Enable drag/drop open
	m_pMainWnd->DragAcceptFiles();

	// Enable DDE Execute open
	EnableShellOpen();
	RegisterShellFileTypes(TRUE);

	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	// Dispatch commands specified on the command line
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;
	pMainFrame->ShowWindow(SW_SHOWMAXIMIZED);
	pMainFrame->UpdateWindow();

	return TRUE;
}

/***************************************************************************/
/*                                                                         */
/* LocateEmfsFolder: locate the Emfs folder                                */
/*                                                                         */
/***************************************************************************/

CString   Pgm12aApp::LocateEmfsFolder () {
 char drive[_MAX_DRIVE];
 char dir[_MAX_PATH];
 char filen[_MAX_PATH];
 char ext[_MAX_EXT];
 char cl[_MAX_PATH];
 char cl2[_MAX_PATH];
 // m_pszHelpFilePath has the full path to where the .hlp file might be
 // if running from the ide, may have the Debug or Release folder as part
 // of the path string, so remove that portion
 strcpy_s (cl, _MAX_PATH, theApp.m_pszHelpFilePath);
 _splitpath_s (cl, drive, _MAX_DRIVE, dir, _MAX_PATH, filen, _MAX_PATH, ext, _MAX_EXT);
 strcpy_s (cl2, _MAX_PATH, dir);
 CString s = cl2;
 s.MakeUpper ();
 int i = s.Find ("DEBUG\\");
 if (i != -1) cl2[i] = 0;
 else i = s.Find ("RELEASE\\");
 if (i != -1) cl2[i] = 0;
 CString filename = drive;
 filename += cl2;
 filename += "Emfs";
 // attempt to make this directory
 if (_mkdir (filename) == -1) { // try to make Emfs folder
  if (errno != EEXIST) {        // failed because it exists? then is ok too
   AfxMessageBox (
    "Error: The Emfs folder cannot be created. Cannot make temporary metafiles",
    MB_ICONSTOP | MB_OK);
   filename = "";
   return filename;
  }
 }
 filename += "\\";  
 return filename;
}

/***************************************************************************/
/*                                                                         */
/* RetrievePath: obtain the installed path from the system registry        */
/*                                                                         */
/***************************************************************************/

CString   Pgm12aApp::RetrievePath () {
 CString subkey = "SOFTWARE\\WindowsMFCII\\Pgm12a\\Settings";
 CString entry = "Emfs";
 CString filename = "";
 HKEY hkey;
 LONG ok;
 // attempt to make the Settings key, if it is not there yet
 ok = RegCreateKeyEx (HKEY_CURRENT_USER, subkey, 0, 0, 0, KEY_CREATE_SUB_KEY, 0, &hkey, 0);
 if (ok != ERROR_SUCCESS) return filename;

 // attempt to open the current user Settings key with our value
 ok = RegOpenKeyEx (HKEY_CURRENT_USER, subkey, 0, KEY_READ, &hkey);
 if (ok == ERROR_SUCCESS) {
  DWORD dwType, dwCount;
  // key exists, now verify BthFile entry is a string
  ok = RegQueryValueEx (hkey, entry, NULL, &dwType, NULL, &dwCount);
  if (ok == ERROR_SUCCESS && dwType == REG_SZ) {
   // now retrieve the actual path string
			ok = RegQueryValueEx (hkey, entry, NULL, &dwType,
                         (LPBYTE) filename.GetBuffer (dwCount/sizeof(TCHAR)),
                         &dwCount);
			filename.ReleaseBuffer(); // release the CString buffer locked by GetBuffer
  }
  RegCloseKey (hkey);
 }
 return filename;
}

/***************************************************************************/
/*                                                                         */
/* SavePath: save path of Emfs temp files in registry                      */
/*                                                                         */
/***************************************************************************/

bool   Pgm12aApp::SavePath (CString& filename) {
 CString subkey = "SOFTWARE\\WindowsMFCII\\Pgm12a\\Settings";
 CString entry = "Emfs";
 HKEY hkey;
 LONG ok;
 // build Settings key, if it is not there yet
 ok = RegCreateKeyEx (HKEY_CURRENT_USER, subkey, 0, 0, 0, KEY_CREATE_SUB_KEY, 0, &hkey, 0);
 if (ok != ERROR_SUCCESS) return false;
 // attempt to open the current user Settings key with our value
 ok = RegOpenKeyEx (HKEY_CURRENT_USER, subkey, 0, KEY_WRITE, &hkey);
 if (ok == ERROR_SUCCESS) {
  // key exists, now store the value in BthFile
  ok = RegSetValueEx (hkey, entry, 0, REG_SZ, (const BYTE*) filename.GetBuffer(), filename.GetLength ()+1);
  if (ok == ERROR_SUCCESS) {
			filename.ReleaseBuffer(); // release the CString buffer locked by GetBuffer
   RegCloseKey (hkey);
   return true;
  }
  RegCloseKey (hkey);
 }
 return false;
}

/***************************************************************************/
/*                                                                         */
/* SavePrinterDefaults: saves printer setup results                        */
/*                                                                         */
/***************************************************************************/

void Pgm12aApp::SavePrinterDefaults (HGLOBAL hdevmode, HGLOBAL hdevnames) {
 // remove previous copies, if any
 if (m_hDevNames) GlobalFree (m_hDevNames);
 if (m_hDevMode) GlobalFree (m_hDevMode);

 // lock the new instances from the document results
 LPDEVNAMES ptrnewdevnames = (LPDEVNAMES) GlobalLock (hdevnames);
 LPDEVMODE  ptrnewdevmode  = (LPDEVMODE)  GlobalLock (hdevmode);
 // allocate and lock our copies of the new results
 m_hDevNames = GlobalAlloc (GPTR, GlobalSize (hdevnames));
 m_hDevMode  = GlobalAlloc (GPTR, GlobalSize (hdevmode));
 LPDEVNAMES ptrdevnames = (LPDEVNAMES) GlobalLock (m_hDevNames);
 LPDEVMODE  ptrdevmode  = (LPDEVMODE)  GlobalLock (m_hDevMode);
 // copy user selections into our copies
 memcpy (ptrdevnames, ptrnewdevnames, (size_t) GlobalSize (hdevnames));
 memcpy (ptrdevmode,  ptrnewdevmode,  (size_t) GlobalSize (hdevmode));
 GlobalUnlock (ptrdevmode);
 GlobalUnlock (ptrnewdevmode);
 GlobalUnlock (ptrdevnames);
 GlobalUnlock (ptrnewdevnames);
}

class CAboutDlg : public CDialog {
public:
	CAboutDlg();
	enum { IDD = IDD_ABOUTBOX };
protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD) { }

void CAboutDlg::DoDataExchange(CDataExchange* pDX) {
	CDialog::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
END_MESSAGE_MAP()

void Pgm12aApp::OnAppAbout() {
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}
