#include "stdafx.h"
#include "Pgm12a.h"
#include "SincFunction.h"
#include <cmath>
using namespace std;

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/***************************************************************************/
/*                                                                         */
/* SincFunction: Plot sinc function over user angle range                  */
/*                                                                         */
/***************************************************************************/

SincFunction::SincFunction (int num, double &p1, double &p2) 
             : Function (num, p1, p2) {
 sprintf_s (caption, sizeof(caption),
           "Sinc Function Evaluated from %.1f PI to %.1f PI",
          p1, p2);
 pointsztens = 200;
 strcpy_s (title, sizeof(title), 
              "The sinc Math Function versus Angle (in radians)");
}

/***************************************************************************/
/*                                                                         */
/* Create: construct the plot                                              */
/*                                                                         */
/***************************************************************************/
  
bool      SincFunction::Create () {
 try {
  x = new double [num_pts];    // x,y are world space coords
  y = new double [num_pts];   
  pts = new CPoint [num_pts];  // the page space coords
 }
 catch (...) {
  MessageBeep (0);
  AfxMessageBox ("Out of Memory", MB_OK | MB_ICONSTOP);
  return false;
 }

 double pi = acos (-1.);
 xs = parm1 * pi;
 xe = parm2 * pi;

 // if using hatch brush, reserve 2 points for polygon closure
 int numpts = num_pts;
 if (use_hatch_brush) numpts = num_pts - 2;
 dx = (xe - xs) / (numpts - 1);
 
 int i;
 // construct the graph in world coordinates
 for (i=0; i<numpts; i++) {
  x[i] = xs + dx * i;
  if (fabs (x[i]) <= .00001) y[i] = 0.; // at 0, avoid dividing by 0
  else y[i] = sin (x[i]) / x[i];
  // find the max and min values as points are created
  if (i) {
   if (y[i] > ymax) ymax = y[i];
   if (y[i] < ymin) ymin = y[i];
  }
  else {
   ymax = y[i];
   ymin = y[i];
  }
 }

 // if using hatch brush, force curve polygon closure
 if (use_hatch_brush) {
  i = numpts;
  x[i] = x[i-1];
  y[i] = 0;
  i++;
  x[i] = x[0];
  y[i] = 0;
 }

 double ysz, xsz;

 // scale image in world space coords to page space coords of draw_quad_size
 // so that the graph always is a reasonable physical size
 // by using the larger quadrant extent should the image lie in two quadrants
 if (ymax > 0)
  if (fabs (ymin) > ymax) ysz = fabs (ymin);
  else ysz = ymax;
 else ysz = fabs (ymin);
 yscale = draw_quad_size / ysz;
 
 if (xe > 0)
  if (fabs (xs) > xe) xsz = fabs (xs);
  else xsz = xe;
 else xsz = fabs (xs);
 xscale = draw_quad_size / xsz;

 // scale all plotted points to get from world space to page space
 for (i=0; i<num_pts; i++) {
  pts[i].x = (int) (x[i] * xscale);
  pts[i].y = (int) (y[i] * yscale);
 }
 // and scale the bounds as well
 xpmin = (int) (xs * xscale);
 xpmax = (int) (xe * xscale);
 ypmin = (int) (ymin * yscale);
 ypmax = (int) (ymax * yscale);

 // now set up the scaling to convert from page space coords into device space
 // the window extent includes the true drawing size plus borders on either end
 window_ext_y = (ypmax - ypmin) + 2 * border_size;
 window_ext_x = (xpmax - xpmin) + 2 * border_size;
 // the window origin is that point in page space that becomes the upper left
 // point in device space
 window_org_x = xpmin - border_size;
 window_org_y = ypmax + border_size;

 // now determine which axis are visible in device space
 xvisible = false;
 yvisible = false;
 if (window_ext_x + window_org_x >= 0 && window_org_x <= 0) yvisible = true;
 if (window_org_y >= 0 && window_org_y - window_ext_y <= 0) xvisible = true;

 delete [] x;
 delete [] y;
 x = NULL;
 y = NULL;

 // now we must display the tick marks and the text values represented by them
 // in case an axis is not actually visible, place the info on the
 // left or bottom line outlining the entire window

 // print the x axis ticks and scale
 // if not visible, use the bottom edge as the position to base the ticks
 xtics_ys = xvisible ? ticksize : window_org_y - window_ext_y - ticksize;
 xtics_ye = xvisible ? 0 : window_org_y - window_ext_y;
 
 // here use PI as the units
 int xcts = (int) (xs / pi); // obtain starting number
 int xcte = (int) (xe / pi); // and ending number
 num_xtics_tot = xcte - xcts + 1;

 // allocate arrays for x axis coords and scale messages
 try {
  xtics_x = new int [num_xtics_tot];
  xtics_msg = new char* [num_xtics_tot];
  for (i=0; i<num_xtics_tot; i++) xtics_msg[i] = new char [YSTRLEN];
 }
 catch (...) {
  MessageBeep (0);
  AfxMessageBox ("Out of Memory", MB_OK | MB_ICONSTOP);
  return false;
 }
 
 int j = 0;
 for (i=xcts; i<=xcte; i++) {
  int xx = (int) (i * pi * xscale); // calc x in page space
  // clip and avoid printing at x = 0
  if (xx >= window_org_x && xx <= window_org_x + window_ext_x && i) {
   xtics_x[j]= xx;
   sprintf_s (xtics_msg[j], YSTRLEN, "%d", i);
   j++;
  }
 }
 num_xtics = j;

 // now calculate the y axis ticks and scale values
 // if the y axis is not visible, use the left edge
 ytics_xs = yvisible ? ticksize : window_org_x + ticksize;
 ytics_xe = yvisible ? 0 : window_org_x;

 // come up with a reasonable set of evenly spaced ticks even though the
 // range of values could be anything - first use the the larger
 // portion, if the image lies in the two y quadrants
 ScaleYAxis ();

 // allocate arrays for y axis coord and scale message
 try {
  ytics_y = new int [num_ytics_tot];
  ytics_msg = new char* [num_ytics_tot];
  for (i=0; i<num_ytics_tot; i++) ytics_msg[i] = new char [YSTRLEN];
 }
 catch (...) {
  MessageBeep (0);
  AfxMessageBox ("Out of Memory", MB_OK | MB_ICONSTOP);
  return false;
 }

 // now calc the y axis - handle clipping
 j = 0;
 // calc the ticks on the y axis - handle clipping an don't prnt at y = 0
 for (i=-num_ytics_tot/2; i<=num_ytics_tot/2; i++) {
  // calculate the y position in page space coords
  int yy = (int) (ybase * i * yscale);
  if (yy <= window_org_y && yy >= window_org_y - window_ext_y && i) {
   ytics_y[j] = yy;
   if (!istenths) sprintf_s (ytics_msg[j], YSTRLEN, "%d", i);
   else sprintf_s (ytics_msg[j], YSTRLEN, "%.1f", i * .2);
   j++;
  }
 }
 num_ytics = j;

 // now set the y units scale in the upper right corner
 sprintf_s (yscale_msg, sizeof(yscale_msg), "(Y units: %f)", ybase);
 yscale_msg_up = true;

 return true;
}
