#include "stdafx.h"
#pragma hdrstop
#include "EditBar.h"
#include "resource.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/***************************************************************************/
/*                                                                         */
/* ComboBox: special Defines                                               */
/*                                                                         */
/***************************************************************************/

#define COMBOBOX_INDEX 3
#define COMBOBOX_WIDTH 150
#define COMBOBOX_HEIGHT 150
#define COMBOBOX_BITMAP 3

/***************************************************************************/
/*                                                                         */
/* EditBar Buttons: But leave space for the ComboBox which replaces one    */
/*                  of the separators                                      */
/*                                                                         */
/***************************************************************************/

static UINT BASED_CODE EditButtons[] = {
	// same order as in the bitmap 'EditBar.bmp'
 ID_EDIT_CUT,
 ID_EDIT_COPY,
 ID_EDIT_PASTE,
	ID_SEPARATOR,
	ID_SEPARATOR,
};

BEGIN_MESSAGE_MAP(EditBar, CToolBar)
 ON_MESSAGE(CM_FONT_CHANGE, FontChange)
END_MESSAGE_MAP()

/***************************************************************************/
/*                                                                         */
/* EditBar and ~EditBar: default constructor and destructor                */
/*                                                                         */
/***************************************************************************/

          EditBar::EditBar() { }
          EditBar::~EditBar() { }

/***************************************************************************/
/*                                                                         */
/* Create: create the edit bar and add in the Combobox                     */
/*                                                                         */
/***************************************************************************/

BOOL      EditBar::Create (CWnd* pParentWnd) {
 m_bVertical = FALSE;

 // set the main editbar style
	DWORD dwStyle = WS_CHILD | WS_VISIBLE | CBRS_TOP | CBRS_SIZE_DYNAMIC | 
                 CBRS_TOOLTIPS | CBRS_FLYBY;

 // create and load the bitmap buttoms
	if (!(CToolBar::Create (pParentWnd, dwStyle, IDR_EDITBAR) &&
     LoadToolBar (IDR_EDITBAR)))
  return FALSE;
	
 // setup a default size for the Combobox
	CRect rect(-COMBOBOX_WIDTH, -COMBOBOX_HEIGHT, 0, 0);
 // create the Combobox
	if (!m_SearchBox.Create  (WS_CHILD | CBS_DROPDOWN | CBS_AUTOHSCROLL |
                           WS_VSCROLL | CBS_HASSTRINGS, rect, this,
                           IDC_FONTENUM_CBOX))
		return FALSE;

 // create and install a Combobox font
	HFONT hFont = (HFONT)GetStockObject(DEFAULT_GUI_FONT);
	if (hFont == NULL) hFont = (HFONT)GetStockObject(ANSI_VAR_FONT);
 m_SearchBox.SendMessage(WM_SETFONT, (WPARAM)hFont);
 DeleteObject (hFont);

 // setup the location of the Combobox
	if(!SetHorizontal()) return FALSE;

	return TRUE;
}

/***************************************************************************/
/*                                                                         */
/* SetHorizontal: setup window positon of Combobox                         */
/*                                                                         */
/***************************************************************************/

BOOL      EditBar::SetHorizontal () {
	CRect rect;
	m_bVertical = FALSE;
	SetBarStyle (GetBarStyle () | CBRS_ALIGN_BOTTOM);
 // replace the separator with the combobox control id
	SetButtonInfo (COMBOBOX_INDEX, IDC_FONTENUM_CBOX, TBBS_SEPARATOR,
                COMBOBOX_WIDTH);
	// if box exists, reposition it by getting the current position of the 
 // replaced button and placing the Combobox there
 if (m_SearchBox.m_hWnd != NULL)	{
		GetItemRect (COMBOBOX_INDEX, rect);
		m_SearchBox.SetWindowPos (NULL, rect.left, rect.top, 0, 0, SWP_NOZORDER |
                            SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOCOPYBITS);
		m_SearchBox.ShowWindow (SW_SHOW);
	}
	return TRUE;
}

/***************************************************************************/
/*                                                                         */
/* SetVertical: reset for a vertical toolbar - combobox will look strange  */
/*                                                                         */
/***************************************************************************/

BOOL      EditBar::SetVertical() {
	CRect rect;
 m_bVertical = TRUE;
 // perform the same sequence as when horizontal
	SetBarStyle (GetBarStyle () | CBRS_ALIGN_BOTTOM);
	SetButtonInfo (COMBOBOX_INDEX, IDC_FONTENUM_CBOX, TBBS_SEPARATOR,
                COMBOBOX_WIDTH);
	if (m_SearchBox.m_hWnd != NULL)	{
		GetItemRect(COMBOBOX_INDEX, rect);
		m_SearchBox.SetWindowPos (NULL, rect.left, rect.top, 0, 0, SWP_NOZORDER |
                            SWP_NOACTIVATE | SWP_NOSIZE | SWP_NOCOPYBITS);
		m_SearchBox.ShowWindow (SW_SHOW);
	}
	return TRUE;
}

/***************************************************************************/
/*                                                                         */
/* CalcDynamicLayout: dynamically switch between laying out horz or vert   */
/*                                                                         */
/***************************************************************************/

CSize     EditBar::CalcDynamicLayout (int nLength, DWORD dwMode) {
	// if we're committing, then set the buttons appropriately
	if (dwMode & LM_COMMIT) 	{
		if (dwMode & LM_VERTDOCK)	{
			if (!m_bVertical) SetVertical ();
		}
		else {
			if (m_bVertical)	SetHorizontal ();
		}
		return CToolBar::CalcDynamicLayout (nLength, dwMode);
	}
 // here we are not yet changing them, so get the proposed new size
	else	{
		BOOL bOld = m_bVertical;
		BOOL bSwitch = (dwMode & LM_HORZ) ? bOld : !bOld;
 	if (bSwitch)	{
			if (bOld)	SetHorizontal ();
			else	SetVertical ();
		}
		CSize sizeResult = CToolBar::CalcDynamicLayout (nLength, dwMode);
		if (bSwitch)	{
			if (bOld)	SetHorizontal ();
			else	SetVertical ();
		}
		return sizeResult;
	}
}

/***************************************************************************/
/*                                                                         */
/* FontChange: pass on user chosen font change request from Combobox       */
/*                                                                         */
/***************************************************************************/

LRESULT   EditBar::FontChange (WPARAM, LPARAM lf) {
 AfxGetApp()->m_pMainWnd->PostMessage (CM_FONT_CHANGE, 0, lf);
 return 0;
}
