#include "stdafx.h"
#pragma hdrstop
#include "FontEnumCBox.h"
#include "resource.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

/***************************************************************************/
/*                                                                         */
/* FontEnumComboBox: default constructor                                   */
/*                                                                         */
/***************************************************************************/

FontEnumComboBox::FontEnumComboBox() {
}

/***************************************************************************/
/*                                                                         */
/* ~FontEnumComboBox: remove the fonts in the array and delete the items   */
/*                                                                         */
/***************************************************************************/

FontEnumComboBox::~FontEnumComboBox() {
 int num, i;
 num = fonts_array.GetSize ();
 for (i=0; i<num; i++) delete (FONTITEM*) fonts_array.GetAt (i);
 fonts_array.RemoveAll ();
}

/***************************************************************************/
/*                                                                         */
/* FontEnumComboBox Message Response Table: trap Selection Changed         */
/*                                                                         */
/***************************************************************************/

BEGIN_MESSAGE_MAP(FontEnumComboBox, CComboBox)
	ON_CONTROL_REFLECT(CBN_SELCHANGE, OnSelChange)
END_MESSAGE_MAP()

/***************************************************************************/
/*                                                                         */
/* PreTraslateMessage: check on key strokes in the ComboBox if desired     */
/*                                                                         */
/***************************************************************************/

BOOL FontEnumComboBox::PreTranslateMessage(MSG* pMsg) {
	// for example, looking for the enter key
 //if ((pMsg->message != WM_KEYDOWN) || (pMsg->wParam != VK_RETURN))
		return CComboBox::PreTranslateMessage(pMsg);
}

/***************************************************************************/
/*                                                                         */
/* Create: get Printer dc and enum fonts and load ComboBox                 */
/*                                                                         */
/***************************************************************************/

BOOL FontEnumComboBox::Create (DWORD dwStyle, const RECT& rect,
                               CWnd* pParentWnd, UINT nID) {
 // create the ComboBox first
	if (CComboBox::Create(dwStyle, rect, pParentWnd, nID)) {
  static char   printer[80];
  char         *device, *driver, *output, *next;
  CDC          *ptrprinterDC = new CDC;

  // get access to a printer DC from the ini file installed printer
  GetProfileString ("windows", "device", "...", printer, 80);
  if ((device = strtok_s (printer, ",", &next)) != NULL &&
      (driver = strtok_s (NULL,    ",", &next)) != NULL &&
      (output = strtok_s (NULL,    ",", &next)) != NULL) {
  
   // a default printer exists, so get a DC for it
   ptrprinterDC->CreateDC (driver, device, output, NULL);
   // build font list by enumerating fonts, adding to CPtrArray and to list view
   BuildFontList (ptrprinterDC);
   delete ptrprinterDC;
  }
 }
 else return FALSE;
 return TRUE;
}

/***************************************************************************/
/*                                                                         */
/* OnSelChange: respond when the user selection has changed                */
/*                                                                         */
/***************************************************************************/

void      FontEnumComboBox::OnSelChange () {
 int i;
 FONTITEM *ptrf;
 i = GetCurSel ();                   // get the current selection
 if (i <= fonts_array.GetSize ()) {  // if within range,
  curfontid = i;
  ptrf = (FONTITEM*) fonts_array.GetAt (i); //retrieve the font
  // and post the font has changed user message on up the line
  GetParent ()-> PostMessage (CM_FONT_CHANGE, 0, (LPARAM) (&(ptrf->lf)));
 }
}

/***************************************************************************/
/*                                                                         */
/* BuildFontList: enumerates fonts and adds to the list view and array     */
/*                                                                         */
/***************************************************************************/

void      FontEnumComboBox::BuildFontList (CDC *ptrprinterDC) {
 // determine whether running under Windows 95 or NT
 DWORD version = ::GetVersion ();
 BOOL  win95   = (BYTE) version >= 4; // TRUE for Win 95; FALSE for Win NT

 // construct the model LOGFONT to enumerate - use only DEFAULT_CHARSET
 LOGFONT lf;
 memset (&lf, 0, sizeof (LOGFONT));
 lf.lfCharSet = DEFAULT_CHARSET;

 // enumerate all fonts on either Win 95 or Win NT system
 if (win95) ::EnumFontFamiliesEx (ptrprinterDC->GetSafeHdc(), &lf, 
                        (FONTENUMPROC) EnumPrinterFonts, (LPARAM) this, NULL);
 else ::EnumFontFamilies (ptrprinterDC->GetSafeHdc(), NULL, 
                          (FONTENUMPROC) EnumPrinterFonts, (LPARAM) this);
 // now add the array of fonts to the list view
 AddFontsToList ();
}

/***************************************************************************/
/*                                                                         */
/* EnumPrinterFonts: call back function to handle each font                */
/*                                                                         */
/***************************************************************************/

BOOL CALLBACK AFX_EXPORT  FontEnumComboBox::EnumPrinterFonts (ENUMLOGFONT *ptrelf, 
                                                NEWTEXTMETRICEX* /* ptrntm*/,
                                                int fonttype, LPVOID pthis) {
 // eliminate unwanted fonts, such as Turkish and MAC fonts
 if (ptrelf->elfLogFont.lfCharSet == DEFAULT_CHARSET ||
     ptrelf->elfLogFont.lfCharSet == ANSI_CHARSET ||
     ptrelf->elfLogFont.lfCharSet == SYMBOL_CHARSET ||
     ptrelf->elfLogFont.lfCharSet == OEM_CHARSET) {
  // if TrueType, construct our copy of the enumerated font and add to array
  if (fonttype & TRUETYPE_FONTTYPE) {
   FONTITEM *ptrf = new FONTITEM;
   memcpy (&ptrf->lf, &ptrelf->elfLogFont, sizeof (LOGFONT));
   ptrf->ttFont = TRUE;
   ((FontEnumComboBox*) pthis)->fonts_array.Add (ptrf);
  }
  // if printer font, construct our copy of the enumerated font & add to array
  else if (fonttype & DEVICE_FONTTYPE) {
   FONTITEM *ptrf = new FONTITEM;
   memcpy (&ptrf->lf, &ptrelf->elfLogFont, sizeof (LOGFONT));
   ptrf->ttFont = FALSE;
   ((FontEnumComboBox*) pthis)->fonts_array.Add (ptrf);
  }
 }
 return 1;
}

/***************************************************************************/
/*                                                                         */
/* AddFontsToList: adds all of the fonts to the list view control          */
/*                                                                         */
/***************************************************************************/

void      FontEnumComboBox::AddFontsToList () {
 int num, i;
 FONTITEM *ptrf;

 num = fonts_array.GetSize (); // get total number of fonts to add
 for (i=0; i<num; i++) {
  // access the current font
  ptrf = (FONTITEM*) fonts_array.GetAt (i);
  // use the face name in the list view
  AddString (ptrf->lf.lfFaceName);
 }
 SetCurSel (0);
 curfontid = 0;
}
