#include "stdafx.h"
#pragma  hdrstop
#include "Pgm10a.h"
#include "GeoDoc.h"
#include "GeoView.h"
#include "MainFrame.h"

/***************************************************************************/
/*                                                                         */
/* defines for the default settings                                        */
/*                                                                         */
/***************************************************************************/

#define INITSCROLL  25      // initial scroll percentage
#define INITZOOM     2      // initial zoom factor
#define INITSCALE 6000      // initial map scale

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(GeoView, CScrollView)

/***************************************************************************/
/*                                                                         */
/* GeoView Message Response Table                                          */
/*                                                                         */
/***************************************************************************/

BEGIN_MESSAGE_MAP(GeoView, CScrollView)
	ON_WM_DESTROY()
 ON_WM_PALETTECHANGED ()
 ON_WM_QUERYNEWPALETTE ()
 ON_MESSAGE (CM_FONT_CHANGE, FontChange)
 ON_MESSAGE(WM_DISPLAYCHANGE,      OnDisplayChange)
 ON_BN_CLICKED (CM_MAPCB_UPLEFT, CmDummy)
 ON_BN_CLICKED (CM_MAPCB_UP, CmDummy)
 ON_BN_CLICKED (CM_MAPCB_UPRIGHT, CmDummy)
 ON_BN_CLICKED (CM_MAPCB_LEFT, CmDummy)
 ON_BN_CLICKED (CM_MAPCB_RIGHT, CmDummy)
 ON_BN_CLICKED (CM_MAPCB_SETSCALE, CmDummy)
 ON_BN_CLICKED (CM_MAPCB_DOWNLEFT, CmDummy)
 ON_BN_CLICKED (CM_MAPCB_DOWN, CmDummy)
 ON_BN_CLICKED (CM_MAPCB_DOWNRIGHT, CmDummy)
 ON_BN_CLICKED (CM_MAPCB_SETCENTER, CmDummy)
 ON_BN_CLICKED (CM_MAPCB_SETZOOM, CmDummy)
 ON_BN_CLICKED (CM_MAPCB_SETSCROLL, CmDummy)
 ON_BN_CLICKED (CM_MAPCB_ZOOM_IN, CmDummy)
 ON_BN_CLICKED (CM_MAPCB_ZOOM_OUT, CmDummy)
 ON_BN_CLICKED (CM_OPTIONS_SHOW_CITY, CmDummy)
 ON_BN_CLICKED (CM_OPTIONS_SHOW_TERRAIN, CmDummy)
 ON_BN_CLICKED (CM_OPTIONS_SHOW_FACTORY, CmDummy)
 ON_BN_CLICKED (CM_OPTIONS_SHOW_WEATHER, CmDummy)
 ON_BN_CLICKED (CM_OPTIONS_SHOW_SHIPYARD, CmDummy)
 ON_BN_CLICKED (ID_EDIT_CUT, CmDummy)
 ON_BN_CLICKED (ID_EDIT_COPY, CmDummy)
 ON_BN_CLICKED (ID_EDIT_PASTE, CmDummy)
 ON_BN_CLICKED (CM_LINE1, CmDummy)
 ON_BN_CLICKED (CM_LINE2, CmDummy)
 ON_BN_CLICKED (CM_LINE3, CmDummy)
 ON_BN_CLICKED (CM_LINE4, CmDummy)
 ON_BN_CLICKED (CM_COLOR_BLACK, CmDummy)
 ON_BN_CLICKED (CM_COLOR_RED, CmDummy)
 ON_BN_CLICKED (CM_COLOR_BLUE, CmDummy)
 ON_BN_CLICKED (CM_COLOR_GREEN, CmDummy)
 ON_BN_CLICKED (CM_COLOR_YELLOW, CmDummy)
 ON_BN_CLICKED (CM_COLOR_PURPLE, CmDummy)
END_MESSAGE_MAP()

/***************************************************************************/
/*                                                                         */
/* GeoView: constructor sets object pointers to NULL                       */
/*                                                                         */
/***************************************************************************/

GeoView::GeoView () {
 ptrpalette = NULL;
 ptrdib     = NULL;
 ptrlf      = NULL;
}

/***************************************************************************/
/*                                                                         */
/* ~GeoView: remove objects                                                */
/*                                                                         */
/***************************************************************************/

GeoView::~GeoView () {
 RemoveObjects ();
 if (ptrlf) delete ptrlf;
}

/***************************************************************************/
/*                                                                         */
/* FontChange: respond to font change messages from ComboBox               */
/*                                                                         */
/***************************************************************************/

LRESULT   GeoView::FontChange (WPARAM, LPARAM lf) {
 // if no LOGFONT is allocated, get one; otherwise reuse it
 if (!ptrlf) ptrlf = new LOGFONT;
 // copy in the user's choice
 memcpy (ptrlf, (LOGFONT*) lf, sizeof(LOGFONT));
 // install a 12 point version
 ptrlf->lfHeight = -12;
 ptrlf->lfWidth = 0;
 // paint the text window in this font
 Invalidate ();
 return 0;
}

/***************************************************************************/
/*                                                                         */
/* CmDummy: dummy function for command handlers                            */
/*                                                                         */
/***************************************************************************/

void      GeoView::CmDummy () {
 MessageBox ("Not Implemented Yet", MB_OK);
}

/***************************************************************************/
/*                                                                         */
/* OnDraw: paint the CDib and if there is a font, show the text            */
/*                                                                         */
/***************************************************************************/

void      GeoView::OnDraw (CDC* pDC) {
	GeoDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	MainFrame *ptrfrm = (MainFrame*) (AfxGetApp()->m_pMainWnd);
 if (!ptrfrm) return;

 // display a new caption and display a test statusbar message
 ptrfrm->DisplayNewCaption ("Geographical Maps - Test New Caption");
 ptrfrm->DisplayOnStatusBar ("Test status bar message");

 // first step is to install a color palette if doing screen or preview
 if (ptrdib) {
  if (!ptrpalette) {                   // case LOADDIB has no palette yet
   ptrpalette = new CPalette ();       // so make the basic palette
   ptrdib->MakePalette (ptrpalette);
  }                                 
  if (ptrpalette) {                    // use existing palette
   pDC->SelectPalette (ptrpalette, FALSE);
   pDC->RealizePalette ();        
  }
  pDC->SetStretchBltMode (COLORONCOLOR); // set for optimum color convert
 }
 
 // screen display - use the Dib
 if (ptrdib)
  // dib origin is backwards in height - copy dib bits to screen
  // fairly slow operation, if much copying is needed, use memory DC
  SetDIBitsToDevice (pDC->m_hDC, 0, 0, bmp_size.cx, bmp_size.cy,
                     0, 0, 0, bmp_size.cy, ptrdib-> GetDibBitsAddr (), 
                     ptrdib->GetBitmapInfo (), DIB_RGB_COLORS);
 
 // if there is a user font chosen, display a bit of text in that font
 if (ptrlf) {
  CFont font;
  font.CreateFontIndirect (ptrlf);
  CFont *ptroldfont = pDC->SelectObject (&font);
  pDC->TextOut (10, 10, "Hello");
  pDC->SelectObject (ptroldfont);
 }
}

/***************************************************************************/
/*                                                                         */
/* OnInitialUpdate: Load user profile data, display on MapControlBar       */
/*                                                                         */
/***************************************************************************/

void      GeoView::OnInitialUpdate () {
 // load user profile data
	MainFrame *ptrfrm = (MainFrame*) (AfxGetApp()->m_pMainWnd);
 if (!ptrfrm) return;
 ReadSetUpProfileData ();
 // display initial settings on the Map Control Bar
 ptrfrm->GetMapControlBar ()->DisplayAll (&center, zoom, scroll, scale);	
 
 // display some data on the Map Local Stats Bar
 char pos[30];
 center.FormatPosition (pos, sizeof (pos));
 ptrfrm->GetMapLocalStatsBar ()->DisplayDayTime (date, time);
 ptrfrm->GetMapLocalStatsBar ()->DisplayMapNowAtValues (
  "Europe", pos, time, date, "5:30", "6:30", "Partly Cloudy", "Light Mud");

 // display a new caption and a test message on the statusbar
 ptrfrm->DisplayNewCaption ("Geographical Maps - Test New Caption");
 ptrfrm->DisplayOnStatusBar ("Test status bar message");

 AdjustBmpLoading ();     // handle load options
 AdjustDisplayOptions (); // handle display options
	CScrollView::OnInitialUpdate();
}

/***************************************************************************/
/*                                                                         */
/* ReadSetUpProfileData - input any position, map values, and date/time    */
/*                                                                         */
/***************************************************************************/

void      GeoView::ReadSetUpProfileData () {
 char section[40];
 char entry[40];
 char latstr[10], lonstr[10];
 long lat, lon;
 char file[80];
 
 HINSTANCE hs = AfxGetApp()->m_hInstance;
 LoadString (hs, IDS_GEO_INI, file, sizeof(file));

 LoadString (hs, IDS_GEO_PROFILE_DEMOMAP, section, sizeof(section));
 LoadString (hs, IDS_GEO_PROFILE_DATE, entry, sizeof (entry));
 GetPrivateProfileString (section, entry, "0", date, sizeof(date), file);
 LoadString (hs, IDS_GEO_PROFILE_TIME, entry, sizeof (entry));
 GetPrivateProfileString (section, entry, "0", time, sizeof(time), file);

 LoadString (hs, IDS_GEO_PROFILE_LAT, entry, sizeof (entry));
 GetPrivateProfileString (section, entry, "0", latstr, sizeof (latstr),
                          file);
 LoadString (hs, IDS_GEO_PROFILE_LON, entry, sizeof (entry));
 GetPrivateProfileString (section, entry, "0", lonstr, sizeof (lonstr),
                          file);
 lat = atol (latstr);
 lon = atol (lonstr);
 center.SetLatLon (lat, lon);

 LoadString (hs, IDS_GEO_PROFILE_ZOOM, entry, sizeof (entry));
 zoom = GetPrivateProfileInt (section, entry, INITZOOM, file);
 LoadString (hs, IDS_GEO_PROFILE_SCROL, entry, sizeof (entry));
 scroll = GetPrivateProfileInt (section, entry, INITSCROLL, file);
 LoadString (hs, IDS_GEO_PROFILE_SCALE, entry, sizeof (entry));
 GetPrivateProfileString (section, entry, "INITSCALE", lonstr, sizeof (lonstr),
                          file);
 scale = atof (lonstr);
}

/***************************************************************************/
/*                                                                         */
/* WriteSetUpProfileData - output any position, map values, and date/time  */
/*                                                                         */
/***************************************************************************/
  
void      GeoView::WriteSetUpProfileData () {
 char section[40];
 char entry[40];
 char latstr[10], lonstr[10];
 char file[80];
 
 HINSTANCE hs = AfxGetApp()->m_hInstance;
 LoadString (hs, IDS_GEO_INI, file, sizeof(file));

 LoadString (hs, IDS_GEO_PROFILE_DEMOMAP, section, sizeof(section));
 LoadString (hs, IDS_GEO_PROFILE_DATE, entry, sizeof (entry));
 WritePrivateProfileString (section, entry, date, file);
 LoadString (hs, IDS_GEO_PROFILE_TIME, entry, sizeof (entry));
 WritePrivateProfileString (section, entry, time, file);

 wsprintf (latstr, "%ld", center.Latitude);
 wsprintf (lonstr, "%ld", center.Longitude);
 LoadString (hs, IDS_GEO_PROFILE_LAT, entry, sizeof (entry));
 WritePrivateProfileString (section, entry, latstr, file);
 LoadString (hs, IDS_GEO_PROFILE_LON, entry, sizeof (entry));
 WritePrivateProfileString (section, entry, lonstr, file);

 LoadString (hs, IDS_GEO_PROFILE_ZOOM, entry, sizeof (entry));
 wsprintf (latstr, "%d",  zoom);
 WritePrivateProfileString (section, entry, latstr, file);
 LoadString (hs, IDS_GEO_PROFILE_SCROL, entry, sizeof (entry));
 wsprintf (latstr, "%d",  scroll);
 WritePrivateProfileString (section, entry, latstr, file);
 LoadString (hs, IDS_GEO_PROFILE_SCALE, entry, sizeof (entry));
 sprintf_s (latstr, sizeof(latstr), "%7.2f", scale);
 WritePrivateProfileString (section, entry, latstr, file);
}

/***************************************************************************/
/*                                                                         */
/* OnDestroy: save the user profile data                                   */
/*                                                                         */
/***************************************************************************/

void GeoView::OnDestroy () {
 WriteSetUpProfileData ();	
	CScrollView::OnDestroy ();
}

/***************************************************************************/
/*                                                                         */
/* RemoveObjects: delete all items associated with a dib                   */
/*                                                                         */
/***************************************************************************/

void      GeoView::RemoveObjects () {
 if (ptrpalette) delete ptrpalette;
 ptrpalette = NULL;
}

/***************************************************************************/
/*                                                                         */
/* OnUpdate: reload bitmap, set display options, update window             */
/*                                                                         */
/***************************************************************************/

void     GeoView::OnUpdate (CView *ptrsender, LPARAM hint, CObject *ptrdata) {
 AdjustBmpLoading ();     // handle load options
 AdjustDisplayOptions (); // handle display options
 CScrollView::OnUpdate (ptrsender, hint, ptrdata); // pass on to base class
}

/***************************************************************************/
/*                                                                         */
/* AdjustBmpLoading: handle Bmp Load options                               */
/*                                                                         */
/***************************************************************************/

void      GeoView::AdjustBmpLoading () {
 RemoveObjects ();
 ptrdib = GetDocument ()->GetDib ();     // acquire the DIB from the document
 if (!ptrdib) {                          // here there is no DIB loaded
  bmp_size = CSize (0, 0);               // set sizes to null
  t_size = bmp_size;
 }
 else {                                  // here there is a DIB
  bmp_size.cx = ptrdib->GetDibWidth ();  // save DIB's true size
  bmp_size.cy = ptrdib->GetDibHeight ();
  t_size = bmp_size;                     // use DIB size as scroll view's size
  ptrpalette = new CPalette ();         // construct a basic palette
  ptrdib->MakePalette (ptrpalette);
 }
}

/***************************************************************************/
/*                                                                         */
/* AdjustDisplayOptions: handle display options                            */
/*                                                                         */
/***************************************************************************/

void      GeoView::AdjustDisplayOptions () {
 if (ptrdib) SetScrollSizes (MM_TEXT, t_size, CSize (10, 10), CSize (1, 1));
 else SetScrollSizes (MM_TEXT, CSize (0, 0), CSize (10, 10), CSize (1, 1));
}

/***************************************************************************/
/*                                                                         */
/* OnPaletteChanged: realize our palette when system palette changes       */
/*                                                                         */
/***************************************************************************/

void      GeoView::OnPaletteChanged (CWnd *ptrwin) {
 if (ptrwin == this) return;    // ignore our own changing
 OnQueryNewPalette ();
}

/***************************************************************************/
/*                                                                         */
/* OnQueryNewPalette: upon window activation, realize our palette          */
/*                                                                         */
/***************************************************************************/

BOOL      GeoView::OnQueryNewPalette () {
 if (!ptrdib) return 0;
 BOOL delete_palette = FALSE;
 CDC *ptrdc = GetDC ();
 if (!ptrpalette) {
  ptrpalette = new CPalette ();
  ptrdib->MakePalette (ptrpalette);
  if (ptrpalette) delete_palette = TRUE;  // flag it for deletion
  else return 0;
 }
 CPalette *ptroldpal = ptrdc->SelectPalette (ptrpalette, FALSE);
 UINT num = ptrdc->RealizePalette ();        
 if (num) InvalidateRect (NULL, TRUE);
 ptrdc->SelectPalette (ptroldpal, TRUE);
 ptrdc->RealizePalette ();
 ReleaseDC (ptrdc);
 if (delete_palette) delete ptrpalette;
 return (BOOL) num;
}

/***************************************************************************/
/*                                                                         */
/* OnDisplayChange: user has changed the display settings, must update     */
/*                                                                         */
/***************************************************************************/

LRESULT   GeoView::OnDisplayChange (WPARAM which, LPARAM lp) {
 LRESULT retcd = CScrollView::OnDisplayChange (which, lp);
 AdjustBmpLoading ();
 AdjustDisplayOptions ();
 return retcd;
}

/***************************************************************************/
/*                                                                         */
/* OnKeyDown: provide a keyboard scroller interface                        */
/*                                                                         */
/***************************************************************************/

void      GeoView::OnKeyDown (UINT key, UINT, UINT) {
 // check for and handle any possible keyboard scroll request
 switch (key) {

  case VK_UP:                   // requests scroll up 1 line
    SendMessage (WM_VSCROLL, SB_LINEUP, 0L);  break;

  case VK_DOWN:                 // requests scroll down 1 line
   SendMessage (WM_VSCROLL, SB_LINEDOWN, 0L); break;
                
  case VK_LEFT:  // requests scroll left 1 col
   SendMessage (WM_HSCROLL, SB_LINEUP, 0L); break;

  case VK_RIGHT: // requests scroll right 1 col
   SendMessage (WM_HSCROLL, SB_LINEDOWN, 0L); break;

  case VK_PRIOR: // request scroll 1 page up
   SendMessage (WM_VSCROLL, SB_PAGEUP, 0L); break;

  case VK_NEXT:  // request scroll 1 page down
   SendMessage (WM_VSCROLL, SB_PAGEDOWN, 0L); break;

  case VK_END:   // request goto the bottom
   SendMessage (WM_HSCROLL, SB_PAGEDOWN, 0L); break;

  case VK_HOME:  // request goto the top
   SendMessage (WM_HSCROLL, SB_PAGEUP, 0L);  break;
 }
}

#ifdef _DEBUG
void GeoView::AssertValid() const { CScrollView::AssertValid(); }
void GeoView::Dump(CDumpContext& dc) const { CScrollView::Dump(dc); }
GeoDoc* GeoView::GetDocument() {// non-debug version is inline
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(GeoDoc)));
	return (GeoDoc*)m_pDocument;
}
#endif //_DEBUG
