#include "stdafx.h"
#include "Pgm10a.h"
#include "MainFrame.h"
#include "resource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(MainFrame, CFrameWnd)

/***************************************************************************/
/*                                                                         */
/* MainFrame Message Response Table                                        */
/*                                                                         */
/***************************************************************************/

BEGIN_MESSAGE_MAP(MainFrame, CFrameWnd)
	ON_WM_CREATE()
	ON_COMMAND(ID_VIEW_TOOLBAR, OnViewToolbar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_TOOLBAR, OnUpdateViewToolbar)
	ON_COMMAND(ID_VIEW_STATUS_BAR, OnViewStatusBar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_STATUS_BAR, OnUpdateViewStatusBar)
	ON_COMMAND(ID_VIEW_MAPCONTROLBAR, OnViewMapControlBar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_MAPCONTROLBAR, OnUpdateViewMapControlBar)
	ON_COMMAND(ID_VIEW_MAPLOCALSTATSBAR, OnViewMapLocalStatsBar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_MAPLOCALSTATSBAR, OnUpdateViewMapLocalStatsBar)
	ON_COMMAND(ID_VIEW_OPTIONSBAR, OnViewOptionsBar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_OPTIONSBAR, OnUpdateViewOptionsBar)
	ON_WM_CLOSE()
	ON_COMMAND(ID_VIEW_EDITBAR, OnViewEditBar)
	ON_UPDATE_COMMAND_UI(ID_VIEW_EDITBAR, OnUpdateViewEditBar)
	ON_COMMAND(ID_VIEW_PALETTEBOX, OnViewPaletteBox)
 ON_MESSAGE(CM_FONT_CHANGE, FontChange)
	ON_UPDATE_COMMAND_UI(ID_VIEW_PALETTEBOX, OnUpdateViewPaletteBox)
END_MESSAGE_MAP()

static UINT indicators[] = {
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

static UINT BASED_CODE PaletteBox[] = {
  CM_LINE1,
  CM_LINE2,
  CM_LINE3,
  CM_LINE4,
  ID_SEPARATOR,
  ID_SEPARATOR,
  CM_COLOR_BLACK,
  CM_COLOR_RED,
  CM_COLOR_GREEN,
  CM_COLOR_YELLOW,
  CM_COLOR_BLUE,
  CM_COLOR_PURPLE,
};

/***************************************************************************/
/*                                                                         */
/* App's ini file section identifiers to be used to read/write placement   */
/*                                                                         */
/***************************************************************************/

static TCHAR BASED_CODE szSection[] = _T("Settings");
static TCHAR BASED_CODE szWindowPos[] = _T("WindowPos");
static TCHAR szFormat[] = _T("%u,%u,%d,%d,%d,%d,%d,%d,%d,%d");

/***************************************************************************/
/*                                                                         */
/* ReadWindowPlacement: read a window placement from the settings section  */
/*                      of an app's ini file                               */
/*                                                                         */
/***************************************************************************/

static BOOL PASCAL NEAR ReadWindowPlacement (LPWINDOWPLACEMENT pwp) {
	CString strBuffer = AfxGetApp()->GetProfileString(szSection, szWindowPos);
	if (strBuffer.IsEmpty())
		return FALSE;

	WINDOWPLACEMENT wp;
	int nRead = sscanf_s (strBuffer, szFormat,
	                     	&wp.flags, &wp.showCmd,
                     		&wp.ptMinPosition.x, &wp.ptMinPosition.y,
                     		&wp.ptMaxPosition.x, &wp.ptMaxPosition.y,
                     		&wp.rcNormalPosition.left, &wp.rcNormalPosition.top,
                     		&wp.rcNormalPosition.right, &wp.rcNormalPosition.bottom);
	if (nRead != 10) return FALSE;

	wp.length = sizeof wp;
	*pwp = wp;
	return TRUE;
}

/***************************************************************************/
/*                                                                         */
/* WriteWindowPlacement: write window placement to settings section of     */
/*                       app's ini file                                    */
/*                                                                         */
/***************************************************************************/

static void PASCAL NEAR WriteWindowPlacement (LPWINDOWPLACEMENT pwp) {
	TCHAR szBuffer[sizeof("-32767")*8 + sizeof("65535")*2];
 wsprintf(szBuffer, szFormat,
	         pwp->flags, pwp->showCmd,
          pwp->ptMinPosition.x, pwp->ptMinPosition.y,
         	pwp->ptMaxPosition.x, pwp->ptMaxPosition.y,
	         pwp->rcNormalPosition.left, pwp->rcNormalPosition.top,
         	pwp->rcNormalPosition.right, pwp->rcNormalPosition.bottom);
	AfxGetApp ()->WriteProfileString (szSection, szWindowPos, szBuffer);
}

/***************************************************************************/
/*                                                                         */
/* MainFrame and ~MainFrame: default constuctor and destructor             */
/*                                                                         */
/***************************************************************************/

MainFrame::MainFrame () { }
MainFrame::~MainFrame () { }

/***************************************************************************/
/*                                                                         */
/* FontChange: pass on any user font change from the ComboBox              */
/*                                                                         */
/***************************************************************************/

LRESULT   MainFrame::FontChange (WPARAM, LPARAM lf) {
 CView *ptrview = GetActiveView ();
 if (ptrview) ptrview->PostMessage (CM_FONT_CHANGE, 0, lf);
 return 0;
}

/***************************************************************************/
/*                                                                         */
/* CmDummy: dummy command handler                                          */
/*                                                                         */
/***************************************************************************/

void      MainFrame::CmDummy () {
 MessageBox ("Not Implemented Yet", MB_OK);
}

/***************************************************************************/
/*                                                                         */
/* GetOptionsBar: retrieve the Options bar                                 */
/*                                                                         */
/***************************************************************************/

OptionsBar*        MainFrame::GetOptionsBar () {
 return &optionsbar;
}

/***************************************************************************/
/*                                                                         */
/* GetMapControlBar: retrieve the Map Control bar                          */
/*                                                                         */
/***************************************************************************/

MapControlBar*     MainFrame::GetMapControlBar () {
 return &mapcontrolbar;
}

/***************************************************************************/
/*                                                                         */
/* GetMapLocalStatsBar: retrieve the Map Local Stats Bar                   */
/*                                                                         */
/***************************************************************************/

MapLocalStatsBar*  MainFrame::GetMapLocalStatsBar () {
 return &maplocalstatsbar;
}

/***************************************************************************/
/*                                                                         */
/* OnCreate: load tool bars and handle docking and any saved locations     */
/*                                                                         */
/***************************************************************************/

int       MainFrame::OnCreate (LPCREATESTRUCT lpCreateStruct) {
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

 // load in any possible saved window placement data
	WINDOWPLACEMENT wp;
	if (ReadWindowPlacement(&wp))
		SetWindowPlacement(&wp);  // data exists so set up the window as it last was

 // create all bars
	if (m_wndToolBar.Create(this) &&	m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
  if (maplocalstatsbar.Create (this))
   if (mapcontrolbar.Create (this)) 
     if (optionsbar.Create (this))
      if (editbar.Create (this)) 
       if (m_wndStatusBar.Create (this) &&
           m_wndStatusBar.SetIndicators (indicators,
                                		       sizeof (indicators) / sizeof (UINT)))
        if (palettebox.Create (this, WS_CHILD | WS_VISIBLE | CBRS_SIZE_FIXED |
                        CBRS_TOP | CBRS_TOOLTIPS | CBRS_FLYBY, IDB_PALETTEBOX)
            && palettebox.LoadBitmap (IDB_PALETTEBOX)
            && palettebox.SetButtons (PaletteBox, sizeof(PaletteBox)/sizeof(UINT)));
        else {
         TRACE0("Failed to create palette box\n");
         return -1; 
        }
       else {
        TRACE0("Failed to create status bar\n");
        return -1; 
       }
      else {
       TRACE0("Failed to create edit bar\n");
       return -1;
      }
     else {
      TRACE0("Failed to create options bar\n");
      return -1;
     }
   else {
    TRACE0("Failed to create map control bar\n");
    return -1;
   }
  else {
   TRACE0("Failed to create map local stats bar\n");
   return -1;
  }
 else	{
	 TRACE0("Failed to create toolbar\n");
 	return -1;
 }

 // notify the framewindow of docking requirements
 EnableDocking (CBRS_ALIGN_ANY);

	m_wndToolBar.SetBarStyle (m_wndToolBar.GetBarStyle() |
		                         CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC);
 m_wndToolBar.SetWindowText (_T("Main Tool Bar"));
	m_wndToolBar.EnableDocking (CBRS_ALIGN_ANY);
	DockControlBar (&m_wndToolBar);

 editbar.SetWindowText (_T("Edit Bar"));
	editbar.EnableDocking (CBRS_ALIGN_TOP | CBRS_ALIGN_BOTTOM);
	DockControlBarLeftOf (&editbar,&m_wndToolBar);

 palettebox.SetColumns (2); // set the palette box to 2 columns
 palettebox.SetBarStyle (palettebox.GetBarStyle() |CBRS_TOOLTIPS | CBRS_FLYBY);
 palettebox.SetWindowText (_T("Pal"));
	palettebox.EnableDocking (CBRS_ALIGN_ANY);
	DockControlBar (&palettebox, AFX_IDW_DOCKBAR_LEFT);

 mapcontrolbar.SetWindowText(_T("Map Controls"));
 mapcontrolbar.EnableDocking (0);
 
 maplocalstatsbar.SetWindowText(_T("Map Local Stats"));
 maplocalstatsbar.EnableDocking (0);

 optionsbar.SetWindowText(_T("Opt"));
 optionsbar.EnableDocking (0);
 
 // begin cut section - once you have the tool bars positioned to your
 // desired location and have terminated the app, try commenting out
 // this cut section from this point through the end cut comment below
 // and just reload the tool bars - notice the very different actions

 // in case this is the first time that the app is run and no state info
 // is yet saved, set up an initial first position, and then based upon
 // the actual setup, repeat process with the better guess for initial 
 // position
 
 CRect r, q;
 CPoint p;

 GetClientRect (&r);                 // get window area less bars
 maplocalstatsbar.GetWindowRect(&q); // get dims of Map Local Stats Bar
 p.x = r.right - q.Width ();         // convert into a point placement
 p.y = r.top + 30;
 FloatControlBar (&maplocalstatsbar, p); // and float it here

 p.y += 100;                         // insert an educated guess offset
 FloatControlBar (&optionsbar, p);   // and float the Options Bar below it

 mapcontrolbar.GetWindowRect (&q);   // get dims of Map Control Bar
 p.x = r.right - q.Width ();         // install its location in point
 p.y = r.bottom - q.Height ();
 FloatControlBar (&mapcontrolbar, p);// now float it here
 
 // force the layout to take effect
 RepositionBars (AFX_IDW_CONTROLBAR_FIRST, AFX_IDW_CONTROLBAR_LAST,
                 0, reposQuery, r);
 
 // now construct a more optimum layout of the three floating bars
 int top = r.top;
 maplocalstatsbar.GetWindowRect(&q);
 p.x = r.right - q.Width();
 p.y = top;
 ClientToScreen (&p);
 FloatControlBar (&maplocalstatsbar, p);
 
 top += q.Height () + 22;
 p.y = top;
 mapcontrolbar.GetWindowRect (&q);
 p.x = r.right - q.Width ();
 ClientToScreen (&p);
 FloatControlBar (&mapcontrolbar, p);
 
 top += q.Height () + 22;
 p.y = top;
 optionsbar.GetWindowRect (&q);
 p.x = r.right - q.Width ();
 ClientToScreen (&p);
 FloatControlBar (&optionsbar, p);
 // end of cut section

 // now load in any previously saved states, beginning with the number of
 // columns which is not saved by the SaveBarState function 
	palettebox.SetColumns (AfxGetApp ()->GetProfileInt (_T("General"),
                                                     _T("Columns"), 2));
	LoadBarState(_T("General"));
	return 0;
}

/***************************************************************************/
/*                                                                         */
/* DockControlBarLeftOf: dock a control bar to the left of another bar     */
/*                                                                         */
/***************************************************************************/

void      MainFrame::DockControlBarLeftOf (CToolBar* Bar,CToolBar* LeftOf) {
 CRect rect;
	DWORD dw;
	UINT n;

 RecalcLayout (); // force MFC to adjust the dimensions of all docked ToolBars
	LeftOf->GetWindowRect (&rect);// get correct rectangle
	rect.OffsetRect (1,0);        // get first pixel position beyond for this bar
	dw=LeftOf->GetBarStyle ();    // determine how main bar is docked
	n = 0;
	n = (dw&CBRS_ALIGN_TOP) ? AFX_IDW_DOCKBAR_TOP : n;
	n = (dw&CBRS_ALIGN_BOTTOM && n==0) ? AFX_IDW_DOCKBAR_BOTTOM : n;
	n = (dw&CBRS_ALIGN_LEFT && n==0) ? AFX_IDW_DOCKBAR_LEFT : n;
	n = (dw&CBRS_ALIGN_RIGHT && n==0) ? AFX_IDW_DOCKBAR_RIGHT : n;

 // if the default parms are used,
 // DockControlBar will dock each Toolbar on a seperate line
 // By calculating a rectangle, we simulate a Toolbar being dragged to 
 // that location and docked
	DockControlBar (Bar, n, &rect);
}

/*****************************************************************************/
/*                                                                           */
/* DisplayNewCaption: install a new caption on the title bar                 */
/*                                                                           */
/*****************************************************************************/

void      MainFrame::DisplayNewCaption (char *msg) {
 SetWindowText (msg);
}

/***************************************************************************/
/*                                                                         */
/* DisplayOnStatusBar: show a message on the main status bar               */
/*                                                                         */
/***************************************************************************/

void      MainFrame::DisplayOnStatusBar (char *s) {
 m_wndStatusBar.SetPaneText (0, s);
}

/***************************************************************************/
/*                                                                         */
/* OnClose: store number of columns & store window placement with barstates*/
/*                                                                         */
/***************************************************************************/

void      MainFrame::OnClose () {
 // save all control bar states
	SaveBarState (_T("General"));
	AfxGetApp ()->WriteProfileInt (_T("General"), _T("Columns"),
                                palettebox.GetColumns ());
	WINDOWPLACEMENT wp;
	wp.length = sizeof wp;
 // get window location
	if (GetWindowPlacement (&wp)) {
		wp.flags = 0;
		if (IsZoomed ())	wp.flags |= WPF_RESTORETOMAXIMIZED;
		// and write it to the .INI file
		WriteWindowPlacement(&wp);
	}    
	CFrameWnd::OnClose();
}

/***************************************************************************/
/*                                                                         */
/* PreCreateWindow: setup window style options                             */
/*                                                                         */
/***************************************************************************/

BOOL      MainFrame::PreCreateWindow (CREATESTRUCT& cs) {
	cs.style = WS_OVERLAPPED | WS_CAPTION | FWS_ADDTOTITLE | WS_THICKFRAME |
            WS_SYSMENU | WS_MINIMIZEBOX | WS_MAXIMIZEBOX | WS_MAXIMIZE;
	return CFrameWnd::PreCreateWindow(cs);
}

/***************************************************************************/
/*                                                                         */
/* OnViewToolbar: hide/show main tool bar                                  */
/*                                                                         */
/***************************************************************************/

void      MainFrame::OnViewToolbar () {
 if ((m_wndToolBar.GetStyle () & WS_VISIBLE) != 0)
		ShowControlBar (&m_wndToolBar, FALSE, FALSE);
 else ShowControlBar (&m_wndToolBar, TRUE, FALSE);
 m_wndToolBar.Invalidate ();
}

/***************************************************************************/
/*                                                                         */
/* OnUpdateViewToolbar: enable hide/show main tool bar                     */
/*                                                                         */
/***************************************************************************/

void      MainFrame::OnUpdateViewToolbar (CCmdUI* pCmdUI) {
 pCmdUI->SetCheck (m_wndToolBar.GetStyle () & WS_VISIBLE);
}

/***************************************************************************/
/*                                                                         */
/* OnViewStatusBar: hide/show the main status bar                          */
/*                                                                         */
/***************************************************************************/

void      MainFrame::OnViewStatusBar () {
 if ((m_wndStatusBar.GetStyle () & WS_VISIBLE) != 0)
 		ShowControlBar (&m_wndStatusBar, FALSE, FALSE);
 else ShowControlBar (&m_wndStatusBar, TRUE, FALSE);
 m_wndStatusBar.Invalidate ();
}

/***************************************************************************/
/*                                                                         */
/* OnUpdateViewStatusBar: enable hide/show the main status bar             */
/*                                                                         */
/***************************************************************************/

void      MainFrame::OnUpdateViewStatusBar (CCmdUI* pCmdUI) {
 pCmdUI->SetCheck (m_wndStatusBar.GetStyle () & WS_VISIBLE);
}

/***************************************************************************/
/*                                                                         */
/* OnViewMapControlBar: hide/show Map Control Bar                          */
/*                                                                         */
/***************************************************************************/

void      MainFrame::OnViewMapControlBar () {
 if ((mapcontrolbar.GetStyle () & WS_VISIBLE) != 0)
		ShowControlBar (&mapcontrolbar, FALSE, FALSE);
 else ShowControlBar (&mapcontrolbar, TRUE, FALSE);
 mapcontrolbar.Invalidate ();
}

/***************************************************************************/
/*                                                                         */
/* OnUpdateViewMapControlBar: enable hide/show Map Control Bar             */
/*                                                                         */
/***************************************************************************/

void      MainFrame::OnUpdateViewMapControlBar (CCmdUI* pCmdUI) {
 pCmdUI->SetCheck (mapcontrolbar.GetStyle () & WS_VISIBLE);
}

/***************************************************************************/
/*                                                                         */
/* OnViewMapLocalStatsBar: hide/show Map Local Stats Bar                   */
/*                                                                         */
/***************************************************************************/

void      MainFrame::OnViewMapLocalStatsBar () {
 if ((maplocalstatsbar.GetStyle () & WS_VISIBLE) != 0)
		ShowControlBar (&maplocalstatsbar, FALSE, FALSE);
 else ShowControlBar (&maplocalstatsbar, TRUE, FALSE);
 maplocalstatsbar.Invalidate ();
}

/***************************************************************************/
/*                                                                         */
/* OnUpdateViewMapLocalStatsBar: enable hide/show Map Local Stats Bar      */
/*                                                                         */
/***************************************************************************/

void      MainFrame::OnUpdateViewMapLocalStatsBar (CCmdUI* pCmdUI) {
 pCmdUI->SetCheck (maplocalstatsbar.GetStyle () & WS_VISIBLE);
}

/***************************************************************************/
/*                                                                         */
/* OnViewOptionsBar: hide/show the Options Bar                             */
/*                                                                         */
/***************************************************************************/

void      MainFrame::OnViewOptionsBar () {
 if ((optionsbar.GetStyle () & WS_VISIBLE) != 0)
		ShowControlBar (&optionsbar, FALSE, FALSE);
 else ShowControlBar (&optionsbar, TRUE, FALSE);
 optionsbar.Invalidate ();
}

/***************************************************************************/
/*                                                                         */
/* OnUpdateViewOptionsBar: enable hide/show Options bar                    */
/*                                                                         */
/***************************************************************************/

void      MainFrame::OnUpdateViewOptionsBar (CCmdUI* pCmdUI) {
 pCmdUI->SetCheck (optionsbar.GetStyle () & WS_VISIBLE);
}

/***************************************************************************/
/*                                                                         */
/* OnViewEditBar: hide/show the Edit Bar                                   */
/*                                                                         */
/***************************************************************************/

void      MainFrame::OnViewEditBar () {
 if ((editbar.GetStyle () & WS_VISIBLE) != 0)
		ShowControlBar (&editbar, FALSE, FALSE);
 else ShowControlBar (&editbar, TRUE, FALSE);
 editbar.Invalidate ();
}

/***************************************************************************/
/*                                                                         */
/* OnUpdateViewEditBar: enable hide/show EditBar                           */
/*                                                                         */
/***************************************************************************/

void      MainFrame::OnUpdateViewEditBar (CCmdUI* pCmdUI) {
 pCmdUI->SetCheck (editbar.GetStyle () & WS_VISIBLE);
}

/***************************************************************************/
/*                                                                         */
/* OnViewPaletteBox: hide/show the Palette Box                             */
/*                                                                         */
/***************************************************************************/

void      MainFrame::OnViewPaletteBox() {
 if ((palettebox.GetStyle () & WS_VISIBLE) != 0)
		ShowControlBar (&palettebox, FALSE, FALSE);
 else ShowControlBar (&palettebox, TRUE, FALSE);
 palettebox.Invalidate ();
}

/***************************************************************************/
/*                                                                         */
/* OnUpdateViewPaletteBox: enable hide/show Palette Box                    */
/*                                                                         */
/***************************************************************************/

void      MainFrame::OnUpdateViewPaletteBox (CCmdUI *pCmdUI) {
 pCmdUI->SetCheck (palettebox.GetStyle () & WS_VISIBLE);
}

#ifdef _DEBUG
void MainFrame::AssertValid() const { CFrameWnd::AssertValid(); }
void MainFrame::Dump(CDumpContext& dc) const { CFrameWnd::Dump(dc); }
#endif //_DEBUG
