#pragma once
#include "OptionsBar.h"       // access to the options controlbar
#include "MapControlBar.h"    // access to the map control bar
#include "MapLocalStatsBar.h" // access to the map local stats bar
#include "EditBar.h"          // access to the edit bar
#include "PaletteBox.h"       // access to the palette box

#define INITSCROLL  25      // initial scroll percentage
#define INITZOOM     2      // initial zoom factor
#define INITSCALE 6000      // initial map scale
#define INITLAT      0      // initial map position
#define INITLON      0

/***************************************************************************/
/*                                                                         */
/* MainFrame: Apps main frame controls the various tool bars               */
/*                                                                         */
/***************************************************************************/

class MainFrame : public CFrameWnd {
protected: // create from serialization only
	MainFrame();
	DECLARE_DYNCREATE(MainFrame)
                                       // control bar embedded members
	CStatusBar       m_wndStatusBar;      // main status bar
	CToolBar         m_wndToolBar;        // main tool bar
 MapLocalStatsBar maplocalstatsbar;    // map local stat bar
 MapControlBar    mapcontrolbar;       // map control bar
 OptionsBar       optionsbar;          // options bar
 EditBar          editbar;             // edit bar
 PaletteBox       palettebox;          // floating palette

public:
                   // helper function for docking
void               DockControlBarLeftOf (CToolBar* Bar,CToolBar* LeftOf);

                   // get access to various control bars for displaying
OptionsBar*        GetOptionsBar ();
MapControlBar*     GetMapControlBar ();
MapLocalStatsBar*  GetMapLocalStatsBar ();

void               DisplayNewCaption (char*);  // display a new title
void               DisplayOnStatusBar (char*); // display msg on statusbar
virtual BOOL       PreCreateWindow(CREATESTRUCT& cs);
virtual           ~MainFrame();

#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnViewToolbar();
	afx_msg void OnUpdateViewToolbar(CCmdUI* pCmdUI);
	afx_msg void OnViewStatusBar();
	afx_msg void OnUpdateViewStatusBar(CCmdUI* pCmdUI);
	afx_msg void OnViewMapControlBar();
	afx_msg void OnUpdateViewMapControlBar(CCmdUI* pCmdUI);
	afx_msg void OnViewMapLocalStatsBar();
	afx_msg void OnUpdateViewMapLocalStatsBar(CCmdUI* pCmdUI);
	afx_msg void OnViewOptionsBar();
	afx_msg void OnUpdateViewOptionsBar(CCmdUI* pCmdUI);
	afx_msg void OnClose();
	afx_msg void OnViewEditBar();
	afx_msg void OnUpdateViewEditBar(CCmdUI* pCmdUI);
	afx_msg void OnViewPaletteBox();
	afx_msg void OnUpdateViewPaletteBox(CCmdUI* pCmdUI);
 afx_msg void CmDummy ();
 afx_msg LRESULT FontChange (WPARAM, LPARAM); // respond to font changes
	DECLARE_MESSAGE_MAP()
};
