#include "stdafx.h"
#include "Pgm17b.h"
#include "Pgm17bForm.h"
#include "Random.h"
#include "Pgm17bDoc.h"

IMPLEMENT_DYNCREATE(Pgm17bForm, CFormView)

CCriticalSection Pgm17bForm::critSection;
CMutex           Pgm17bForm::getLunch;

// structure to pass needed data to the thread
struct CustomerParms {
 int               maxWaitToGetIntoStore;
 int               maxTimeSpentShopping;
 int               maxWaitTimeForLunch;
 int               maxTimeEatingLunch;
 int               maxTimeToWaitForSalesClerk;
 int               shopperId;
 DWORD             startTime;
 CSemaphore*       ptrGetIntoShop;
 CSemaphore*       ptrGetSalesClerk;
 CMutex*           ptrGetLunch;
 CPtrArray*        ptrarray;
 CCriticalSection* ptrcrit;
};

// thread prototype
UINT CustomerThread (LPVOID pParam);

Pgm17bForm::Pgm17bForm() : CFormView(Pgm17bForm::IDD), maxCustomers(30), openTime(1000),
      numberSalesClerks(2), maxDelayBetweenCustomerCreation(30), maxWaitToGetIntoStore(20),
      maxTimeSpentShopping(80), maxTimeToWaitForSalesClerk(25), maxWaitTimeForLunch(20),
      maxTimeEatingLunch(30) { }

Pgm17bForm::~Pgm17bForm() { }

void Pgm17bForm::DoDataExchange(CDataExchange* pDX) {
 CFormView::DoDataExchange(pDX);
 DDX_Control(pDX, IDC_LIST_RESULTS, list);
 DDX_Text(pDX, IDC_EDIT_MAXCUSTOMERS, maxCustomers);
 DDV_MinMaxInt (pDX, maxCustomers, 1, 50);
 DDX_Text(pDX, IDC_EDIT_MAXTIMEOPEN, openTime);
 DDV_MinMaxInt (pDX, openTime, 100, 10000);
 DDX_Text(pDX, IDC_EDIT_NUMCLERKS, numberSalesClerks);
 DDV_MinMaxInt (pDX, numberSalesClerks, 1, 5);
 DDX_Text(pDX, IDC_EDIT_MAX_DELAYTIME, maxDelayBetweenCustomerCreation);
 DDV_MinMaxInt (pDX, maxDelayBetweenCustomerCreation, 10, 1000);
 DDX_Text(pDX, IDC_EDIT_MAXWAITGETIN, maxWaitToGetIntoStore);
 DDV_MinMaxInt (pDX, maxWaitToGetIntoStore, 1, 100);
 DDX_Text(pDX, IDC_EDIT_MAXTIMESHOP, maxTimeSpentShopping);
 DDV_MinMaxInt (pDX, maxTimeSpentShopping, 10, 1000);
 DDX_Text(pDX, IDC_EDIT_MAXWAITFORCLERK, maxTimeToWaitForSalesClerk);
 DDV_MinMaxInt (pDX, maxTimeToWaitForSalesClerk, 10, 50);
 DDX_Text(pDX, IDC_EDIT_MAXWAITLUNCH, maxWaitTimeForLunch);
 DDV_MinMaxInt (pDX, maxWaitTimeForLunch, 10, 80);
 DDX_Text(pDX, IDC_EDIT_MAXEAT_TIME, maxTimeEatingLunch);
 DDV_MinMaxInt (pDX, maxTimeEatingLunch, 15, 100);
}

BEGIN_MESSAGE_MAP(Pgm17bForm, CFormView)
 ON_BN_CLICKED(IDC_BUTTON_RUN, &Pgm17bForm::OnRunSimulation)
END_MESSAGE_MAP()

#ifdef _DEBUG
void Pgm17bForm::AssertValid() const { CFormView::AssertValid(); }
#ifndef _WIN32_WCE
void Pgm17bForm::Dump(CDumpContext& dc) const { CFormView::Dump(dc); }
#endif
#endif //_DEBUG

void Pgm17bForm::OnRunSimulation() {
 CWaitCursor w;
 UpdateData (TRUE);    // get new parameters
 list.ResetContent (); // empty listbox

 Pgm17bDoc* pDoc = (Pgm17bDoc*) GetDocument();
 pDoc->EmptyMsgs ();   // empty array of messages
 CPtrArray* ptrarray = &pDoc->msgs;
 
 // setup two semaphores
 CSemaphore getIntoShop (0, maxCustomers);
 CSemaphore getSalesClerk (numberSalesClerks, numberSalesClerks);

 Random random;

 DWORD closeTime;
 int numberCustomers = 1;
 int customer;

 DWORD startTime = GetTickCount(); // store the starting time
 closeTime = startTime + openTime; // calculate the closing time

 AddMessage ("Opening the shop for business", ptrarray, &critSection);
 AddMessage ("  Id  time", ptrarray, &critSection);
 // clear semaphore so shoppers can enter the shop
 ReleaseSemaphore (getIntoShop, maxCustomers, NULL);

 // add new customers until closing time
 while (GetTickCount() < closeTime) {
  // create the new thread's parameter instance
  CustomerParms* ptrparms = new CustomerParms;
  ptrparms->maxWaitToGetIntoStore = maxWaitToGetIntoStore;
  ptrparms->maxTimeSpentShopping = maxTimeSpentShopping;
  ptrparms->maxWaitTimeForLunch = maxWaitTimeForLunch;
  ptrparms->maxTimeEatingLunch = maxTimeEatingLunch;
  ptrparms->maxTimeToWaitForSalesClerk = maxTimeToWaitForSalesClerk;
  ptrparms->ptrGetIntoShop = &getIntoShop;
  ptrparms->ptrGetSalesClerk= &getSalesClerk;
  ptrparms->ptrGetLunch = &getLunch;
  ptrparms->ptrarray = ptrarray;
  ptrparms->ptrcrit = &critSection;
  ptrparms->shopperId = numberCustomers++;
  ptrparms->startTime = startTime;

  // create the new customer thread
  CWinThread* ptrthread = AfxBeginThread (CustomerThread, ptrparms);
  ptrthread->SetThreadPriority (THREAD_PRIORITY_LOWEST);

  // wait until another customer comes to the shop
  Sleep (random.GetRandom (maxDelayBetweenCustomerCreation));
 }

 // loop done, so it is time to close the shop
 char msg[100];
 wsprintf (msg, "        %4lu: Wait for remaining shoppers to finish and leave",
           GetTickCount() - startTime);
 AddMessage (msg, ptrarray, &critSection);

 // wait for each running thread to terminate, based on semaphore getIntoShop
 for (customer=1; customer <= maxCustomers; customer++) {
  WaitForSingleObject (getIntoShop, INFINITE);
  char m[100];
  wsprintf (m, "Ending countdown of customers that could still be in the store: %04d",
            customer);
  AddMessage (m, ptrarray, &critSection);
 }
 wsprintf (msg, "       %4lu: Shop has closed for the day", GetTickCount() - startTime);
 AddMessage (msg, ptrarray, &critSection);
 for (long i=0; i<ptrarray->GetSize(); i++) {
  CString* ptrs = (CString*) ptrarray->GetAt (i);
  list.AddString (*ptrs);
 }
}

void AddMessage (const char* m, CPtrArray* ptrarray, CCriticalSection* ptrc) {
 CString* ptrs = new CString;
 *ptrs = m;
 // two choices to use the critical section
 // one is to lock and unlock directly
 // ptrc->Lock ();
 // ptrarray->Add (ptrs);
 // ptrc->Unlock ();
 // the other is to use CSingleLock
 CSingleLock singleLock (ptrc);
 singleLock.Lock(INFINITE);   // attempt to lock the shared resource
 if (singleLock.IsLocked()) { // if the resource has been locked
  ptrarray->Add (ptrs);       // use it - here add a string to the array
  singleLock.Unlock();        // and unlock it
 }
}

UINT CustomerThread (LPVOID parm) {
 CustomerParms* ptrp = (CustomerParms*) parm;
 CPtrArray* ptrarray = ptrp->ptrarray;
 // seed the random number generator differently than other threads
 Random random ((unsigned) (time (0) + ptrp->shopperId));
 DWORD startTime = ptrp->startTime;
 DWORD result;
 int duration;

 // wait till the shopper can enter the supermarket.
 duration = random.GetRandom (ptrp->maxWaitToGetIntoStore);
 char m[100];
 wsprintf (m, "%4d: %4lu: customer will wait this long to get inside: %d", ptrp->shopperId, GetTickCount() - startTime, duration);
 AddMessage (m, ptrarray, ptrp->ptrcrit);
 result = WaitForSingleObject (ptrp->ptrGetIntoShop, duration);
 if (result == WAIT_TIMEOUT) {
  wsprintf (m, "%4d: %4lu: shopper gave up trying to get in and left", ptrp->shopperId, GetTickCount() - startTime);
  AddMessage (m, ptrarray, ptrp->ptrcrit);
  delete ptrp;
  return 0;
 }
 // the customer has successfully entered the supermarket - first action, shop
 wsprintf (m, "%4d: %4lu: customer enters the shop and begins shopping", ptrp->shopperId, GetTickCount() - startTime);
 AddMessage (m, ptrarray, ptrp->ptrcrit);
 duration = random.GetRandom (ptrp->maxTimeSpentShopping);
 Sleep (duration);
 wsprintf (m, "%4d: %4lu: customer shopped this long: %d", ptrp->shopperId, GetTickCount() - startTime, duration);
 AddMessage (m, ptrarray, ptrp->ptrcrit);
 // customer done shopping, but has a one-in-two chance of getting lunch here
 if (random.GetRandom (2) == 1) {
  // customer decides to grab lunch
  duration = random.GetRandom (ptrp->maxWaitTimeForLunch);
  wsprintf (m, "%4d: %4lu: customer will wait this long for lunch: %d", ptrp->shopperId, GetTickCount() - startTime, duration);
  AddMessage (m, ptrarray, ptrp->ptrcrit);
  // two mutex choices here: use CSingleLock or a wait and release mutex
  CSingleLock l(ptrp->ptrGetLunch);
  l.Lock (duration);
  if (l.IsLocked()) {
   // result = WaitForSingleObject (ptrp->ptrGetLunch, duration);
   // if (result == 0) {
   // here customer has a waitress, so orders lunch
   wsprintf (m, "%4d: %4lu: customer had to wait this long for lunch", ptrp->shopperId, GetTickCount() - startTime);
   AddMessage (m, ptrarray, ptrp->ptrcrit);
   duration = random.GetRandom (ptrp->maxTimeEatingLunch);
   Sleep (duration);
   wsprintf (m, "%4d: %4lu: customer spent this time eating: %d", ptrp->shopperId, GetTickCount() - startTime, duration);
   AddMessage (m, ptrarray, ptrp->ptrcrit);
   // customer is done eating, time to head to the sales clerk
   // ReleaseMutex (ptrp->ptrGetLunch);
   l.Unlock ();
  }
  else {
   // customer gives up waiting for the waitress and leaves the lunch counter
   wsprintf (m, "%4d: %4lu: customer gave up waiting for lunch", ptrp->shopperId, GetTickCount() - startTime);
   AddMessage (m, ptrarray, ptrp->ptrcrit);
  }
 }
 else {
  wsprintf (m, "%4d: %4lu: customer chooses not to get any lunch", ptrp->shopperId, GetTickCount() - startTime);
  AddMessage (m, ptrarray, ptrp->ptrcrit);
 }
 // customer now has to wait for a sales clerk
 wsprintf (m, "%4d: %4lu: customer now infinitely waiting for sales clerk", ptrp->shopperId, GetTickCount() - startTime);
 AddMessage (m, ptrarray, ptrp->ptrcrit);
 WaitForSingleObject (ptrp->ptrGetSalesClerk, INFINITE);
 // handle the customer's purchases
 duration = random.GetRandom (ptrp->maxTimeToWaitForSalesClerk);
 Sleep (duration);
 wsprintf (m, "%4d: %4lu: customer has been handled by sales clerk - took this long: %d", ptrp->shopperId, GetTickCount() - startTime, duration);
 AddMessage (m, ptrarray, ptrp->ptrcrit);
 // mark customer done with this sales clerk
 ReleaseSemaphore (ptrp->ptrGetSalesClerk, 1, NULL);
 // customer now leaved the shop
 wsprintf (m, "%4d: %4lu: customer leaves the shop", ptrp->shopperId, GetTickCount() - startTime);
 AddMessage (m, ptrarray, ptrp->ptrcrit);
 ReleaseSemaphore (ptrp->ptrGetIntoShop, 1, NULL);
 delete ptrp;
 return 0;
}
