#include "StdAfx.h"
#include "FrameWin.h"
#include "resource.h"

/***************************************************************************/
/*                                                                         */
/* FrameWin Events Response Table                                          */
/*                                                                         */
/***************************************************************************/

BEGIN_MESSAGE_MAP(FrameWin, CFrameWnd)
 ON_WM_SIZE ()
 ON_WM_CREATE () 
 ON_WM_CLOSE ()
 ON_WM_DESTROY ()
 ON_WM_PAINT ()
 ON_COMMAND(CM_FONTCHANGE,       CmFontChange)
 ON_COMMAND(CM_EXIT,             CmExit)
 ON_COMMAND(CM_HELPABOUT,        CmHelpAbout)
END_MESSAGE_MAP()

/***************************************************************************/
/*                                                                         */
/* Framewin: Construct the frame window                                    */
/*                                                                         */
/***************************************************************************/

          FrameWin::FrameWin (CWnd* ptrparent, const char* title)
                  : CFrameWnd () {

 ptrlogfont   = NULL;                  
 ptrttlistbox = NULL;                

 Create (AfxRegisterWndClass (
               CS_VREDRAW | CS_HREDRAW,                // register window style
               AfxGetApp()->LoadStandardCursor (IDC_ARROW), // use arrow cursor
               (HBRUSH) GetStockObject (LTGRAY_BRUSH),      // use gray brush
               AfxGetApp()->LoadStandardIcon (IDI_APPLICATION)),// set min icon
         title,               // window caption
         WS_OVERLAPPEDWINDOW, // wndclass DWORD style
         rectDefault,         // set initial window position
         ptrparent,           // the parent window, here none
        "MAINMENU");          // assign the main menu
}

/***************************************************************************/
/*                                                                         */
/* OnCreate: enumerate fonts and place TrueType fonts in a ListBox         */
/*                                                                         */
/***************************************************************************/

int      FrameWin::OnCreate (LPCREATESTRUCT lpCS) {

 int retcd = CFrameWnd::OnCreate (lpCS); // make Windows interface element

 // build the TrueType ListBox
 ptrttlistbox = new CListBox ();
 ptrttlistbox-> Create (WS_CHILD | WS_VISIBLE | WS_VSCROLL | LBS_SORT,
                        CRect (0,0, 20, 300), this, IDC_TTLISTBOX);

 // determine whether running under Windows 95 or NT
 DWORD version = ::GetVersion ();
 BOOL  win95   = (BYTE) version >= 4; // TRUE for Win 95; FALSE for Win NT

 // construct the model LOGFONT to enumerate - use only DEFAULT_CHARSET
 LOGFONT lf;
 memset (&lf, 0, sizeof (LOGFONT));
 lf.lfCharSet = DEFAULT_CHARSET;

 static char   printer[80];
 char         *device, *driver, *output, *next;
 CDC          *ptrprinterDC = new CDC;

 // get access to a printer DC from the ini file installed printer
 GetProfileString ("windows", "device", "...", printer, 80);
 if ((device = strtok_s (printer, ",", &next)) != NULL &&
     (driver = strtok_s (NULL,    ",", &next)) != NULL &&
     (output = strtok_s (NULL,    ",", &next)) != NULL) {
  
  // a default printer exists, so get a DC for it
  ptrprinterDC->CreateDC (driver, device, output, NULL);

  // enumerate all fonts on either Win 95 or Win NT system
  if (win95) ::EnumFontFamiliesEx (ptrprinterDC->GetSafeHdc(), &lf, 
                         (FONTENUMPROC) EnumPrinterFonts, (LPARAM) this, NULL);
  else ::EnumFontFamilies (ptrprinterDC->GetSafeHdc(), NULL, 
                          (FONTENUMPROC) EnumPrinterFonts, (LPARAM) this);
  delete ptrprinterDC;
 }
 return retcd;
}

/***************************************************************************/
/*                                                                         */
/* EnumPrinterFonts: call back function to handle each font                */
/*                                                                         */
/***************************************************************************/

BOOL CALLBACK AFX_EXPORT  FrameWin::EnumPrinterFonts (ENUMLOGFONT *ptrlf, 
                                                NEWTEXTMETRICEX* /* ptrntm*/,
                                                int fonttype, LPVOID pthis) {
 // eliminate unwanted fonts, such as Turkish and MAC fonts
 if (ptrlf->elfLogFont.lfCharSet == DEFAULT_CHARSET ||
     ptrlf->elfLogFont.lfCharSet == ANSI_CHARSET ||
     ptrlf->elfLogFont.lfCharSet == SYMBOL_CHARSET ||
     ptrlf->elfLogFont.lfCharSet == OEM_CHARSET) {
  ((FrameWin*) pthis)->AddFontToList (ptrlf, fonttype);
 }
 return 1;
}

/***************************************************************************/
/*                                                                         */
/* AddFontToList: add this font to the correct list                        */
/*                                                                         */
/***************************************************************************/

void      FrameWin::AddFontToList (ENUMLOGFONT *ptrlf, int type) {

 // construct our copy of the enumerated font
 // add font to the corresponding array - note if it is TrueType, I ignore
 // other ORed possibilities - TrueType fonts are also added to the ListBox
 // for display purposes - there are usually too many of them to show on screen
 if (type & TRUETYPE_FONTTYPE) {
  ENUMLOGFONT *ptrelf = new ENUMLOGFONT;
  memcpy (ptrelf, ptrlf, sizeof (ENUMLOGFONT));
  truetype_fonts.Add (ptrelf);
  ptrttlistbox->AddString (ptrelf->elfLogFont.lfFaceName);
 }
 else if (type & RASTER_FONT) {
  ENUMLOGFONT *ptrelf = new ENUMLOGFONT;
  memcpy (ptrelf, ptrlf, sizeof (ENUMLOGFONT));
  raster_fonts.Add (ptrelf);
 }
 else if (type & DEVICE_FONT) {
  ENUMLOGFONT *ptrelf = new ENUMLOGFONT;
  memcpy (ptrelf, ptrlf, sizeof (ENUMLOGFONT));
  device_fonts.Add (ptrelf);
 }
 else if (type & VECTOR_FONT) {
  ENUMLOGFONT *ptrelf = new ENUMLOGFONT;
  memcpy (ptrelf, ptrlf, sizeof (ENUMLOGFONT));
  vector_fonts.Add (ptrelf);
 }
 else if (ptrlf->elfLogFont.lfCharSet == OEM_CHARSET) {
  ENUMLOGFONT *ptrelf = new ENUMLOGFONT;
  memcpy (ptrelf, ptrlf, sizeof (ENUMLOGFONT));
  oem_fonts.Add (ptrelf);
 }
 else {
  AfxMessageBox ("Not found type");
 }
}

/***************************************************************************/
/*                                                                         */
/* OnDestroy: remove listbox and the four font queues                      */
/*                                                                         */
/***************************************************************************/
  
void      FrameWin::OnDestroy () {

 CFrameWnd::OnDestroy();
 if (ptrttlistbox) delete ptrttlistbox;
 if (ptrlogfont) delete ptrlogfont; // delete any user font;
 int num, i;
 num = truetype_fonts.GetSize ();
 for (i=0; i<num; i++) delete (ENUMLOGFONT*) truetype_fonts.GetAt (i);
 truetype_fonts.RemoveAll ();
 num = raster_fonts.GetSize ();
 for (i=0; i<num; i++) delete (ENUMLOGFONT*) raster_fonts.GetAt (i);
 raster_fonts.RemoveAll ();
 num = device_fonts.GetSize ();
 for (i=0; i<num; i++) delete (ENUMLOGFONT*) device_fonts.GetAt (i);
 device_fonts.RemoveAll ();
 num = vector_fonts.GetSize ();
 for (i=0; i<num; i++) delete (ENUMLOGFONT*) vector_fonts.GetAt (i);
 vector_fonts.RemoveAll ();
 num = oem_fonts.GetSize ();///
 for (i=0; i<num; i++) delete (ENUMLOGFONT*) oem_fonts.GetAt (i);///
 oem_fonts.RemoveAll ();///
}

/***************************************************************************/
/*                                                                         */
/* OnPaint: display the four categories of enumerated fonts                */
/*                                                                         */
/***************************************************************************/

void      FrameWin::OnPaint () {

 CPaintDC dc (this);
 int num, i, j = 0;
 GetAvgCharDims (dc);
 dc.SetBkMode (TRANSPARENT);

 // note TrueType List Box is already handled - must show other three groups
 ENUMLOGFONT *ptrelf;

 // set width to just beyond the ListBox
 j = 220 / avg_char_width + 6;

 // display the device or printer supplied fonts
 dc.TextOut (avg_char_width * j, 0, "Printer Fonts");
 num = device_fonts.GetSize ();
 for (i=0; i<num; i++) {
  ptrelf = (ENUMLOGFONT*) device_fonts.GetAt (i);
  dc.TextOut (avg_char_width * j, avg_char_height * (i+2), ptrelf->elfLogFont.lfFaceName);
 }
 if (num == 0)
  dc.TextOut (avg_char_width * j, avg_char_height * (i+2), "none found");
 j += 20;

 // display the raster or screen only fonts
 dc.TextOut (avg_char_width * j, 0, "Screen Only Fonts");
 num = raster_fonts.GetSize ();
 for (i=0; i<num; i++) {
  ptrelf = (ENUMLOGFONT*) raster_fonts.GetAt (i);
  dc.TextOut (avg_char_width * j, avg_char_height * (i+2), ptrelf->elfLogFont.lfFaceName);
 }
 if (num == 0)
  dc.TextOut (avg_char_width * j, avg_char_height * (i+2), "none found");
 j += 20;

 // display the vector fonts
 dc.TextOut (avg_char_width * j, 0, "Vector Fonts");
 num = vector_fonts.GetSize ();
 for (i=0; i<num; i++) {
  ptrelf = (ENUMLOGFONT*) vector_fonts.GetAt (i);
  dc.TextOut (avg_char_width * j, avg_char_height * (i+2), ptrelf->elfLogFont.lfFaceName);
 }
 if (num == 0)
  dc.TextOut (avg_char_width * j, avg_char_height * (i+2), "none found");
 j += 20;

 // display the oem fonts
 dc.TextOut (avg_char_width * j, 0, "OEM Fonts");
 num = oem_fonts.GetSize ();
 for (i=0; i<num; i++) {
  ptrelf = (ENUMLOGFONT*) oem_fonts.GetAt (i);
  dc.TextOut (avg_char_width * j, avg_char_height * (i+2), ptrelf->elfLogFont.lfFaceName);
 }
 if (num == 0)
  dc.TextOut (avg_char_width * j, avg_char_height * (i+2), "none found");

}

/***************************************************************************/
/*                                                                         */
/* GetAvgCharDims: retrieve current font's average dimensions              */
/*                                                                         */
/***************************************************************************/

void      FrameWin::GetAvgCharDims (CDC &dc) {

 TEXTMETRIC tm;

 dc.GetTextMetrics (&tm);                  // get the font information

 // calculate average character parameters
 avg_char_width  = tm.tmAveCharWidth;
 avg_char_height = tm.tmHeight + tm.tmExternalLeading;
 avg_caps_width  = (tm.tmPitchAndFamily & 1 ? 3 : 2) * avg_char_width / 2;
}

/***************************************************************************/
/*                                                                         */
/* CmExit:  determine if the app can be shut down                          */
/*                                                                         */
/***************************************************************************/

void      FrameWin::CmExit () {

 PostMessage (WM_CLOSE);
}

/***************************************************************************/
/*                                                                         */
/* OnClose: determine if the app can be shut down                          */
/*                                                                         */
/***************************************************************************/

void      FrameWin::OnClose () {

 if (DisplayMsg (IDS_MSG_QUERY, IDS_MSG_QUIT, MB_YESNO | MB_ICONQUESTION) ==
    IDYES) CFrameWnd::OnClose ();
}

/***************************************************************************/
/*                                                                         */
/* OnSize: position the ListBox of TrueType fonts to left edge             */
/*                                                                         */
/***************************************************************************/

void      FrameWin::OnSize (UINT flags, int width, int height) { 

 CFrameWnd::OnSize    (flags, width, height);
 CRect rect;
 GetClientRect (rect);
 if (ptrttlistbox) ptrttlistbox->SetWindowPos (NULL, 0, 0, 220, rect.Height(), 
                                               SWP_NOZORDER);
}

/***************************************************************************/
/*                                                                         */
/* DisplayMsg: displays passed error message and return user response      */
/*                                                                         */
/***************************************************************************/

int       FrameWin::DisplayMsg (int id1, int id2, UINT flags) {

 CString msg1;
 CString msg2;
 msg1.LoadString (id1);
 msg2.LoadString (id2);
 return MessageBox (msg2, msg1, flags);
}

/***************************************************************************/
/*                                                                         */
/* CmFontChange: get users font request and implement it                   */
/*                                                                         */
/***************************************************************************/

void      FrameWin::CmFontChange () {
 
 CFontDialog  *ptrdlg;
 static char   printer[80];
 char         *device, *driver, *output, *next;
 CDC          *ptrprinterDC = new CDC;

 // get access to a printer DC from the ini file installed printer
 GetProfileString ("windows", "device", "...", printer, 80);
 if ((device = strtok_s (printer, ",", &next)) != NULL &&
     (driver = strtok_s (NULL,    ",", &next)) != NULL &&
     (output = strtok_s (NULL,    ",", &next)) != NULL) {
  // a default printer exists, so get a DC for it
  ptrprinterDC->CreateDC (driver, device, output, NULL);
  // if the user has already chosen a font, use that one as the initial font
  if (ptrlogfont) ptrdlg = new CFontDialog (ptrlogfont, CF_EFFECTS | 
                      CF_PRINTERFONTS | CF_FORCEFONTEXIST, ptrprinterDC, this);
  else ptrdlg = new CFontDialog (NULL, CF_EFFECTS |
                      CF_PRINTERFONTS | CF_FORCEFONTEXIST, ptrprinterDC, this);
 }
 else {
  // cannot find the default printer, so use printer only fonts instead, if
  // user has already chosen a font, use it as the initial font in the dialog
  if (ptrlogfont) ptrdlg = new CFontDialog (ptrlogfont, CF_EFFECTS | 
                          CF_PRINTERFONTS | CF_FORCEFONTEXIST, NULL, this);
  else ptrdlg = new CFontDialog (NULL, CF_EFFECTS |
                          CF_PRINTERFONTS | CF_FORCEFONTEXIST, NULL, this);
 }
 
 if (ptrdlg->DoModal () == IDOK) {     // get user font choice
  if (!ptrlogfont) ptrlogfont = new LOGFONT; // 1st time, create LOGFONT
  memcpy (ptrlogfont, &(ptrdlg->m_lf), sizeof (LOGFONT)); // copy user choice
  Invalidate();                        // on new dims, and reshow
  
 }
 delete ptrprinterDC;
 delete ptrdlg;
}

/***************************************************************************/
/*                                                                         */
/* CmHelpAbout: shows who wrote this app                                   */
/*                                                                         */
/***************************************************************************/

void      FrameWin::CmHelpAbout () {

 // informational modal dialog - create instance of CDialog and execute it
 CDialog aboutdlg ("IDD_ABOUT", this);
 aboutdlg.DoModal ();
}
