#include "stdafx.h"
#include "Pgm14b.h"
#include "MainFrm.h"
#include "LocateDlg.h"
#include "CheckBoxPP.h"
#include "ListBoxLinesPP.h"
#include "RadioBtnsPP.h"
#include "NamesPP.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

BEGIN_MESSAGE_MAP(MainFrame, CFrameWnd)
 ON_WM_SIZE ()
 ON_WM_PAINT ()
 ON_WM_CREATE () 
 ON_WM_CLOSE ()
 ON_WM_DESTROY ()
 ON_WM_MOUSEMOVE ()
 ON_COMMAND(CM_PROPERTYSHEET,CmPropertySheet)
 ON_COMMAND(CM_LOCATION_DLG, CmLocationDlg)
 ON_WM_HSCROLL()                           // trap slider/trackbar events
 ON_EN_CHANGE (IDC_SPINEDIT, OnSpinChange) // trap spin changes
END_MESSAGE_MAP()

/***************************************************************************/
/*                                                                         */
/* MainFrame: initialize members and calls Create                          */
/*                                                                         */
/***************************************************************************/

MainFrame::MainFrame (const char* title)
                    : CFrameWnd () {
 DWORD style = WS_OVERLAPPEDWINDOW;           // set basic window styles
 style &= !(WS_MINIMIZEBOX | WS_MAXIMIZEBOX); // remove max/min buttons
 style |= WS_SYSMENU | WS_THICKFRAME;

 CRect rect (10, 10, 600, 400);               // set init pos and size

 LoadAccelTable ("MAINMENU"); // install keybd accelerators
           
 Create ( AfxRegisterWndClass (
               CS_VREDRAW | CS_HREDRAW,       // register window style UINT
               NULL,                          // will load in our cursor next
               ::CreateSolidBrush (GetSysColor(COLOR_WINDOW)),
               AfxGetApp()->LoadIcon (IDI_TREE)),// set min icon
          title,               // window caption
          style,               // wndclass DWORD style
          rect,                // set initial window position
          0,                   // the parent window, here none
          "MAINMENU");         // assign the main menu

 ptrlocatedlg = NULL;  // set no TLocateDlg modeless cursor pos yet
 track_on = false;     // indicate mouse position tracking is not on yet

 // clear and set all transfer buffers

 memset (&real_names_db, 0, sizeof(TRANSFER_NAMES)); // initialize names

 which_cursor = RadioBtnsPP::Tree; // set our member to default cursor
 
 // load that cursor
 hcursor = LoadCursor (AfxGetApp()->m_hInstance, MAKEINTRESOURCE (IDC_TREE));
 SetClassLong (m_hWnd, GCL_HCURSOR, (LONG) hcursor);
 
 show_name    = FALSE;  // init check box xfer buf
 show_cursor  = TRUE;   // show cursor type but not names
 linenum      = 3;      // set default line for showing names on
}

/***************************************************************************/
/*                                                                         */
/* OnCreate: make toolbar and set up the window                            */
/*                                                                         */
/***************************************************************************/

int MainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct) {
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

 TEXTMETRIC  tm;

 // set the system font's characteristics in tm
 CClientDC *ptrdc = new CClientDC (this);  // acquire a DC
 ptrdc->GetTextMetrics (&tm);              // get the information
 delete ptrdc;                             // delete the dc

 // calculate average character parameters
 avg_char_width  = tm.tmAveCharWidth;
 avg_char_height = tm.tmHeight + tm.tmExternalLeading;
 avg_caps_width  = (tm.tmPitchAndFamily & 1 ? 3 : 2) * avg_char_width / 2;

 // the following illustrates how to change the background color AFTER
 // it has been set initially in the wndclass structure
 // you need to get the handle of the original brush so that it can be
 // deleted, if not, you will get a memory leak

 hbkgrndbrush = CreateSolidBrush (RGB (192, 192, 192)); // make new background brush

 // get old brush so we can delete it after installing new brush
 HBRUSH oldbrush = (HBRUSH) (GetClassLong (m_hWnd, GCL_HBRBACKGROUND));
 // install new brush
 SetClassLong (m_hWnd, GCL_HBRBACKGROUND, (LONG) hbkgrndbrush); 
 DeleteObject (oldbrush); // failure to delete old brush = memory leak
 
 // construct the slider/trackbar control
 CRect r (300,0,500,40);
 slider.Create (TBS_HORZ|TBS_AUTOTICKS|TBS_BOTH|TBS_ENABLESELRANGE|WS_CHILD|WS_VISIBLE|WS_BORDER,
                r, this, IDC_SLIDER);
 slider.SetRange (3, linetot + 2, TRUE); // force range into current possibilities
 slider.SetLineSize(1);                  // set amount one click/arrow key reports
 slider.SetPageSize(1);                  // set amount one PgXX reports
 slider.SetPos(3);                       // set initial position

 // create the edit control for the spin control's buddy window
 CRect er (300,70,345,100);
 editctl.Create (WS_CHILD|WS_VISIBLE, er, this, IDC_SPINEDIT);

 // create the spin or up/down control
 CRect sr (300,70,365,100);
 spinctl.Create (UDS_ARROWKEYS|UDS_SETBUDDYINT|UDS_ALIGNRIGHT|WS_CHILD|WS_BORDER|WS_VISIBLE,
                 sr, this, IDC_SPINCTL);
 spinctl.SetBuddy (&editctl);           // install the edit control as the buddy window
 spinctl.SetRange (3, linetot + 2);     // set the range of the spin control
 spinctl.SetPos (3);                    // set its initial position

	return 0;
}

/***************************************************************************/
/*                                                                         */
/* OnSpinChange: respond to spin control's edit box change of value        */
/*                                                                         */
/***************************************************************************/

void MainFrame::OnSpinChange () {

 CString msg;
 editctl.GetWindowText (msg);  // retrieve the current text
 linenum = atoi (msg);         // install in the line number
 // force line number into range
 if (linenum <3) linenum = 3;  
 if (linenum > linetot + 2) linenum = linetot + 2;

 slider.SetPos (linenum);      // set the corresponding slider position
 Invalidate();
}

/***************************************************************************/
/*                                                                         */
/* OnHScroll: respond to slider/trackbar events - similar to scrolling     */
/*                                                                         */
/***************************************************************************/

void MainFrame::OnHScroll (UINT type, UINT pos, CScrollBar*) {

 switch (type) {

  case TB_BOTTOM:        // slider to min value
   linenum = 3; break;
  case TB_LINEDOWN:      // right/down arrow
  case TB_PAGEDOWN:      // pgdn key or mouse click before slider
   linenum++; break;
  case TB_LINEUP:        // left/up arrow
  case TB_PAGEUP:        // pgup key or mouse click after slider
   linenum--; break;
  case TB_THUMBPOSITION: // slider moved using the mouse to pos
  case TB_THUMBTRACK:    // slider dragged using the mouse to pos
   linenum = pos; break;
  case TB_TOP:           // slider to max position
   linenum = linetot + 2; break;
 };

 // force into the current range
 if (linenum <3) linenum = 3;
 if (linenum > linetot + 2) linenum = linetot + 2;

 // update all controls to the new settings - both slider and spin and edit
 slider.SetPos (linenum);
 spinctl.SetPos (linenum);
 Invalidate();     // force a repaint to display at the new line number 
}

/***************************************************************************/
/*                                                                         */
/* OnDestroy:     delete inited items                                      */
/*                                                                         */
/***************************************************************************/

void MainFrame::OnDestroy () {
 if (ptrlocatedlg) delete ptrlocatedlg;     // only delete it if it was newed
 CFrameWnd::OnDestroy ();
}

/***************************************************************************/
/*                                                                         */
/* OnSize: acquire the current dimensions of the client window             */
/*                                                                         */
/***************************************************************************/

void MainFrame::OnSize (UINT a, int b, int c) {
 CFrameWnd::OnSize (a, b, c);
 CRect rect;
 GetClientRect (&rect);             // get the size of the client window
 height = rect.Height();            // calc and save current height
 width  = rect.Width();             // calc and save current width

 char msg[10];
 int  i, j;

 // now dynamically adjust the number of possible lines upon which the names
 // can be displayed:

 // retrieve printf control string "Line %02d"
 LoadString (AfxGetApp()->m_hInstance, IDS_LINEMSG, msg, sizeof(msg));

 // insert line choices into the list box lines that will be loaded into box
 j = height / avg_char_height - 5;  // -3 for the cursor msgs; -2 since 3 lines of names
 
 if (j<0) j=1;             // set at least line 3
 linetot = j;
 for (i=0; i<j; i++) wsprintf (line[i], msg, i+3);

 // set new slider and spin controls' range based on window size
 // and set their current position
 slider.SetRange (3, linetot + 2, TRUE);
 slider.SetPos (linenum);
 spinctl.SetRange (3, linetot + 2);
 spinctl.SetPos (linenum);

 Invalidate();             // force repainting of window
}

/***************************************************************************/
/*                                                                         */
/* OnMouseMove: display current mouse position                             */
/*                                                                         */
/***************************************************************************/

void MainFrame::OnMouseMove (UINT, CPoint pt) {
 // if tracking is on, force the modeless dialog to display new position
 if (track_on) ptrlocatedlg->ShowPos (pt);
}

/***************************************************************************/
/*                                                                         */
/* OnPaint: displays instructions and the current system color in use      */
/*                                                                         */
/***************************************************************************/

void MainFrame::OnPaint () {
 CPaintDC dc (this);
 CString msg;
 dc.SetBkMode (TRANSPARENT);

 if (show_cursor) {
  if (which_cursor == RadioBtnsPP::Tree) msg.LoadString (IDS_ISTREE);
  else msg.LoadString (IDS_ISARROW);
  dc.TextOut (avg_char_width, avg_char_height, msg);
 }
 if (show_name) {
  msg.LoadString (IDS_NAMEID);
  dc.TextOut (avg_char_width, avg_char_height*(linenum), msg);
  dc.TextOut (avg_char_width, avg_char_height*(linenum+1), real_names_db.first_name);
  dc.TextOut (avg_char_width, avg_char_height*(linenum+2), real_names_db.last_name);
 }
}

/***************************************************************************/
/*                                                                         */
/* OnClose: determine if the app can be shut down                          */
/*                                                                         */
/***************************************************************************/

void MainFrame::OnClose () {
 CString msgtitle;
 CString msgtext;
 msgtext.LoadString (IDS_MSG_QUIT);
 msgtitle.LoadString (IDS_MSG_QUERY);

 if (MessageBox (msgtext, msgtitle, MB_YESNO | MB_ICONQUESTION) == IDYES)
  CFrameWnd::OnClose ();
}

/***************************************************************************/
/*                                                                         */
/* CmLocationDlg: cursor location modeless dialog activation               */
/*                No data xfer - ShowPos updates the dlg controls          */
/*                                                                         */
/***************************************************************************/

void MainFrame::CmLocationDlg () {
 // note dialog already exists - simply Create a new instance and show it 
 if (!track_on) {                      // avoid multiple instances of the dlg
  ptrlocatedlg = new LocateDlg (this);
  POINT p;
  GetCursorPos (&p);                   // retrieve current mouse position
  CPoint pt (p);                       // convert to CPoint
  ptrlocatedlg->ShowPos (pt);          // display new position
  ptrlocatedlg->ShowWindow (SW_SHOW);  // make dialog visible
  track_on = TRUE;                     // indicate tracking is active
 }  
}

/***************************************************************************/
/*                                                                         */
/* CmPropertySheet: launch the tabbed dialog box                           */
/*              Use automatic DDX exchange method                          */
/*                                                                         */
/***************************************************************************/

void  MainFrame::CmPropertySheet () {
 RadioBtnsPP radio;
 radio.which_btn_is_on = which_cursor;       // xfer current settings into dlg
 CheckBoxPP ckbox;
 ckbox.ck_show_name   = show_name;    // transfer current settings into dlg
 ckbox.ck_show_cursor = show_cursor;
 ListBoxLinesPP listbox (this);
 NamesPP edit(&xfer_edit_names);          // construct dialog
 memcpy (&xfer_edit_names, &real_names_db, sizeof (TRANSFER_NAMES));
 CPropertySheet tabdlg (IDS_TAB_TITLE);
 tabdlg.AddPage (&edit);
 tabdlg.AddPage (&listbox);
 tabdlg.AddPage (&radio);
 tabdlg.AddPage (&ckbox);
 
 tabdlg.DoModal (); // create & execute dlg
 if (which_cursor != radio.which_btn_is_on) {
  which_cursor = radio.which_btn_is_on;
  if (which_cursor == RadioBtnsPP::Tree) {  // install correct cursor
   hcursor = LoadCursor (AfxGetApp()->m_hInstance, MAKEINTRESOURCE (IDC_TREE));
   SetClassLong (m_hWnd, GCL_HCURSOR, (LONG) hcursor);
  }
  else {
   hcursor = AfxGetApp()->LoadStandardCursor (IDC_ARROW), // use arrow cur
   SetClassLong (m_hWnd, GCL_HCURSOR, (LONG) hcursor);
  }
 }
 show_name   = ckbox.ck_show_name;   // update our members for next time
 show_cursor = ckbox.ck_show_cursor;
 memcpy (&real_names_db, &xfer_edit_names, sizeof (TRANSFER_NAMES));
 if (linenumnew >= 0)            // if OK, set our new line number
  linenum = linenumnew + 3;       // save new line number
 slider.SetPos (linenum);
 spinctl.SetPos (linenum);

 Invalidate (); // force paint to display msg of new cursor                                   
}

#ifdef _DEBUG
void MainFrame::AssertValid() const { CFrameWnd::AssertValid(); }
void MainFrame::Dump(CDumpContext& dc) const { CFrameWnd::Dump(dc); }
#endif //_DEBUG
